// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Toronto.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:25  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Globalization;
using Stochastix.Utilities;

namespace Stochastix.Business.Calendars
{
	/// <summary>
	/// Toronto calendar.
	/// </summary>
	/// <remarks>
	/// Holidays:
	/// <list type="table">
	/// <item><description>Saturdays</description></item>
	/// <item><description>Sundays</description></item>
	/// <item><description>New Year's Day, January 1st (possibly moved to Monday)</description></item>
	/// <item><description>Good Friday</description></item>
	/// <item><description>Easter Monday</description></item>
	/// <item><description>Victoria Day, The Monday on or preceding 24 May</description></item>
	/// <item><description>Canada Day, July 1st (possibly moved to Monday)</description></item>
	/// <item><description>Provincial Holiday, first Monday of August</description></item>
	/// <item><description>Labour Day, first Monday of September</description></item>
	/// <item><description>Thanksgiving Day, second Monday of October</description></item>
	/// <item><description>Remembrance Day, November 11th (possibly moved to Monday)</description></item>
	/// <item><description>Christmas, December 25th (possibly moved to Monday or Tuesday)</description></item>
	/// <item><description>Boxing Day, December 26th (possibly moved to Monday or Tuesday)</description></item>
	/// </list>
	/// </remarks>
	/// <seealso href="http://www.national-holidays.com/">National Holidays (website)</seealso>
	[ RcsId("$Id: Toronto.cs,v 1.1.1.1 2003/02/01 18:25:25 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	[ FactoryItem("Toronto") ]
	public sealed class Toronto : WesternCalendarBase, IBusinessCalendar
	{
		/// <summary>
		/// Parameterless COM constructor - use <see cref="Instance"/> member instead.
		/// </summary>
		/// <remarks>
		/// This constructor is provided for COM compatibility only.
		/// Use the static member <see cref="Instance"/> to get an object
		/// of this type.
		/// </remarks>
		[ Obsolete("Use Toronto.Instance in .NET applications.") ]
		public Toronto() 
			: base( "Toronto", CultureInfo.CreateSpecificCulture("en-CA"))
		{}

		#region FactoryItem pattern

		[ Obsolete() ] // just to ignore the CS0618 warning below
		static Toronto()
		{
			Instance = new Toronto();	// CS0618
		}

		/// <summary>
		/// A static instance of this type.
		/// </summary>
		static public readonly IBusinessCalendar Instance;

		#endregion

		/// <summary>
		/// Implementation of the public <see cref="IsBusinessDay"/> method.
		/// </summary>
		/// <remarks>
		/// This method must be implemented by concrete
		/// calendar implementations.
		/// </remarks>
		/// <param name="d">The day value, between 1 and 31.</param>
		/// <param name="m">The month, one of <see cref="Month"/>.</param>
		/// <param name="y">The year, between 1 and 9999.</param>
		/// <param name="w">A <see cref="DayOfWeek"/> enumerated constant that
		/// indicates the day of the week. 
		/// This property value ranges from zero, indicating Sunday,
		/// to six, indicating Saturday.</param>
		/// <param name="dd">The day of the year, between 1 and 366.</param>
		/// <param name="em">The day of Easter Monady in the year, between 1 and 366.</param>
		/// <returns><c>True</c> when the given day is a business day.</returns>
		override protected bool IsBusinessDay(
			int d, Month m, int y,
			DayOfWeek w, int dd, int em )
		{
			if ((w == DayOfWeek.Saturday || w == DayOfWeek.Sunday)
				// New Year's Day (possibly moved to Monday)
				|| ((d == 1 || ((d == 2 || d == 3) && w == DayOfWeek.Monday)) && m == Month.January)
				// Good Friday
				|| (dd == em-3)
				// Easter Monday
				|| (dd == em)
				// The Monday on or preceding 24 May (Victoria Day)
				|| (d > 17 && d <= 24 && w == DayOfWeek.Monday && m == Month.May)
				// July 1st, possibly moved to Monday (Canada Day)
				|| ((d == 1 || ((d == 2 || d == 3) && w == DayOfWeek.Monday)) &&  m == Month.July)
				// first Monday of August (Provincial Holiday)
				|| (d <= 7 && w == DayOfWeek.Monday && m == Month.August)
				// first Monday of September (Labor Day)
				|| (d <= 7 && w == DayOfWeek.Monday && m == Month.September)
				// second Monday of October (Thanksgiving Day)
				|| (d > 7 && d <= 14 && w == DayOfWeek.Monday && m == Month.October)
				// November 11th (Remembrance Day, possibly moved to Monday)
				|| ((d == 11 || (d == 12 && w == DayOfWeek.Monday)) && m == Month.November)
				// Christmas (possibly moved to Monday or Tuesday)
                || ((d == 25 || (d == 27 && (w == DayOfWeek.Monday || w == DayOfWeek.Tuesday))) && m == Month.December)
                // Boxing Day (possibly moved to Monday or Tuesday)
                || ((d == 26 || (d == 28 && (w == DayOfWeek.Monday || w == DayOfWeek.Tuesday))) && m == Month.December)
			)
				return false;
			return true;
		}
	}
}
