// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Period.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:24  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Business.Resources;

namespace Stochastix.Business
{
	/// <summary>
	/// Time period described by a number of a given time unit.
	/// </summary>
	[ RcsId("$Id: Period.cs,v 1.1.1.1 2003/02/01 18:25:24 jens Exp $") ]
	[ Description("Time period described by a number of a given time unit.") ]
	public struct Period // TODO : IComparable
	{
		/// <summary>
		/// Length of this Period measured in <see cref="TimeUnit"/>s.
		/// </summary>
		public int Length;

		/// <summary>
		/// <see cref="TimeUnit"/>s used for this Period.
		/// </summary>
		public TimeUnit Units;

		/// <summary>
		/// Construct a Period from the given length and time unit.
		/// </summary>
		/// <param name="length">Length of the Period measured in units.</param>
		/// <param name="units"><see cref="TimeUnit"/>s to be used for this Period.</param>
		public Period(int length, TimeUnit units) 
		{
			this.Length = length;
			this.Units = units;
		}

		/// <summary>
		/// Fixing frequency in 1/years.
		/// </summary>
		/// <value>The fixing frequency in times/year</value>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the period is not one of 1m, 2m, 3m, 4m, 6m, 12m and 1y.
		/// </exception>
		public int Frequency
		{
			get 
			{
				switch (Units) 
				{
					case TimeUnit.Months:
						if( 12 % Length != 0 ) break;
						return 12 / Length;
					case TimeUnit.Years:
						if( Length != 1 ) break;
						return Length;
					default:
						break;
				}
				throw new InvalidOperationException( Strings.Get(GetType(), "PrdFreq" ) );
			}
		}

		/// <summary>
		/// Returns a <see cref="String"/> that represents the current Object.
		/// </summary>
		/// <remarks>
		/// <para>
		/// This method returns a human-readable string that is culture-sensitive.
		/// </para>
		/// <para>
		/// The default implementation returns the fully qualified name of the type
		/// of the Object. This method can be overridden in a derived
		/// class to return values that are meaningful for that type. 
		/// </para>
		/// </remarks>
		/// <returns>
		/// A <see cref="String"/> that represents the current Object.
		/// </returns>
		[ Description("A stringified representation of this object.") ]
		[ DispId(0) ]	// DISPID_VALUE
		override public String ToString() 
		{
			return Length.ToString() 
				+ Units.ToString().Substring(0,1).ToLower();
		}

		/// <summary>
		/// Parse a <see cref="String"/> argument into the corresponding
		/// <see cref="Period"/>.
		/// </summary>
		/// <remarks>
		/// Accepts any combination of an integer number and one character
		/// out of <c>DdWwMmYy</c>.
		/// </remarks>
		/// <example>
		/// Accepted inputs are eg. "3m", "10y", "1w".
		/// </example>
		/// <param name="periodString">A <see cref="String"/> to be parsed.</param>
		/// <returns></returns>
		static public Period Parse(string periodString) 
		{
			periodString = periodString.Trim();
			if( periodString.Length<2 )
				throw new ArgumentException(
					Strings.Get(typeof(Period), "PrdParseLen"), "periodString");
			int iPos = periodString.IndexOfAny("DdWwMmYy".ToCharArray());
			if (iPos != periodString.Length-1)
                throw new ArgumentException(
					Strings.Get(typeof(Period), "PrdParseUnit"), "periodString");

			TimeUnit units = TimeUnit.Days;
			switch(periodString[iPos])
			{
				case 'd':
				case 'D':
					units = TimeUnit.Days;
					break;
				case 'w':
				case 'W':
					units = TimeUnit.Weeks;
					break;
				case 'm':
				case 'M':
					units = TimeUnit.Months;
					break;
				case 'y':
				case 'Y':
					units = TimeUnit.Years;
					break;
			}
			return new Period(
				int.Parse( periodString.Substring(0, iPos)), units );
		}

		/// <summary>
		/// Returns a <see cref="DateTime"/> that is the specified <see cref="Period"/> away 
		/// from the specified <see cref="DateTime"/>. Ignores business days,
		/// rolling conventions and local Calendar settings.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to which to add the <see cref="Period"/>.</param>
		/// <param name="p">The <see cref="Period"/> to add.</param>
		/// <returns>The <see cref="DateTime"/> that results from adding the specified time units.</returns>
		public static DateTime operator+(DateTime date, Period p)
		{
			switch(p.Units)       
			{         
				case TimeUnit.Days:
					return date.AddDays(p.Length);
				case TimeUnit.Weeks:
					// This may not be correct for some calendars
					// where a week has not always 7 days.
					// Calendar.AddWeeks() is the right way to go
					return date.AddDays(p.Length * 7);
				case TimeUnit.Months:
					// dito with months resp. Calendar.AddMonths()
					return date.AddMonths(p.Length);
				case TimeUnit.Years:
					return date.AddYears(p.Length);
				default:            
					throw new ArgumentOutOfRangeException(
						"p", p.Units, Strings.Get(typeof(Period), "PrdUnit") );
			}
		}

		/// <summary>
		/// Returns a <see cref="DateTime"/> that is the specified <see cref="Period"/> before 
		/// the specified <see cref="DateTime"/>. Ignores business days,
		/// rolling conventions and local Calendar settings.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> from which to subtract the <see cref="Period"/>.</param>
		/// <param name="p">The <see cref="Period"/> to subtract.</param>
		/// <returns>The <see cref="DateTime"/> that results from subtracting the specified time units.</returns>
		public static DateTime operator-(DateTime date, Period p)
		{
			return date + new Period(-p.Length, p.Units);
		}
	}
}
