// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: WesternCalendarBase.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:24  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/31 18:49:39  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Globalization;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Diagnostics;
using Stochastix.Utilities;

namespace Stochastix.Business.Calendars
{
	/// <summary>
	/// Partial implementation providing the means of
	/// determining the Easter Monday for a given year.
	/// </summary>
	[ RcsId("$Id: WesternCalendarBase.cs,v 1.1.1.1 2003/02/01 18:25:24 jens Exp $") ]
	[ ComVisible(false) ]
	public abstract class WesternCalendarBase : CalendarBase
	{
		/// <summary>
		/// Construct a new BusinessCalendar based on a localized 
		/// culture and its default calendar.
		/// </summary>
		/// <param name="culture">The associated <see cref="System.Globalization.CultureInfo"/>.</param>
		/// <param name="name">The name of this business calendar.</param>
		protected WesternCalendarBase(String name, CultureInfo culture) 
			: base(name, culture, culture.Calendar) 
		{}
		
		#region Business days

		/// <summary>
		/// Returns <c>true</c> iff the date is a business day for the given market.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to check.</param>
		/// <returns><c>true</c> iff the date is a business day for the given market.</returns>
		override public bool IsBusinessDay(DateTime date) 
		{
			return IsBusinessDay( 
				date.Day, (Month)date.Month, date.Year,
				date.DayOfWeek, date.DayOfYear, EasterMonday(date.Year)
				);
		}

		/// <summary>
		/// Implementation of the public <see cref="IsBusinessDay"/> method.
		/// </summary>
		/// <remarks>
		/// This method must be implemented by concrete calendar implementations.
		/// </remarks>
		/// <param name="day">The day value, between 1 and 31.</param>
		/// <param name="month">The month, one of <see cref="Month"/>.</param>
		/// <param name="year">The year, between 1 and 9999.</param>
		/// <param name="w">A <see cref="DayOfWeek"/> enumerated constant that
		/// indicates the day of the week. This property value ranges from zero, 
		/// indicating Sunday, to six, indicating Saturday.</param>
		/// <param name="dayOfYear">The day of the year, between 1 and 366.</param>
		/// <param name="easterMonday">The day of Easter Monday in the year, between 1 and 366.</param>
		/// <returns><c>True</c> when the given day is a business day.</returns>
		abstract protected bool IsBusinessDay(int day, Month month, int year,
			DayOfWeek w, int dayOfYear, int easterMonday);

		#endregion

		#region Easter monday calculations

		/// <summary>
		/// Determine Easter Monday for a given year.
		/// </summary>
		/// <param name="year">The year we are interested in.</param>
		/// <returns>Easter monday expressed relative to first day the of year.</returns>
		protected static int EasterMonday(int year) 
		{
			return easterMonday[year-1900];
		}

		private static byte[] easterMonday = { 
			107,  98,  90, 103,  95, 114, 106,  91, 111, 102,   // 1900-1909
			87, 107,  99,  83, 103,  95, 115,  99,  91, 111,   // 1910-1919
			96,  87, 107,  92, 112, 103,  95, 108, 100,  91,   // 1920-1929
			111,  96,  88, 107,  92, 112, 104,  88, 108, 100,   // 1930-1939
			85, 104,  96, 116, 101,  92, 112,  97,  89, 108,   // 1940-1949
			100,  85, 105,  96, 109, 101,  93, 112,  97,  89,   // 1950-1959
			109,  93, 113, 105,  90, 109, 101,  86, 106,  97,   // 1960-1969
			89, 102,  94, 113, 105,  90, 110, 101,  86, 106,   // 1970-1979
			98, 110, 102,  94, 114,  98,  90, 110,  95,  86,   // 1980-1989
			106,  91, 111, 102,  94, 107,  99,  90, 103,  95,   // 1990-1999
			115, 106,  91, 111, 103,  87, 107,  99,  84, 103,   // 2000-2009
			95, 115, 100,  91, 111,  96,  88, 107,  92, 112,   // 2010-2019
			104,  95, 108, 100,  92, 111,  96,  88, 108,  92,   // 2020-2029
			112, 104,  89, 108, 100,  85, 105,  96, 116, 101,   // 2030-2039
			93, 112,  97,  89, 109, 100,  85, 105,  97, 109,   // 2040-2049
			101,  93, 113,  97,  89, 109,  94, 113, 105,  90,   // 2050-2059
			110, 101,  86, 106,  98,  89, 102,  94, 114, 105,   // 2060-2069
			90, 110, 102,  86, 106,  98, 111, 102,  94, 107,   // 2070-2079
			99,  90, 110,  95,  87, 106,  91, 111, 103,  94,   // 2080-2089
			107,  99,  91, 103,  95, 115, 107,  91, 111, 103    // 2090-2099
		};

		#endregion

	}
}