// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: InvCumulativeNormalDistribution.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:43:00  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:35:51  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Distributions
{
	/// <summary>
	/// Inverse cumulative normal distribution class.
	/// </summary>
	/// <remarks>
	/// Given x between zero and one as the integral value of a gaussian 
	/// normal distribution this class provides the value y such that
	/// formula here ...
	/// 
	/// It uses Beasly and Springer approximation, with an improved
	/// approximation for the tails. See Boris Moro, "The Full Monte", 
	/// 1995, Risk Magazine. This class can also be used to generate a 
	/// gaussian normal distribution from a uniform distribution.
	/// 
	/// This is especially useful when a gaussian normal distribution
	/// is generated from a quasi-random uniform distribution: in this
	/// case the traditional Box-Muller approach and its variants
	/// would not preserve the sequence's low-discrepancy.
	/// </remarks>
	[ RcsId("$Id: InvCumulativeNormalDistribution.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(false) ]
	public class InvCumulativeNormalDistribution  : IUnaryFunction
	{
		public InvCumulativeNormalDistribution() : this(0.0, 1.0)
		{}
		public InvCumulativeNormalDistribution(double average) 
			: this(average, 1.0)
		{}
		public InvCumulativeNormalDistribution(double average,
			double sigma)
		{
			if( sigma <= 0.0 )
				throw new ArgumentOutOfRangeException( "sigma", sigma,
					Strings.Format(GetType(), "DistSigmaGr0", sigma) );
			this.average = average;
			this.sigma = sigma;
		}

		private double average, sigma;

		// IUnaryFunction
		public double Value(double x)
		{
			if( x <= 0.0 || x >= 1.0) 
				throw new ArgumentOutOfRangeException( "x", x,
					Strings.Format(GetType(), "DistInvX", x) );
			return average + Function(x)*sigma;
		}

		/// <summary>
		/// A lightweight static delegate for average=0.0, sigma=1.0.
 		/// Note: We do NOT check for x in ]0,1[ here!
		/// You can also instatiate this class and then use
		/// instance.Value as well.
		/// </summary>
		public static double Function(double x)
		{
			double result;
			double temp = x - 0.5;
			if (Math.Abs(temp) < 0.42) 
			{
				// Beasley and Springer, 1977
				result = temp*temp;
				return temp * (((a3*result+a2)*result+a1)*result+a0) /
					((((b3*result+b2)*result+b1)*result+b0)*result+1.0);
			} 
			else 
			{
				// improved approximation for the tail (Moro 1995)
				if (x<0.5)
					result = x;
				else
					result = 1.0-x;
				result = Math.Log(-Math.Log(result));
				result = c0 + result*(c1+result*(c2+result*(c3+result*
					(c4+result*(c5+result*(c6+result*(c7+result*c8)))))));
				if (x<0.5)
					return -result;
				else
					return result;
			}
		}


		private const double a0 =  2.50662823884;
		private const double a1 = -18.61500062529;
		private const double a2 =  41.39119773534;
		private const double a3 = -25.44106049637;

		private const double b0 =  -8.47351093090;
		private const double b1 =  23.08336743743;
		private const double b2 = -21.06224101826;
		private const double b3 =   3.13082909833;

		private const double c0 = 0.3374754822726147;
		private const double c1 = 0.9761690190917186;
		private const double c2 = 0.1607979714918209;
		private const double c3 = 0.0276438810333863;
		private const double c4 = 0.0038405729373609;
		private const double c5 = 0.0003951896511919;
		private const double c6 = 0.0000321767881768;
		private const double c7 = 0.0000002888167364;
		private const double c8 = 0.0000003960315187;
	}
}
