// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SegmentIntegral.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:43:33  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:35:28  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics
{
	/// <summary>
	/// Integral of a one-dimensional function.
	/// </summary>
	/// <latex>
	/// Given a number \f$ N \f$ of intervals, the integral of
	/// a function \f$ f \f$ between \f$ a \f$ and \f$ b \f$ is 
	/// calculated by means of the trapezoid formula
	/// \f[
	/// 	\int_{a}^{b} f \mathrm{d}x = 
	/// 	\frac{1}{2} f(x_{0}) + f(x_{1}) + f(x_{2}) + \dots 
	/// 	+ f(x_{N-1}) + \frac{1}{2} f(x_{N})
	/// \f]
	/// where \f$ x_0 = a \f$, \f$ x_N = b \f$, and 
	/// \f$ x_i = a+i \Delta x \f$ with \f$ \Delta x = (b-a)/N \f$.
	/// </latex>
	[ RcsId("$Id: SegmentIntegral.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(false) ]
	public class SegmentIntegral
	{
		/// <summary>
		/// Initialize a new SegmentIntegral with the given intervals.
		/// </summary>
		/// <param name="intervals"></param>
		public SegmentIntegral(int intervals)
		{
			if( intervals <= 0 ) 
				// At least 1 interval needed, {0} given.
				throw new ArgumentOutOfRangeException( "intervals", intervals,
					Strings.Format(GetType(), "SegmInter", intervals) );
			this.intervals = intervals;
		}

		private int intervals;

		/// <summary>
		/// Computes the integral value of the given one-dimensional function.
		/// </summary>
		/// <param name="f"></param>
		/// <param name="a"></param>
		/// <param name="b"></param>
		/// <returns></returns>
		public double Value(UnaryFunction f, double a, double b)
		{
			if( a >= b) 
				// To compute an integral on [a,b] it must be a<b
				throw new ArgumentException( Strings.Format( 
					GetType(), "SegmParams", a, b) );

			double dx  = (b-a) / intervals;
			double sum = 0.5 * ( f(a)+f(b) );
			double end = b - 0.5*dx;
			for (double x = a+dx; x < end; x += dx)
				sum += f(x);
			return sum*dx;
		}
	}
}
