// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BilinearInterpolation.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics.Interpolations
{
	/// <summary>
	/// Bilinear interpolation between discrete points.
	/// </summary>
	[ RcsId("$Id: BilinearInterpolation.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	public class BilinearInterpolation : Interpolation2D
	{

		/// <summary>
		/// Default constructor for this class.
		/// </summary>
		/// <overloads>
		/// Initialize a new bilinear interpolation.
		/// </overloads>
		/// <remarks>
		/// You need to <see cref="Initialize"/> a class constructed by the
		/// default constructor.
		/// </remarks>
		public BilinearInterpolation() : base()
		{}

		/// <summary>
		/// Initialize a new bilinear interpolation.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x and y must have been sorted.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <param name="z"></param>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists and matrix do not have the same size
		/// or do not provide enough points to interpolate.
		/// </exception>
		public BilinearInterpolation(IList x, IList y, Matrix z) : base(x, y, z)
		{}

		/// <summary>
		/// Initialize a class constructed by the default constructor.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x and y must have been sorted.
		/// </remarks>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists and matrix do not have the
		/// same size or do not provide enough points to interpolate.
		/// </exception>
		public override void Initialize( IList x, IList y, Matrix z)
		{
			base.Initialize(x, y, z);
			this.z = z;
		}

		Matrix z = null;


		/// <summary>
		/// Interpolated value.
		/// </summary>
		/// <remarks>
		/// This method must be overridden to provide an implementation
		/// of the actual interpolation.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <param name="allowExtrapolation"></param>
		/// <returns>The interpolated value.</returns>
		/// <exception cref="ArgumentOutOfRangeException">
		/// Thrown when extrapolation has not been allowed and the passed value
		/// is outside the allowed range.
		/// </exception>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the class was not properly initialized.
		/// </exception>
		public override double Value( double x, double y, bool allowExtrapolation )
		{
			int[] xy = Locate(x, y, allowExtrapolation);
			int ix = xy[0], iy = xy[1];

			double z1=z[iy,ix];
			double z2=z[iy,ix+1];
			double z3=z[iy+1,ix];
			double z4=z[iy+1,ix+1];

			double t=(x-this.x[ix])/(this.x[ix+1]-this.x[ix]);
			double u=(y-this.y[iy])/(this.y[iy+1]-this.y[iy]);

			return (1.0-t) * (1.0-u) * z1+
			          t    * (1.0-u) * z2+
			       (1.0-t) *    u    * z3+
			          t    *    u    * z4;
		}

	}
}
