// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Interpolation.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics.Interpolations
{
	/// <summary>
	/// Abstract base class for 1-D interpolations.
	/// </summary>
	/// <remarks>
	/// Classes derived from this class will override <see cref="Value"/> to
	/// provide interpolated values from two sequences of equal length,
	/// representing discretized values of a variable and a function of
	/// the former, respectively.
	/// </remarks>
	[ RcsId("$Id: Interpolation.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	public abstract class Interpolation 
	{

		/// <summary>
		/// Default constructor for this class.
		/// </summary>
		/// <overloads>
		/// Initialize a new Interpolation.
		/// </overloads>
		/// <remarks>
		/// You need to <see cref="Initialize"/> a class constructed by the
		/// default constructor.
		/// </remarks>
		protected Interpolation()
		{}

		/// <summary>
		/// Initialize a new Interpolation.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x must have been sorted.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists do not have the same size
		/// or do not provide enough points to interpolate.
		/// </exception>
		public Interpolation(IList x, IList y)
		{
			Initialize(x,y);
		}

		/// <summary>
		/// Initialize a class constructed by the default constructor.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x must have been sorted.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists do not have the same size
		/// or do not provide enough points to interpolate.
		/// </exception>
		public virtual void Initialize( IList x, IList y)
		{
			if( x.Count < 2)
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: not enough points to interpolate" ));
			if( x.Count != y.Count )
				throw new ArgumentException( Strings.Get( GetType(), 
					"TODO: lists must be of equivalent size." ) );

			this.x = new double[x.Count];
			x.CopyTo(this.x,0);
		}

		protected double[] x = null;

		/// <summary>
		/// Interpolated value.
		/// </summary>
		/// <remarks>
		/// This method must be overridden to provide an implementation
		/// of the actual interpolation.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="allowExtrapolation"></param>
		/// <returns>The interpolated value.</returns>
		/// <exception cref="ArgumentOutOfRangeException">
		/// Thrown when extrapolation has not been allowed and the passed value
		/// is outside the allowed range.
		/// </exception>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the class was not properly initialized.
		/// </exception>
		public abstract double Value( double x, bool allowExtrapolation );


		/// <summary>
		/// Locate index to be used for interpolation.
		/// </summary>
		/// <param name="x"></param>
		/// <param name="allowExtrapolation"></param>
		/// <returns></returns>
		/// <exception cref="ArgumentOutOfRangeException">
		/// Thrown when extrapolation has not been allowed and the passed value
		/// is outside the allowed range.
		/// </exception>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the class was not properly initialized.
		/// </exception>
		protected int Locate(double x, bool allowExtrapolation)
		{
			// check late initialization
			if( this.x == null) 
				throw new InvalidOperationException( Strings.Get( GetType(),
					"TODO: Interpolation has not been initialized." ) );

			if (x < this.x[0]) 
			{
				if( allowExtrapolation ) 
					return 0;
				else
					throw new ArgumentOutOfRangeException( "x", x, Strings.Get( GetType(),
						"TODO: Value outside range and extrapolation disabled." ) );

			} 
			else if (x > this.x[this.x.Length-1]) 
			{
				if( allowExtrapolation ) 
					return this.x.Length-2;
				else
					throw new ArgumentOutOfRangeException( "x", x, Strings.Get( GetType(),
						"TODO: Value outside range and extrapolation disabled." ) );
			} 
			else 
			{
				int pos = Array.BinarySearch(this.x, 0, this.x.Length-1, 
					x + Double.Epsilon);
				if( pos < 0) pos = ~pos; // next larger object
				return pos - 1;
			}
		}



	}
}
