// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BLAS.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:11  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:47:16  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:35:01  stochastix
 *  moved to LA namespace
 *
 *  Revision 1.4  2002/08/20 16:42:54  stochastix
 *  Added some BLAS extensions;
 *  moved native lib to DotNum.Native.dll;
 *  killed bugs involving "ref" types
 *
 *  Revision 1.3  2002/08/19 15:36:13  stochastix
 *  Changed to cdecl conventions (was: stdcall)
 *  to ease integration of other libraries (eg. ATLAS).
 *
 *  Revision 1.2  2002/08/15 18:12:22  stochastix
 *  Change GetVersionXXX names
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:09  stochastix
 *  Initial revision
 *
 *  Revision 1.3  2002/08/09 18:11:06  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Diagnostics;
using System.Text;
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.LA
{

	/// <summary>
	/// Interface to optimized BLAS library.
	/// </summary>
	[ RcsId("$Id: BLAS.cs,v 1.1.1.1 2003/02/01 18:42:11 jens Exp $") ] 
	[ ComVisible(false) ]
	public sealed class BLAS
	{
		public static readonly string Version;

		#region Static Constructor and availability check

		private const string IMPORT_DLL = "Stochastix.Numerics.Native.dll";
		private const string DLL_Version = "GetVersionBLAS";


		static BLAS() 
		{
			StringBuilder buf = new StringBuilder(120);
			try 
			{
				GetVersion(buf, buf.Capacity);
			} 
			catch (Exception e) 
			{
				Trace.WriteLine("BLAS not loaded: " + e.Message);
				return;
			}
			Version = buf.ToString().Trim();
			Trace.WriteLine("BLAS loaded '" + Version + "'");
		}

		[ DllImport(IMPORT_DLL, EntryPoint=DLL_Version) ]
		private static extern void GetVersion(StringBuilder buf, int bufSize);

		#endregion

		#region BLAS Level 1

		public static void scal(int n, double alpha, 
			ref double x, int incx)
		{
			// n>50 is a rough estimation of the interop overhead
			if( n>50 && Version != null) 
				DSCAL(ref n, ref alpha, ref x, ref incx);
			else
				DSCAL_unsafe(ref n, ref alpha, ref x, ref incx);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSCAL(ref int n, ref double alpha, 
			ref double x, ref int incx);
		private static unsafe void DSCAL_unsafe(ref int n, ref double alpha, 
			ref double x, ref int incx)
		{
			if (n <= 0) return;
			if (incx==0 || n==1)
				x *= alpha;
			else fixed (double* ax = &x) 
			{
				int nr=n;
				double* px = ax;	// writable copy
				if( incx==1)
				{
					if( alpha == 0.0) 
						for(; nr!=0; nr--)	// TODO: optimize loops
							*px++ = 0.0;
					else 
						for(; nr!=0; nr--) 	// TODO: optimize loops
							*px++ *= alpha;
				} 
				else 
				{
					if( alpha == 0.0) 
						for(; nr!=0; nr--, px+=incx)
							*px = 0.0;
					else 
						for(; nr!=0; nr--, px+=incx) 
							*px *= alpha;
				}
			}
		}


		public static void axpy(int n, double alpha, 
			ref double x, int incx, ref double y, int incy)
		{
			if( n>32 && Version != null)
				DAXPY(ref n, ref alpha, ref x, ref incx, ref y, ref incy);
			else 
				DAXPY_unsafe(ref n, ref alpha, ref x, ref incx, ref y, ref incy);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DAXPY(ref int n, ref double alpha, 
			ref double x, ref int incx, ref double y, ref int incy);
		private static unsafe void DAXPY_unsafe(ref int n, ref double alpha, 
			ref double x, ref int incx, ref double y, ref int incy)
		{
			if( n<=0 || alpha==0.0) return;
			fixed (double* ax = &x, ay = &y) 
			{
				int nr=n;
				double* py=ay, px=ax;
				if( incx==1 && incy==1)
				{
					if( alpha == 1.0)
						for(; nr!=0; nr--)	// TODO: optimize loops
							*py++ += *px++;
					else 
						for(; nr!=0; nr--)	// TODO: optimize loops
							*py++ += *px++ * alpha;
				} 
				else
				{
					if( alpha == 1.0)
						for(; nr!=0; nr--, py+=incy, px+=incx)	
							*py += *px;
					else 
						for(; nr!=0; nr--, py+=incy, px+=incx)
							*py += *px * alpha;
				}
			}
		}


		public static double dot(int n, ref double x, int incx, 
			ref double y, int incy)
		{
			if( n>32 && Version != null)
				return DDOT(ref n, ref x, ref incx, ref y, ref incy);
			else
				return DDOT_unsafe(ref n, ref x, ref incx, ref y, ref incy);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern double DDOT(ref int n, ref double x, ref int incx, 
			ref double y, ref int incy);
		private static unsafe double DDOT_unsafe(ref int n, ref double x, ref int incx, 
			ref double y, ref int incy)
		{
			double sum = 0.0;
			if( n<=0) return sum;
			fixed (double* ax = &x, ay = &y) 
			{
				int nr=n;
				double* py=ay, px=ax;
				if( incx==1 && incy==1)
					for(; nr!=0; nr--)	// TODO: optimize loop (unroll sum)
						sum += *px++ * *py++;
				else
					for(; nr!=0; nr--, py+=incy, px+=incx)
						sum += *px * *py;
			}
			return sum;
		}


		public static void swap(int n, ref double x, int incx, 
			ref double y, int incy)
		{
			if( n>32 && Version != null)
				DSWAP(ref n, ref x, ref incx, ref y, ref incy);
			else
				DSWAP_unsafe(ref n, ref x, ref incx, ref y, ref incy);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DSWAP(ref int n, ref double x, 
			ref int incx, ref double y, ref int incy);
		private static unsafe void   DSWAP_unsafe(ref int n, ref double x, 
			ref int incx, ref double y, ref int incy)
		{
			if( n<=0) return;
			fixed (double* ax = &x, ay = &y) 
			{
				int nr=n;
				double* py=ay, px=ax;
				if( incx==1 && incy==1)
				{
					for(; nr!=0; nr--)	// TODO: unroll loop
					{
						double tmp = *py;
						*py++ = *px;
						*px++ = tmp;
					}
				} 
				else
				{
					for(; nr!=0; nr--, py+=incy, px+=incx)	
					{
						double tmp = *py;
						*py = *px;
						*px = tmp;
					}
				}
			}
		}

		public static void copy(int n, ref double x, int incx, 
			ref double y, int incy)
		{
			if( n>50 && Version != null)
				DCOPY(ref n, ref x, ref incx, ref y, ref incy);
			else
				DCOPY_unsafe(ref n, ref x, ref incx, ref y, ref incy);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DCOPY(ref int n, ref double x, ref int incx, 
			ref double y, ref int incy);
		private static unsafe void   DCOPY_unsafe(ref int n, ref double x, ref int incx, 
			ref double y, ref int incy)
		{
			if( n<=0) return;
			fixed (double* ax = &x, ay = &y) 
			{
				int nr=n;
				double* py=ay, px=ax;
				if( incx==1 && incy==1)
				{
					for(; nr!=0; nr--)	// TODO: unroll loop
						*py++ = *px++;
				} 
				else
				{
					for(; nr!=0; nr--, py+=incy, px+=incx)	
						*py = *px;
				}
			}
		}
		public static double nrm2(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return DNRM2(ref n, ref x, ref incx);
			else
				return DNRM2_unsafe(ref n, ref x, ref incx);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern double DNRM2(ref int n, ref double x, ref int incx);
		private static unsafe double DNRM2_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0.0;
			fixed (double* ax = &x) 
			{
				int nr=n;
				double* px = ax;	// writable copy
				double scale = 0.0;
				double ssq = 1.0;

				for(; nr!=0; nr--, px+=incx)
				{
					if (*px != 0.0) 
					{
						double absxi = Math.Abs(*px);
						if (scale < absxi) 
						{
							double tmp = scale / absxi;
							ssq = ssq * tmp*tmp + 1.0;	// 2nd power
							scale = absxi;
						} 
						else 
						{
							double tmp = absxi / scale;
							ssq += tmp*tmp;	// 2nd power
						}
					}
				}
				return scale * Math.Sqrt(ssq);
			}
		}

		public static double asum(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return DASUM(ref n, ref x, ref incx);
			else
				return DASUM_unsafe(ref n, ref x, ref incx);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern double DASUM(ref int n, ref double x, ref int incx);
		private static unsafe double DASUM_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0.0;
			fixed (double* ax = &x) 
			{
				int nr=n;
				double* px = ax;	// writable copy
				double ret_val = 0.0;

				if( incx>1 )	// code for increment not equal to 1
				{
					for( ;nr!=0; nr--, px+=incx)
						ret_val += Math.Abs(*px);
				} 
				else			// code for increment equal to 1
				{
					for( ;nr!=0; nr--)	// TODO: unroll loop
						ret_val += Math.Abs(*px++);
				}
				return ret_val;
			}
		}

		public static int iamax(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return IDAMAX(ref n, ref x, ref incx) - 1;
			else
				return IDAMAX_unsafe(ref n, ref x, ref incx) - 1;
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern int    IDAMAX(ref int n, ref double x, ref int incx);
		private static unsafe int    IDAMAX_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0;	// ERROR!!
			if ( n==1) return 1;
			fixed (double* ax = &x) 
			{
				double* px = ax;	// writable copy

				int ret_val = 1;	// init with first element
				double dmax = Math.Abs(*px);

				if( incx>1 )	// code for increment not equal to 1
				{
					px+=incx;
					for(int i=2; i<=n; i++, px+=incx) 
					{
						double dabs = Math.Abs(*px);
						if( dabs > dmax)
						{
							dmax = dabs;
							ret_val = i;
						}
					}
				} 
				else 
				{
					++px;
					for(int i=2; i<=n; i++) 
					{
						double dabs = Math.Abs(*px++);
						if( dabs > dmax)
						{
							dmax = dabs;
							ret_val = i;
						}
					}
				}
				return ret_val;
			}
		}


		public static int iamin(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return IDAMIN(ref n, ref x, ref incx) - 1;
			else
				return IDAMIN_unsafe(ref n, ref x, ref incx) - 1;
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern int    IDAMIN(ref int n, ref double x, ref int incx);
		private static unsafe int    IDAMIN_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0;	// ERROR!!
			if ( n==1) return 1;
			fixed (double* ax = &x) 
			{
				double* px = ax;	// writable copy

				int ret_val = 1;	// init with first element
				double dmin = Math.Abs(*px);

				if( incx>1 )	// code for increment not equal to 1
				{
					px+=incx;
					for(int i=2; i<=n; i++, px+=incx) 
					{
						double dabs = Math.Abs(*px);
						if( dabs < dmin)
						{
							dmin = dabs;
							ret_val = i;
						}
					}
				} 
				else 
				{
					++px;
					for(int i=2; i<=n; i++) 
					{
						double dabs = Math.Abs(*px++);
						if( dabs < dmin)
						{
							dmin = dabs;
							ret_val = i;
						}
					}
				}
				return ret_val;
			}
		}

		public static double sum(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return DSUM(ref n, ref x, ref incx);
			else
				return DSUM_unsafe(ref n, ref x, ref incx);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern double DSUM(ref int n, ref double x, ref int incx);
		private static unsafe double DSUM_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0.0;
			fixed (double* ax = &x) 
			{
				int nr=n;
				double* px = ax;	// writable copy
				double ret_val = 0.0;

				if( incx>1 )	// code for increment not equal to 1
				{
					for( ;nr!=0; nr--, px+=incx)
						ret_val += *px;
				} 
				else			// code for increment equal to 1
				{
					for( ;nr!=0; nr--)	// TODO: unroll loop
						ret_val += *px++;
				}
				return ret_val;
			}
		}

		public static int imax(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return IDMAX(ref n, ref x, ref incx) - 1;
			else
				return IDMAX_unsafe(ref n, ref x, ref incx) - 1;
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern int    IDMAX(ref int n, ref double x, ref int incx);
		private static unsafe int    IDMAX_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0;	// ERROR!!
			if ( n==1) return 1;
			fixed (double* ax = &x) 
			{
				double* px = ax;	// writable copy
				int ret_val = 1;	// init with first element
				double dmax = *px;

				if( incx>1 )	// code for increment not equal to 1
				{
					px+=incx;
					for(int i=2; i<=n; i++, px+=incx) 
					{
						if( *px > dmax)
						{
							dmax = *px;
							ret_val = i;
						}
					}
				} 
				else 
				{
					++px;
					for(int i=2; i<=n; i++, px++) 
					{
						if( *px > dmax)
						{
							dmax = *px;
							ret_val = i;
						}
					}
				}
				return ret_val;
			}
		}


		public static int imin(int n, ref double x, int incx)
		{
			if( n>50 && Version != null)
				return IDMIN(ref n, ref x, ref incx) - 1;
			else
				return IDMIN_unsafe(ref n, ref x, ref incx) - 1;
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern int    IDMIN(ref int n, ref double x, ref int incx);
		private static unsafe int    IDMIN_unsafe(ref int n, ref double x, ref int incx)
		{
			if (n <= 0 || incx <= 0) return 0;	// ERROR!!
			if ( n==1) return 1;
			fixed (double* ax = &x) 
			{
				double* px = ax;	// writable copy

				int ret_val = 1;	// init with first element
				double dmin = *px;

				if( incx>1 )	// code for increment not equal to 1
				{
					px+=incx;
					for(int i=2; i<=n; i++, px+=incx) 
					{
						if( *px < dmin)
						{
							dmin = *px;
							ret_val = i;
						}
					}
				} 
				else 
				{
					++px;
					for(int i=2; i<=n; i++, px++) 
					{
						if( *px < dmin)
						{
							dmin = *px;
							ret_val = i;
						}
					}
				}
				return ret_val;
			}
		}

		#endregion

		#region BLAS Level 1 (no unsafe code available yet)

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DROTG(ref double a, ref double b, 
			ref double c, ref double s);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DROT(ref int n, ref double x, ref int incx, 
			ref double y, ref int incy, ref double c, ref double s);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DROTMG(ref double d1, ref double d2, 
			ref double x1, ref double y1, ref double param);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void   DROTM(ref int n, ref double x, 
			ref int incx, ref double y, ref int incy, ref double param);

		#endregion

		#region BLAS Level 2

		public static void gemv(char trans, 
			int m, int n, double alpha, ref double a, int lda,
			ref double x, int incx, double beta, ref double y, int incy)
		{
			System.Diagnostics.Debug.Assert(! object.ReferenceEquals(x,y));

			// reverse meaning since C# uses C-style row-major
			if (trans == 'N') trans = 'T';
			else if (trans == 'T' || trans == 'C') trans = 'N';
			else throw new ArgumentException( 
					 Strings.Get(typeof(BLAS), "BlasTransTCN"), "trans");

			if( m<0 || n<0 || incx==0 || incy==0 || lda<Math.Max(1,m) )
				throw new ArgumentException(
					Strings.Get(typeof(BLAS), "BlasGEMV"));

			if( Version != null && n*m>16)
				// swap n and m, see above
				DGEMV(ref trans, ref n, ref m, ref alpha, ref a, ref lda, 
					ref x, ref incx, ref beta, ref y, ref incy);
			else
				DGEMV_unsafe(ref trans, ref n, ref m, ref alpha, ref a, ref lda, 
					ref x, ref incx, ref beta, ref y, ref incy);
		}
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DGEMV(ref char trans, 
			ref int m, ref int n, ref double alpha, ref double a, ref int lda,
			ref double x, ref int incx, ref double beta, ref double y, ref int incy);
		private static unsafe void DGEMV_unsafe(ref char trans, 
			ref int m, ref int n, ref double alpha, ref double a, ref int lda,
			ref double x, ref int incx, ref double beta, ref double y, ref int incy)
		{
			if( m<0 || n<0 || incx==0 || incy==0 || lda<Math.Max(1,m) )
				throw new ArgumentException(
					Strings.Get(typeof(BLAS), "BlasGEMV") );

			int lenx, leny;	// lengths of the vectors x and y
			if (trans == 'N') 
				{ lenx = n; leny = m; } 
			else 
				{ lenx = m; leny = n; }

			// First form  y := beta*y
			DSCAL_unsafe(ref leny, ref beta, ref y, ref incy);

			// early return?
			if (alpha == 0.0) return;

			// set up the start points in x and y.
			int kx = (incx > 0) ? 0 : (1-lenx) * incx;
			int ky = (incy > 0) ? 0 : (1-leny) * incy;

			fixed( double* yp = &y, xp = &x, ap = &a )
			{
				if (trans == 'N') 
				{ 
					#region y := alpha*A*x + y
					int jx = kx;
					if (incy == 1) 
					{
						for (int j=0; j<n; j++, jx+=incx) 
							if (xp[jx] != 0.0) 
							{
								double temp = alpha * xp[jx];
								for (int i=0; i<m; i++)
									yp[i] += temp * ap[j*lda+i];
							}
					}
					else 
					{
						for (int j=0; j<n; j++, jx+=incx) 
							if (xp[jx] != 0.0) 
							{
								double temp = alpha * xp[jx];
								for (int i=0, iy=ky; i<m; i++, iy+=incy)
									yp[iy] += temp * ap[j*lda+i];
							}
					}
					#endregion
				} 
				else 
				{			
					#region y := alpha*A'*x + y
					int jy = ky;	//	Form  
					if (incx == 1) 
					{
						for (int j=0; j<n; j++, jy+=incy) 
						{
							double temp = 0.0;
							for (int i=0; i<m; i++) 
								temp += ap[j*lda+i] * xp[i];
							yp[jy] += alpha * temp;
						}
					}
					else 
					{
						for (int j=0; j<n; j++, jy+=incy) 
						{
							double temp = 0.0;
							for (int i=0, ix=kx; i<m; i++, ix+=incx) 
								temp += ap[j*lda+i] * xp[ix];
							yp[jy] += alpha * temp;
						}
					}
					#endregion
				}
			}
		}


 
		public static void ger( int m, int n, double alpha, 
			ref double x, int incx, ref double y, int incy,
			ref double a, int lda)
		{
			if( m<0 || n<0 || incx==0 || incy==0 || lda<Math.Max(1,m) )
				throw new ArgumentException(
					Strings.Get(typeof(BLAS), "BlasGEMV"));

			// swap n,m and x,y for row-major matrix
			if( Version != null && m*n>16)
				DGER( ref n, ref m , ref alpha, ref y, ref incy, 
					ref x, ref incx, ref a, ref lda);
			else
				DGER_unsafe( ref n, ref m , ref alpha, ref y, ref incy, 
					ref x, ref incx, ref a, ref lda);
		}

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DGER(ref int m, ref int n, ref double alpha, 
			ref double x, ref int incx, ref double y, ref int incy,
			ref double a, ref int lda);

		private static unsafe void DGER_unsafe(ref int m, ref int n, ref double alpha, 
			ref double x, ref int incx, ref double y, ref int incy,
			ref double a, ref int lda)
		{
			// quick return?
			if ( m==0 || n==0 || alpha == 0.0)
				return;

			fixed( double* yp = &y, xp = &x, ap = &a )
			{
				int jy = (incy > 0) ? 0 : (1-n) * incy;
				if (incx == 1) 
				{
					for (int j=0; j<n; j++, jy+=incy) 
						if (yp[jy] != 0.0) 
						{
							double temp = alpha * yp[jy];
							for (int i=0; i<m; i++)
								ap[j*lda+i] += xp[i] * temp;
						}
				} 
				else 
				{
					int kx = (incx > 0) ? 0 : (1-m) * incx;
					for (int j=0; j<n; j++, jy+=incy)
						if (yp[jy] != 0.0) 
						{
							double temp = alpha * yp[jy];
							for (int i=0, ix=kx; i<m; i++, ix+=incx)
								ap[j*lda+i] += xp[ix] * temp;
						}
				}
			}
		}


		#endregion

		#region BLAS Level 2 (no unsafe code available yet)

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYMV( ref char uplo, 
			ref int n, ref double alpha, ref double a, ref int lda, 
			ref double x, ref int incx, ref double beta, ref double y, ref int incy);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYR( ref char uplo, ref int n, ref double alpha, 
			ref double x, ref int incx, ref double a, ref int lda);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYR2( ref char uplo, ref int n, ref double alpha, 
			ref double x, ref int incx, ref double y, ref int incy, 
			ref double a, ref int lda);

		#endregion

		#region BLAS Level 3

		public static void gemm( char transa, char transb,
			int m, int n, int k, double alpha, ref double a, int lda,
			ref double b, int ldb, double beta, ref double c, int ldc)
		{
			System.Diagnostics.Debug.Assert(
				!object.ReferenceEquals(a,b) && 
				!object.ReferenceEquals(a,c) 
				);

			if( transa != 'T' && transa != 'C' && transa != 'N' )
				throw new ArgumentException( 
					Strings.Get(typeof(BLAS), "BlasTransTCN"), "transa");

			if( transb != 'T' && transb != 'C' && transb != 'N' )
				throw new ArgumentException( 
					Strings.Get(typeof(BLAS), "BlasTransTCN"), "transb");

			if( m<0 || n<0 || k<0 || 
				ldb<Math.Max(1, transb=='N'?n:k) || 
				lda<Math.Max(1, transa=='N'?k:m) || ldc<Math.Max(1,n) )
				throw new ArgumentException(
					Strings.Get(typeof(BLAS), "BlasGEMM"));

			// swap ta,tb  n,m and a,b for row-major matrix
			if( Version != null && m*n>16)
				DGEMM( ref transb, ref transa, ref n, ref m, ref k, 
					ref alpha, ref b, ref ldb, ref a, ref lda, 
					ref beta, ref c, ref ldc );
			else
				DGEMM_unsafe( ref transb, ref transa, ref n, ref m, ref k, 
					ref alpha, ref b, ref ldb, ref a, ref lda, 
					ref beta, ref c, ref ldc );
		}

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DGEMM( ref char transa, ref char transb,
			ref int m, ref int n, ref int k, ref double alpha, ref double a, ref int lda,
			ref double b, ref int ldb, ref double beta, ref double c, ref int ldc);

		private static unsafe void DGEMM_unsafe( ref char transa, ref char transb,
			ref int m, ref int n, ref int k, ref double alpha, ref double a, ref int lda,
			ref double b, ref int ldb, ref double beta, ref double c, ref int ldc)
		{
			// quick return?
			if ( n==0 || m==0 || (alpha==0.0 || k==0 ) && beta==1.0 )
				return;

			fixed( double* ap=&a, bp=&b, cp=&c )
			{
				#region alpha == 0.0
				if (alpha == 0.0) 
				{
					// TODO: can use DSCAL if continous data

					if (beta == 0.0) 
						for (int j=0; j<n; j++)
							for (int i=0; i<m; i++)
								cp[j*ldc+i] = 0.0;
					else 
						for (int j=0; j<n; j++)
							for (int i=0; i<m; i++)
								cp[j*ldc+i] *= beta;
					return;
				}
				#endregion

				bool notb = transb == 'N';
				bool nota = transa == 'N';

				if (nota) 
				{
					if (notb) 
					{
						#region C := alpha*B*A + beta*C
						for (int j=0; j<n; j++) 
						{
							if (beta == 0.0) 
								for (int i=0; i<m; i++)
									cp[j*ldc+i] = 0.0;
							else if (beta != 1.0) 
								for (int i=0; i<m; i++)
									cp[j*ldc+i] *= beta;
							for (int l=0; l<k; l++) 
								if (bp[j*ldb+l] != 0.0) 
								{
									double temp = alpha * bp[j*ldb+l];
									for (int i=0; i<n; i++)
										cp[j*ldc+i] += temp * ap[l*lda+i];
								}
						}
						#endregion
					} 
					else 
					{
						#region C := alpha*B'*A + beta*C
						for (int j=0; j<n; j++) 
							for (int i=0; i<m; i++) 
							{
								double temp = 0.0;
								for (int l=0; l<k; l++)
									temp += ap[i*lda+l] * bp[j*ldb+l];
								if (beta == 0.0)
									cp[j*ldc+i] = alpha * temp;
								else
									cp[j*ldc+i] = alpha * temp +
										beta * cp[j*ldc+i];
							}
						#endregion
					}
				} 
				else 
				{	
					if (notb) 
					{
						#region C := alpha*B*A' + beta*C
						for (int j=0; j<n; j++) 
						{
							if (beta == 0.0)
								for (int i=0; i<m; i++)
									cp[j*ldc+i] = 0.0;
							else if (beta != 1.0)
								for (int i=0; i<m; i++)
									cp[j*ldc+i] *= beta;
							for (int l=0; l<k; l++) 
								if (bp[l*ldb+j] != 0.0) 
								{
									double temp = alpha * bp[l*ldb+j];
									for (int i=0; i<m; i++)
										cp[j*ldc+i] += temp * ap[l*lda+i];
								}
						}
						#endregion
					} 
					else
					{
						#region C := alpha*B'*A' + beta*C
						for (int j=0; j<m; j++) 
							for (int i=0; i<n; i++) 
							{
								double temp = 0.0;
								for (int l=0; l<k; l++)
									temp += ap[i*lda+l] * bp[l*ldb+j];
								if (beta == 0.0)
									cp[j*ldc+i] = alpha * temp;
								else
									cp[j*ldc+i] = alpha * temp +
										beta * cp[j*ldc+i];

							}
						#endregion
					}
				}
			}
		}

		#endregion

		#region BLAS Level 3 (no unsafe code available yet)

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYMM( ref char side, ref char uplo, 
			ref int m, ref int n, ref double alpha, ref double a, ref int lda, 
			ref double b, ref int ldb, ref double beta, ref double c, ref int ldc);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYRK( ref char uplo, ref char trans, 
			ref int n, ref int k, ref double alpha, ref double a, ref int lda, 
			ref double beta, ref double c, ref int ldc);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void DSYR2K( ref char uplo, ref char trans, 
			ref int n, ref int k, ref double alpha, ref double a, ref int lda, 
			ref double b, ref int ldb, ref double beta, ref double c, ref int ldc);

		#endregion
	}
}
