// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SymmetricSchurDecomposition.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:11  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:46:31  stochastix
 *  updated license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.LA
{
	/// <summary>
	/// Symmetric threshold Jacobi algorithm.
	/// </summary>
	/// <remarks>
	/// Given a real symmetric matrix S, the Schur decomposition
	/// finds the eigenvalues and eigenvectors of S. If D is the
	/// diagonal matrix formed by the eigenvalues and U the
	/// unitarian matrix of th eigenvector we can write the
	/// Schur decomposition as
	/// 	\f[ S = U \cdot D \cdot U^T \, ,\f]
	/// where \f$ \cdot \f$ is the standard matrix product
	/// and  \f$ ^T  \f$ is the transpose operator.
	/// 
	/// This class implements the Schur decomposition using the
	/// symmetric threshold Jacobi algorithm. For details on the
	/// different Jacobi transfomations you can start from the great book
	/// on matrix computations by Golub and Van Loan: Matrix computation,
	/// second edition The Johns Hopkins University Press
	/// </remarks>
	public class SymmetricSchurDecomposition
	{
		/* \pre s must be symmetric */
		/// <summary>
		/// Constructs a SymmetricSchurDecomposition given
		/// a symmetric <see cref="Matrix"/>.
		/// </summary>
		/// <param name="symmetricMatrix">A symmetric Matrix which is not modified.</param>
		public SymmetricSchurDecomposition(Matrix symmetricMatrix)
		{
			this.size = symmetricMatrix.Rows;
			if( size != symmetricMatrix.Columns )
				throw new ArgumentException( Strings.Get(
					GetType(), "TODO: Input matrix must be square."), "symmetricMatrix");
			this.diagonal = symmetricMatrix.Diagonal.Clone();
			this.eigenVectors = new Matrix(size,size);
			this.eigenVectors.Diagonal.Fill(1.0);
			compute(symmetricMatrix);
		}

		private int size;
		private Vector diagonal;
		private Matrix eigenVectors;

		public Vector Eigenvalues
		{
			get { return diagonal; }
		}
		public Matrix Eigenvectors
		{
			get { return eigenVectors; }
		}

		private static readonly int maxIterations = 100;
		private static readonly double epsPrec = 1e-15;

		private void compute(Matrix symmetricMatrix)
		{
			double threshold;
			Vector tmpDiag = diagonal.Clone();
			Vector tmpAccumulate = new Vector(size);
			Matrix s = symmetricMatrix.Clone();

			for( int ite=1; ite<=maxIterations; ite++)
			{ 
				// abs sum of upper triangel
				double sum = 0.0;
				for (int j=0; j < size-1; j++) 
					for (int k=j+1; k < size; k++)
						sum += Math.Abs(s[j,k]);

				if (sum == 0.0) return;

				// To speed up computation a threshold is introduced to
				// make sure it is worthy to perform the Jacobi rotation
				if (ite < 5)
					threshold = 0.2*sum/(size*size);
				else
					threshold = 0.0;

				// sweep upper triangle
				for (int j=0; j < size-1; j++) 
				{
					for (int k=j+1; k < size; k++) 
					{
						double sine, rho, cosin, heig, tang;
						double smll = Math.Abs(s[j,k]);
						if( ite > 5 &&
							smll < epsPrec * Math.Abs(diagonal[j]) &&
							smll < epsPrec * Math.Abs(diagonal[k]))
						{
							s[j,k] = 0.0;
						}
						else if (Math.Abs(s[j,k]) > threshold) 
						{
							heig = diagonal[k]-diagonal[j];
							if ( smll < epsPrec * Math.Abs(heig) )
							{
								tang = s[j,k]/heig;
							}
							else 
							{
								double beta = 0.5 * heig / s[j,k];
								tang = 1.0 / ( Math.Abs(beta) + Math.Sqrt(1.0+beta*beta) );
								if(beta < 0.0) tang = -tang;
							}
							cosin = 1.0 / Math.Sqrt(1.0+tang*tang);
							sine  = tang * cosin;
							rho   = sine / (1.0+cosin);
							heig  = tang*s[j,k];

							tmpAccumulate[j] -= heig;
							tmpAccumulate[k] += heig;
							diagonal[j] -= heig;
							diagonal[k] += heig;
							s[j,k] = 0.0;

							// perform rotation on upper triangle
							int l=0;
							for (   ; l < j; l++) 
								jacobiRotate(s, rho, sine, l, j, l, k);
							for (++l; l < k; l++) 
								jacobiRotate(s, rho, sine, j, l, l, k);
							for (++l; l < size; l++) 
								jacobiRotate(s, rho, sine, j, l, k, l);

							for (l = 0; l < size; l++) 
								jacobiRotate(eigenVectors, rho, sine, l, j, l, k);
						}
					}
				}
				// tmpDiag.Add(tmpAccumulate);
				// diagonal = tmpDiag.Clone();
				// tmpAccumulate.Fill(0.0);
				for (int j=0; j < size; j++) 
				{
					tmpDiag[j] += tmpAccumulate[j];
					diagonal[j] = tmpDiag[j];
					tmpAccumulate[j] = 0.0;
				}
			}
			throw new ApplicationException( Strings.Get( GetType(), 
				"TODO: Too many iterations reached"));
		}

		/// <summary>
		/// This routines implements the Jacobi, a.k.a. Givens, rotation.
		/// </summary>
		private void jacobiRotate(Matrix m, double rot, double dil, int j1, int k1, int j2, int k2)
		{
			double x1, x2;
			x1 = m[j1,k1];
			x2 = m[j2,k2];
			m[j1,k1] = x1 - dil*(x2 + x1*rot);
			m[j2,k2] = x2 + dil*(x1 - x2*rot);
		}
	}
}
