// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: VML.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:11  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:47:16  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:35:02  stochastix
 *  moved to LA namespace
 *
 *  Revision 1.6  2002/08/21 17:47:55  stochastix
 *  updated
 *
 *  Revision 1.5  2002/08/20 16:43:38  stochastix
 *  Completed VML interface;
 *  moved native lib to DotNum.Native.dll;
 *  killed bugs involving "ref" types
 *
 *  Revision 1.4  2002/08/20 08:51:43  stochastix
 *  added more vector math; comments
 *
 *  Revision 1.3  2002/08/19 15:36:13  stochastix
 *  Changed to cdecl conventions (was: stdcall)
 *  to ease integration of other libraries (eg. ATLAS).
 *
 *  Revision 1.2  2002/08/15 18:12:22  stochastix
 *  Change GetVersionXXX names
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/09 18:11:06  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Diagnostics;
using System.Text;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics.LA
{

	/// <summary>
	/// Interface to optimized vector methematics library.
	/// </summary>
	[ RcsId("$Id: VML.cs,v 1.1.1.1 2003/02/01 18:42:11 jens Exp $") ] 
	[ ComVisible(false) ]
	public sealed class VML
	{
		public static readonly string Version;

		#region Static Constructor and availability check

		private const string IMPORT_DLL = "Stochastix.Numerics.Native.dll";
		private const string DLL_Version = "GetVersionVML";

		private static readonly Mode oldmode;

		static VML() 
		{
			StringBuilder buf = new StringBuilder(120);
			try 
			{
				GetVersion(buf, buf.Capacity);
				oldmode = vmlGetMode();
				vmlSetMode(Mode.VML_HA | Mode.VML_DOUBLE_CONSISTENT | 
					Mode.VML_ERRMODE_ERRNO);
				vmlClearErrStatus();
			} 
			catch (Exception e) 
			{
				Trace.WriteLine("VML not loaded: " + e.Message);
				return;
			}
			Version = buf.ToString().Trim();
			Trace.WriteLine("VML loaded '" + Version + "'");
		}

		[ DllImport(IMPORT_DLL, EntryPoint=DLL_Version) ]
		private static extern void GetVersion(StringBuilder buf, int bufSize);

		#endregion

		#region VML Service Functions

		/// <summary>
		/// Codes for vml?Error function call and errstatus.
		/// </summary>
		public enum Status
		{
			VML_STATUS_OK			=	 0,
			VML_STATUS_BADSIZE		=	-1,
			VML_STATUS_BADMEM		=	-2,
			VML_STATUS_ERRDOM		=	 1,
			VML_STATUS_SING			=	 2,
			VML_STATUS_OVERFLOW		=	 3,
			VML_STATUS_UNDERFLOW	=	 4,
			/// <summary>
			/// VML ops were emulated, status unknown.
			/// </summary>
			VML_STATUS_UNKNOWN		=  -42
		}

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern Status vmlSetErrStatus(Status status);

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern Status vmlGetErrStatus();

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern Status vmlClearErrStatus();

		[ Flags ]
		private enum Mode
		{
			// Defines for accuracy setting LA or HA
			VML_XX					=	0x000,
			VML_LA					=	0x001,
			VML_21					=	0x001,
			VML_50					=	0x001,
			VML_HA					=	0x002,
			VML_11					=	0x004,
			// Defines for precision setting mode (to set or not to set :0))
			VML_DEFAULT_PRECISION	=	0x000,
			VML_FLOAT_CONSISTENT	=	0x010,
			VML_DOUBLE_CONSISTENT	=	0x020,
			VML_RESTORE				=	0x030,
			// Defines for error modes
			VML_ERRMODE_IGNORE		=	0x0100,
			VML_ERRMODE_ERRNO		=	0x0200,
			VML_ERRMODE_STDERR		=	0x0400,
			VML_ERRMODE_EXCEPT		=	0x0800,
			VML_ERRMODE_CALLBACK	=	0x1000,
			VML_ERRMODE_DEFAULT		=	VML_ERRMODE_ERRNO | VML_ERRMODE_CALLBACK | VML_ERRMODE_EXCEPT,
			// Mask for accuracy setting separation
			VML_ACCURACY_MASK		=	0x000f,
			// Mask for precision setting separation
			VML_FPUMODE_MASK		=	0x00f0,
			// Mask for error mode separation
			VML_ERRMODE_MASK		=	0xff00
		}

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern Mode vmlSetMode(Mode newmode);

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern Mode vmlGetMode();

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vdPackI(int n, ref double a, int inca, ref double y);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vdUnpackI(int n, ref double a, ref double y, int incy);

		#endregion

		#region VML Inv / Roots / Pow / Div

		/// <summary>
		/// Performs element by element inversion ( 1.0 / a) of the vector.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Inv(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Inv(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdInv(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdInv(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = 1.0 / *pa++;
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = 1.0 / *pa;
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Performs element by element inversion ( 1.0 / a) of the vector.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdInv( int n, ref double a, ref double r);

		/// <summary>
		/// Computes square root ( a**0.5 ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Sqrt(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
					Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Sqrt(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdSqrt(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdSqrt(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Sqrt(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Sqrt(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes square root ( a**0.5 ) of vector elements..
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdSqrt( int n, ref double a, ref double r);


		/// <summary>
		/// Computes inversion square root ( 1.0 / a**0.5 ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status InvSqrt(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = InvSqrt(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdInvSqrt(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdInvSqrt(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = 1.0 / Math.Sqrt(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = 1.0 / Math.Sqrt(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inversion square root ( 1.0 / a**0.5 ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdInvSqrt( int n, ref double a, ref double r);


		/// <summary>
		/// Computes cube root ( a**(1/3) ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Cbrt(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Cbrt(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdCbrt(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdCbrt(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				vdPowNum(n, ref a, inca, 1.0/3.0, ref r, incr);

			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes cube root ( a**(1/3) ) of vector elements..
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdCbrt( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inversion cube root ( 1.0 / a**(1/3) ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status InvCbrt(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = InvCbrt(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdInvCbrt(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdInvCbrt(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				vdPowNum(n, ref a, inca, -1.0/3.0, ref r, incr);

			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inversion cube root ( 1.0 / a**(1/3) ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdInvCbrt( int n, ref double a, ref double r);


		/// <summary>
		/// Computes power ( a**s ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="s">A number that specifies a power.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static unsafe void vdPowNum( int n, ref double a, int inca, double s,
			ref double r, int incr)
		{
			fixed (double* aa=&a, ar=&r) 
			{
				if( inca==1 && incr==1)
					for(double* pa=aa, pr=ar; n!=0; n--)
						*pr++ = Math.Pow(*pa++, s);
				else 
					for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
						*pr = Math.Pow(*pa, s);
			}
		}

		/// <summary>
		/// Computes power ( a**b ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first input vector (not modified).</param>
		/// <param name="inca">Increment of the first input vector.</param>
		/// <param name="b">First element of the second input vector (not modified).</param>
		/// <param name="incb">Increment of the second input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Pow(int n, ref double a, int inca, 
			ref double b, int incb, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Pow(n, ref a, inca, ref b, incb, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] atmp = new double[n];
					vdPackI(n, ref a, inca, ref atmp[0]);
					return Pow(n, ref atmp[0], 1, ref b, incb, ref r, incr);
				} 
				else if( incb != 1) 
				{
					double[] btmp = new double[n];
					vdPackI(n, ref b, incb, ref btmp[0]);
					vdPow(n, ref a, ref btmp[0], ref r);
				} 
				else 
				{
					vdPow(n, ref a, ref b, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ab=&b, ar=&r) 
					{
						if( inca==1 && incb==1 && incr==1)
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--)
								*pr++ = Math.Pow(*pa++, *pb++);
						else 
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--, pr+=incr, pa+=inca, pb+=incb)
								*pr = Math.Pow(*pa, *pb);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes power ( a**b ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first continous input vector (not modified).</param>
		/// <param name="b">First element of the second continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdPow( int n, ref double a, ref double b, ref double r);


		/// <summary>
		/// Computes division ( a/b ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first input vector (not modified).</param>
		/// <param name="inca">Increment of the first input vector.</param>
		/// <param name="b">First element of the second input vector (not modified).</param>
		/// <param name="incb">Increment of the second input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Div(int n, ref double a, int inca, 
			ref double b, int incb, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Div(n, ref a, inca, ref b, incb, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] atmp = new double[n];
					vdPackI(n, ref a, inca, ref atmp[0]);
					return Div(n, ref atmp[0], 1, ref b, incb, ref r, incr);
				} 
				else if( incb != 1) 
				{
					double[] btmp = new double[n];
					vdPackI(n, ref b, incb, ref btmp[0]);
					vdDiv(n, ref a, ref btmp[0], ref r);
				} 
				else 
				{
					vdDiv(n, ref a, ref b, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ab=&b, ar=&r) 
					{
						if( inca==1 && incb==1 && incr==1)
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--)
								*pr++ = *pa++ / *pb++;
						else 
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--, pr+=incr, pa+=inca, pb+=incb)
								*pr = *pa / *pb;
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes division ( a/b ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first continous input vector (not modified).</param>
		/// <param name="b">First element of the second continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdDiv( int n, ref double a, ref double b, ref double r);

		#endregion

		#region VML Exp / Log / Log10

		/// <summary>
		/// Computes exponent ( e**a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Exp(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Exp(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdExp(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdExp(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Exp(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Exp(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes exponent ( e**a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdExp( int n, ref double a, ref double r);


		/// <summary>
		/// Computes logarithm ( ln a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Ln(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Ln(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdLn(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdLn(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Log(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Log(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes logarithm ( ln a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdLn( int n, ref double a, ref double r);


		/// <summary>
		/// Computes decimal logarithm ( lg a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Log10(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>32 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Log10(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdLog10(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdLog10(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Log10(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Log10(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes decimal logarithm ( lg a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdLog10( int n, ref double a, ref double r);

		#endregion

		#region ( VML Gamma )

		/// <summary>
		/// Computes gamma function on vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Gamma(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));

			// also the gamma function is adevrtised in the header, 
			// it is not available in the library.

//			if( n>=5 && Version != null )
//			{
//				if( incr!=1)
//				{
//					double[] outtmp = new double[n];		// make temp
//					Status res = Gamma(n, ref a, inca, ref outtmp[0], 1);	// recurse
//					vdUnpackI(n, ref outtmp[0], ref r, incr);
//					return res;
//				} 
//				else if( inca != 1) 
//				{
//					double[] intmp = new double[n];
//					vdPackI(n, ref a, inca, ref intmp[0]);
//					vdGamma(n, ref intmp[0], ref r);
//				} 
//				else 
//				{
//					vdGamma(n, ref a, ref r);
//				}
//				return vmlClearErrStatus();
//			}
//			else
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Distributions.GammaFunction.LogValue(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Distributions.GammaFunction.LogValue(*pa);
					}

				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes gamma function on vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdGamma( int n, ref double a, ref double r);

		#endregion
		
		#region VML Sin / Asin / Cos / Acos / SinCos

		/// <summary>
		/// Computes sine (sin a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Sin(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Sin(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdSin(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdSin(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Sin(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Sin(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes sine (sin a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdSin( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse sine (arcsin a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Asin(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Asin(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAsin(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAsin(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Asin(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Asin(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse sine (arcsin a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAsin( int n, ref double a, ref double r);

		/// <summary>
		/// Computes cosine (cos a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Cos(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Cos(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdCos(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdCos(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Cos(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Cos(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes cosine (cos a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdCos( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse cosine (arccos a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Acos(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Acos(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAcos(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAcos(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Acos(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Acos(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse cosine (arccos a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAcos( int n, ref double a, ref double r);

		/// <summary>
		/// Computes sine and cosine ( r1 = sin a, r2 = cos a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r1">First element of the first output vector.</param>
		/// <param name="incr1">Increment of the first output vector.</param>
		/// <param name="r2">First element of the second output vector.</param>
		/// <param name="incr2">Increment of the second output vector.</param>
		public static Status SinCos(int n, ref double a, int inca, 
			ref double r1, int incr1, ref double r2, int incr2)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( inca != 1) 
				{
					double[] atmp = new double[n];
					vdPackI(n, ref a, inca, ref atmp[0]);
					return SinCos(n, ref atmp[0], 1, ref r1, incr1, ref r2, incr2);
				} 
				else if( incr1 != 1)
				{
					double[] rtmp = new double[n];
					Status res = SinCos(n, ref a, inca, ref rtmp[0], 1, ref r2, incr2);
					vdUnpackI(n, ref rtmp[0], ref r1, incr1);
					return res;
				}
				else if( incr2 != 1)
				{
					double[] rtmp = new double[n];
					vdSinCos(n, ref a, ref r1, ref rtmp[0]);
					vdUnpackI(n, ref rtmp[0], ref r2, incr2);
				}
				else
				{
					vdSinCos(n, ref a, ref r1, ref r2);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar1=&r1, ar2=&r2) 
					{
						if( inca==1 && incr1==1 && incr2==1)
							for(double* pa=aa, pr1=ar1, pr2=ar2; n!=0; n--)
							{
								double tmp = *pa++;
								*pr1++ = Math.Sin(tmp);
								*pr2++ = Math.Cos(tmp);
							}
						else 
							for(double* pa=aa, pr1=ar1, pr2=ar2; n!=0; n--, pa+=inca, pr1+=incr1, pr2+=incr2)
							{
								double tmp = *pa;
								*pr1 = Math.Sin(tmp);
								*pr2 = Math.Cos(tmp);
							}
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes sine and cosine ( r1 = sin a, r2 = cos a ) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r1">First element of the first continous output vector.</param>
		/// <param name="r2">First element of the second continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdSinCos( int n, ref double a, ref double r1, ref double r2);

		#endregion

		#region VML Tan / Atan / Atan2

		/// <summary>
		/// Computes tangent (tan a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Tan(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Tan(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdTan(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdTan(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Tan(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Tan(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes tangent (tan a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdTan( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse tangent (arctan a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Atan(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Atan(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAtan(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAtan(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Atan(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Atan(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse tangent (arctan a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAtan( int n, ref double a, ref double r);

		/// <summary>
		/// Computes four-quadrant inverse tangent (arctan a/b) of elements of two vectors.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first input vector (not modified).</param>
		/// <param name="inca">Increment of the first input vector.</param>
		/// <param name="b">First element of the second input vector (not modified).</param>
		/// <param name="incb">Increment of the second input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Atan2(int n, ref double a, int inca, 
			ref double b, int incb, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Atan2(n, ref a, inca, ref b, incb, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] atmp = new double[n];
					vdPackI(n, ref a, inca, ref atmp[0]);
					return Atan2(n, ref atmp[0], 1, ref b, incb, ref r, incr);
				} 
				else if( incb != 1) 
				{
					double[] btmp = new double[n];
					vdPackI(n, ref b, incb, ref btmp[0]);
					vdAtan2(n, ref a, ref btmp[0], ref r);
				} 
				else 
				{
					vdAtan2(n, ref a, ref b, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ab=&b, ar=&r) 
					{
						if( inca==1 && incb==1 && incr==1)
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--)
								*pr++ = Math.Atan2(*pa++, *pb++);
						else 
							for(double* pa=aa, pb=ab, pr=ar; n!=0; n--, pr+=incr, pa+=inca, pb+=incb)
								*pr = Math.Atan2(*pa, *pb);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes four-quadrant inverse tangent (arctan a/b) of elements of two vectors.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the first continous input vector (not modified).</param>
		/// <param name="b">First element of the second continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAtan2( int n, ref double a, ref double b, ref double r);

		#endregion

		#region VML Sinh / Asinh / Cosh / Acosh / Tanh / Atanh

		/// <summary>
		/// Computes hyperbolic sine (sinh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Sinh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Sinh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdSinh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdSinh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Sinh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Sinh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes hyperbolic sine (sinh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdSinh( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse hyperbolic sine (arcsinh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Asinh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Asinh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAsinh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAsinh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Asinh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Asinh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse hyperbolic sine (arcsinh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAsinh( int n, ref double a, ref double r);

		private static double Asinh(double value)
		{
			return Math.Log(value + Math.Sqrt(value*value + 1));
		}

		/// <summary>
		/// Computes hyperbolic cosine (cosh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Cosh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Cosh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdCosh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdCosh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Cosh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Cosh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes hyperbolic cosine (cosh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdCosh( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse hyperbolic cosine (arccosh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Acosh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Acosh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAcosh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAcosh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Acosh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Acosh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse hyperbolic cosine (arccosh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAcosh( int n, ref double a, ref double r);

		private static double Acosh(double value)
		{
			// return Math.Log(value + Math.Sqrt(value*value -1 ));
			return Math.Log(Math.Abs(value) + Math.Sqrt(value*value -1 ));
		}

		/// <summary>
		/// Computes hyperbolic tangent (tanh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Tanh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Tanh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdTanh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdTanh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Math.Tanh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Math.Tanh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes tangent (tanh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdTanh( int n, ref double a, ref double r);

		/// <summary>
		/// Computes inverse hyperbolic tangent (arctanh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the input vector (not modified).</param>
		/// <param name="inca">Increment of the input vector.</param>
		/// <param name="r">First element of the output vector.</param>
		/// <param name="incr">Increment of the output vector.</param>
		public static Status Atanh(int n, ref double a, int inca, ref double r, int incr)
		{
			if( n<0 ) throw new ArgumentOutOfRangeException("n", n, 
						  Strings.Get( typeof(VML), "VmlDimNeg"));
			if( n>=5 && Version != null )
			{
				if( incr!=1)
				{
					double[] outtmp = new double[n];		// make temp
					Status res = Atanh(n, ref a, inca, ref outtmp[0], 1);	// recurse
					vdUnpackI(n, ref outtmp[0], ref r, incr);
					return res;
				} 
				else if( inca != 1) 
				{
					double[] intmp = new double[n];
					vdPackI(n, ref a, inca, ref intmp[0]);
					vdAtanh(n, ref intmp[0], ref r);
				} 
				else 
				{
					vdAtanh(n, ref a, ref r);
				}
				return vmlClearErrStatus();
			}
			else 
				unsafe
				{
					fixed (double* aa=&a, ar=&r) 
					{
						if( inca==1 && incr==1)
							for(double* pa=aa, pr=ar; n!=0; n--)
								*pr++ = Atanh(*pa++);
						else 
							for(double* pa=aa, pr=ar; n!=0; n--, pr+=incr, pa+=inca)
								*pr = Atanh(*pa);
					}
				}
			return Status.VML_STATUS_UNKNOWN;
		}

		/// <summary>
		/// Computes inverse hyperbolic tangent (arctanh a) of vector elements.
		/// </summary>
		/// <param name="n">Specifies the number of elements to be calculated.</param>
		/// <param name="a">First element of the continous input vector (not modified).</param>
		/// <param name="r">First element of the continous output vector.</param>
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		public static extern void vdAtanh( int n, ref double a, ref double r);

		/// <summary>
		/// Inverse hyperbolic tangent, i.e. angle whose hyperbolic tangent is value. 
		/// </summary>
		/// <param name="value"></param>
		/// <returns></returns>
		private static double Atanh(double value)
		{
			return Math.Log((1+value) / (1-value)) / 2;
		}

		#endregion

/*
		public class VmlException : ApplicationException
		{
			internal VmlException(string functionName) : base( 
				String.Format("VML Exception '{0:G}' while doing {1}",
				vmlClearErrStatus(), functionName))
			{}
		}
*/
	}
}
