// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Solver1D.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:12  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:44:45  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:36:34  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:11  stochastix
 *  Initial revision
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Solvers1D
{

	/// <summary>
	/// Abstract base class for 1-D solvers.
	/// </summary>
	/// <remarks>
	/// <para>
	///	This abstract class provides the interface for one-dimensional solvers 
	///	which can find the zeroes of a given function.
	///	</para>
	///	<para>
	///	The implementation of the algorithms was inspired by 
	///	<see href="http://www.library.cornell.edu/nr/bookcpdf.html">Numerical Recipes in C</see>, 
	///	2nd edition, Cambridge University Press, Teukolsky, Vetterling, Flannery - Chapter 9.
	///	</para>
	///	<para>
	///	Some work is needed to resolve the ambiguity of the root finding accuracy
	///	defition: for some algorithms it is the x-accuracy, for others it is
	///	f(x)-accuracy.
	///	</para>
	///	</remarks>
	///	<seealso href="http://www.library.cornell.edu/nr/bookcpdf.html">
	///	Numerical Recipes in C (free online book)
	///	</seealso>
	[ RcsId("$Id: Solver1D.cs,v 1.1.1.1 2003/02/01 18:42:12 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Solver1D : ISolver1D
	{
		private const int MAX_FUNCTION_EVALUATIONS = 100;

		private int maxEvaluations;

		private double lowerBound, upperBound;
		private bool lowerBoundEnforced, upperBoundEnforced;

		/// <summary>
		/// Number of function evaluations for the bracketing routine.
		/// </summary>
		/// <remarks>
		/// An <see cref="ApplicationException"/> should be thrown by derived
		/// classes when a bracket is not found after <see cref="MaxEvaluations"/>
		/// of evaluations.
		/// </remarks>
		protected int evaluationNumber;

		/// <summary>
		/// Derived classes implementing <see cref="SolveImpl"/> assume
		/// that this member was initialized to a valid initial guess.
		/// </summary>
		protected double root;

		/// <summary>
		/// Derived classes implementing <see cref="SolveImpl"/> assume that
		/// <see cref="xMin"/> and <see cref="xMax"/> form a valid bracket.
		/// </summary>
		protected double xMin;

		/// <summary>
		/// Derived classes implementing <see cref="SolveImpl"/> assume that
		/// <see cref="xMin"/> and <see cref="xMax"/> form a valid bracket.
		/// </summary>
		protected double xMax;

		/// <summary>
		/// Derived classes implementing <see cref="SolveImpl"/> assume that
		/// <see cref="fxMin"/> and <see cref="fxMax"/> contain the values of the
		///	function in <see cref="xMin"/> and <see cref="xMax"/>.
		/// </summary>
		protected double fxMin;

		/// <summary>
		/// Derived classes implementing <see cref="SolveImpl"/> assume that
		/// <see cref="fxMin"/> and <see cref="fxMax"/> contain the values of the
		///	function in <see cref="xMin"/> and <see cref="xMax"/>.
		/// </summary>
		protected double fxMax;

		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected Solver1D()
		{
			this.maxEvaluations = MAX_FUNCTION_EVALUATIONS;
			this.lowerBoundEnforced = false;
			this.upperBoundEnforced = false;
		}

		/// <summary>
		/// This method returns the zero of the <see cref="IObjectiveFunction"/>
		/// <pramref name="objectiveFunction"/>, determined with the given accuracy.
		/// </summary>
		/// <remarks>
		/// <i>x</i> is considered a zero if <i>|f(x)| &lt; accuracy</i>.
		/// This method contains a bracketing routine to which an initial
		/// guess must be supplied as well as a step used to scan the range
		/// of the possible bracketing values.
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <param name="guess">Initial guess used to scan the range
		/// of the possible bracketing values.</param>
		/// <param name="step">Initial step used to scan the range
		/// of the possible bracketing values.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="MaxEvaluations"/>).
		/// </exception>
		public double Solve( IObjectiveFunction objectiveFunction, 
			double xAccuracy, double guess, double step)
		{
			const double growthFactor = 1.6;
			int flipflop = -1;

			root = guess;
			fxMax = objectiveFunction.Value(root);

			// monotonically crescent bias, as in optionValue(volatility)
			if (Math.Abs(fxMax) <= xAccuracy)
				return root;
			else if (fxMax > 0.0) 
			{
				xMin = enforceBounds(root - step);
				fxMin = objectiveFunction.Value(xMin);
				xMax = root;
			} 
			else 
			{
				xMin = root;
				fxMin = fxMax;
				xMax = enforceBounds(root+step);
				fxMax = objectiveFunction.Value(xMax);
			}

			evaluationNumber = 2;
			while (evaluationNumber <= maxEvaluations) 
			{
				if (fxMin*fxMax <= 0.0) 
				{
					if (fxMin == 0.0) return xMin;
					if (fxMax == 0.0) return xMax;
					root = (xMax+xMin)/2.0;
					// check whether we really want to pass epsilon
					return SolveImpl(objectiveFunction, 
						Math.Max(Math.Abs(xAccuracy), Double.Epsilon));
				}
				if (Math.Abs(fxMin) < Math.Abs(fxMax)) 
				{
					xMin = enforceBounds(xMin + growthFactor*(xMin-xMax));
					fxMin= objectiveFunction.Value(xMin);
				} 
				else if (Math.Abs(fxMin) > Math.Abs(fxMax)) 
				{
					xMax = enforceBounds(xMax + growthFactor*(xMax-xMin));
					fxMax= objectiveFunction.Value(xMax);
				} 
				else if (flipflop == -1) 
				{
					xMin = enforceBounds(xMin + growthFactor*(xMin-xMax));
					fxMin= objectiveFunction.Value(xMin);
					evaluationNumber++;
					flipflop = 1;
				} 
				else if (flipflop == 1) 
				{
					xMax = enforceBounds(xMax + growthFactor*(xMax-xMin));
					fxMax= objectiveFunction.Value(xMax);
					flipflop = -1;
				}
				evaluationNumber++;
			}

			throw new ApplicationException( Strings.Format( GetType(), 
				"SlvBrktMaxEval", maxEvaluations, formatBracket())
			);
		}

		private string formatBracket() 
		{
			return String.Format( "f[{0:10},{1:10}] -> [{2:20},{3:20}]",
				new object[] { xMin, xMax, fxMin, fxMax });
		}

		/// <summary>
		/// This method returns the zero of the <see cref="IObjectiveFunction"/>
		/// <pramref name="objectiveFunction"/>, determined with the given accuracy.
		/// </summary>
		/// <remarks>
		/// <i>x</i> is considered a zero if <i>|f(x)| &lt; accuracy</i>.
		/// An initial guess must be supplied, as well as two values which 
		/// must bracket the zero (i.e., either 
		/// <i>f(x<sub>min</sub>) &gt; 0</i> &amp;&amp; 
		/// <i>f(x<sub>max</sub>) &lt; 0</i>, or 
		/// <i>f(x<sub>min</sub>) &lt; 0</i> &amp;&amp; 
		/// <i>f(x<sub>max</sub>) &gt; 0</i> must be true). 
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <param name="guess">Initial guess used to scan the range
		/// of the possible bracketing values.</param>
		/// <param name="xMin">Lower bracket.</param>
		/// <param name="xMax">Upper bracket.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="MaxEvaluations"/>).
		/// </exception>
		public double Solve( IObjectiveFunction objectiveFunction, double xAccuracy,
			double guess, double xMin, double xMax)
		{
			if (xMin >= xMax) 
				throw new ArgumentException( Strings.Format(
					GetType(), "SlvBrktMaxEval", xMin, xMax), 
					(String)null);

			if (lowerBoundEnforced && xMin < lowerBound) 
				throw new ArgumentException( Strings.Format(
					GetType(), "SlvLowBound", xMin, lowerBound),
					"xMin");

			if (upperBoundEnforced && xMax > upperBound) 
				throw new ArgumentException( Strings.Format(
					GetType(), "SlvUpBound", xMax, upperBound),
					"xMax");

			this.xMin = xMin;
			this.xMax = xMax;

			fxMin = objectiveFunction.Value(xMin);
			if (Math.Abs(fxMin) < xAccuracy) return xMin;

			fxMax = objectiveFunction.Value(xMax);
			if (Math.Abs(fxMax) < xAccuracy) return xMax;

			evaluationNumber = 2;

			if ( fxMin*fxMax >= 0.0 ) 
				throw new ApplicationException( Strings.Format(
					GetType(), "SlvNotBrkt", formatBracket())
				);

			/* TODO: errors where inconsistent with conditions

			QL_REQUIRE(guess > xMin_, "Solver1D: guess (" +
						DoubleFormatter::toString(guess) + ") < xMin_ (" +
						DoubleFormatter::toString(xMin_) + ")");

			QL_REQUIRE(guess < xMax_, "Solver1D: guess (" +
						DoubleFormatter::toString(guess) + ") > xMax_ (" +
						DoubleFormatter::toString(xMax_) + ")");

			*/

			if ( guess < xMin ) 
				throw new ApplicationException( Strings.Format(
					GetType(), "SlvGuessMin", guess, xMin)
					);

			if ( guess > xMax ) 
				throw new ApplicationException( Strings.Format(
					GetType(), "SlvGuessMax", guess, xMax)
					);

			root = guess;

			return SolveImpl(objectiveFunction, 
				Math.Max(Math.Abs(xAccuracy), Double.Epsilon));

		
		}
		/// <summary>
		/// Maximum number of function evaluations for the bracketing routine. 
		/// An Error is thrown if a bracket is not found after this number of 
		/// evaluations.
		/// </summary>
		public int MaxEvaluations
		{
			get { return maxEvaluations; } 
			set { maxEvaluations = value; }
		}
		
		/// <summary>
		/// The lower bound for the function domain.
		/// </summary>
		public double LowerBound
		{
			get { return lowerBound; }
			set 
			{
				lowerBound = value;
				lowerBoundEnforced = true;
			}
		}

		/// <summary>
		/// The upper bound for the function domain.
		/// </summary>
		public double UpperBound
		{
			get { return upperBound; }
			set 
			{
				upperBound = value;
				upperBoundEnforced = true;
			}
		}

		private double enforceBounds(double x)
		{
			if (lowerBoundEnforced && x < lowerBound)
				return lowerBound;
			if (upperBoundEnforced && x > upperBound)
				return upperBound;
			return x;
		}

		/// <summary>
		/// This method must be implemented in derived classes and 
		/// contains the actual code which searches for the zeroes of 
		/// the <see cref="IObjectiveFunction"/>.
		/// </summary>
		/// <remarks>
		/// It assumes that:
		/// <list type="bullet">
		/// <item>
		///   <description>
		///		<see cref="xMin"/> and <see cref="xMax"/> form a valid bracket; 
		///	  </description>
		/// </item>
		/// <item>
		///   <description>
		///     <see cref="fxMin"/> and <see cref="fxMax"/> contain the values of the
		///		function in <see cref="xMin"/> and <see cref="xMax"/>; 
		///	  </description>
		/// </item>
		/// <item>
		///   <description>
		///     <see cref="root"/> was initialized to a valid initial guess.
		///	  </description>
		/// </item>
		/// </list>
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="MaxEvaluations"/>).
		/// </exception>
		abstract protected double SolveImpl(IObjectiveFunction objectiveFunction, 
			double xAccuracy);
	};

}
