// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RiskStatistics.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:49  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:50:09  stochastix
 *  moved from QuantLib.NET
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics.Resources;
using Stochastix.Numerics.Distributions;
using Stochastix.Utilities;

namespace Stochastix.Numerics.Statistics
{
	/// <summary>
	/// Risk statistic tool.
	/// </summary>
	[ RcsId("$Id: RiskStatistics.cs,v 1.1.1.1 2003/02/01 18:42:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public class RiskStatistics : Accumulator
	{
		public RiskStatistics() : base()
		{}

		#region RiskStatistics / RiskMeasures

		/// <summary>
		/// Potential-Upside at a given percentile.
		/// </summary>
		/// <param name="percentile"></param>
		/// <returns></returns>
		public double PotentialUpside(double percentile)
		{
			return PotentialUpside( percentile,
				Mean, StandardDeviation);
		}

		/// <summary>
		/// Potential upside.
		/// </summary>
		/// <param name="percentile">Percentile must be in range 90%-100%.</param>
		/// <param name="mean"></param>
		/// <param name="std"></param>
		/// <returns></returns>
		public static double PotentialUpside(double percentile, double mean, double std)
		{
			if (percentile>=1.0 || percentile<0.9) 
				throw new ArgumentOutOfRangeException( "percentile", percentile,
					// Percentile (={0}) out of range 90%<=p<100%.
					Strings.Format(	typeof(RiskStatistics), "RiskPerc", percentile) );

			InvCumulativeNormalDistribution gInverse =
				new InvCumulativeNormalDistribution(mean, std);
			// PotenzialUpSide must be a gain
			// this means that it has to be MAX(dist(percentile), 0.0)
			return Math.Max(gInverse.Value(percentile), 0.0);
		}

		/// <summary>
		/// Value-At-Risk at a given percentile.
		/// </summary>
		/// <param name="percentile"></param>
		/// <returns></returns>
		public double ValueAtRisk(double percentile) 
		{
			return ValueAtRisk( percentile,
				Mean, StandardDeviation);
		}

		/// <summary>
		/// Value at Risk (VaR).
		/// </summary>
		/// <param name="percentile">Percentile must be in range 90%-100%.</param>
		/// <param name="mean"></param>
		/// <param name="std"></param>
		/// <returns></returns>
		public static double ValueAtRisk(double percentile, double mean, double std) 
		{
			if (percentile>=1.0 || percentile<0.9) 
				throw new ArgumentOutOfRangeException( "percentile", percentile,
					// Percentile (={0}) out of range 90%<=p<100%.
					Strings.Format(	typeof(RiskStatistics), "RiskPerc", percentile) );

			InvCumulativeNormalDistribution gInverse =
				new InvCumulativeNormalDistribution(mean, std);
			// VAR must be a loss
			// this means that it has to be MIN(dist(1.0-percentile), 0.0)
			// VAR must also be a positive quantity, so -MIN(*)
			return -Math.Min( gInverse.Value(1.0-percentile), 0.0 );
		}

		/// <summary>
		/// Expected Shortfall at a given percentile.
		/// </summary>
		/// <param name="percentile"></param>
		/// <returns></returns>
		public double ExpectedShortfall(double percentile) 
		{
			return ExpectedShortfall( percentile,
				Mean, StandardDeviation);
		}

		/// <summary>
		/// Expected shortfall.
		/// </summary>
		/// <param name="percentile">Percentile must be in range 90%-100%.</param>
		/// <param name="mean"></param>
		/// <param name="std"></param>
		/// <returns></returns>
		public static double ExpectedShortfall(double percentile, double mean, double std)  
		{

			if (percentile>=1.0 || percentile<0.9) 
				throw new ArgumentOutOfRangeException( "percentile", percentile,
					// Percentile (={0}) out of range 90%<=p<100%.
					Strings.Format(	typeof(RiskStatistics), "RiskPerc", percentile) );

			InvCumulativeNormalDistribution gInverse =
				new InvCumulativeNormalDistribution(mean, std);
			double var = gInverse.Value(1.0-percentile);
			NormalDistribution g = new NormalDistribution(mean, std);
			double result = mean - std*std*g.Value(var)/(1.0-percentile);
			// expectedShortfall must be a loss
			// this means that it has to be MIN(result, 0.0)
			// expectedShortfall must also be a positive quantity, so -MIN(*)
			return -Math.Min(result, 0.0);
		}

		/// <summary>
		/// Shortfall (observations below target).
		/// </summary>
		/// <param name="target"></param>
		/// <returns></returns>
		public double Shortfall(double target) 
		{
			return Shortfall( target,
				Mean, StandardDeviation);
		}

		/// <summary>
		/// Shortfall.
		/// </summary>
		/// <param name="target"></param>
		/// <param name="mean"></param>
		/// <param name="std"></param>
		/// <returns></returns>
		public static double Shortfall(double target, double mean, double std)  
		{
			CumulativeNormalDistribution gIntegral =
				new CumulativeNormalDistribution(mean, std);
			return gIntegral.Value(target);
		}

		/// <summary>
		/// Average Shortfall (averaged shortfallness).
		/// </summary>
		/// <param name="target"></param>
		/// <returns></returns>
		double AverageShortfall(double target)  
		{
			return AverageShortfall( target,
				Mean, StandardDeviation);
		}

		/// <summary>
		/// Average shortfall.
		/// </summary>
		/// <param name="target"></param>
		/// <param name="mean"></param>
		/// <param name="std"></param>
		/// <returns></returns>
		public static double AverageShortfall(double target, double mean, double std)  
		{
			CumulativeNormalDistribution gIntegral =
				new CumulativeNormalDistribution(mean, std);
			NormalDistribution g = new NormalDistribution(mean, std);
			return ( (target-mean)*gIntegral.Value(target) + 
				std*std*g.Value(target) );
		}

		#endregion

	}
}