// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Vector.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.8  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.7  2002/10/15 12:55:41  stochastix
 *  updated license
 *
 *  Revision 1.6  2002/08/30 18:25:24  stochastix
 *  disabled operator== etc. -- have to rethink it
 *
 *  Revision 1.5  2002/08/28 15:41:25  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.4  2002/08/28 14:42:06  stochastix
 *  merging work
 *
 *  Revision 1.3  2002/08/21 17:47:55  stochastix
 *  updated
 *
 *  Revision 1.2  2002/08/20 16:44:32  stochastix
 *  Completed VML interface;
 *  added misc. Min/Max/Sum methods
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.3  2002/08/09 18:11:06  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using System.Diagnostics;
using Stochastix.Numerics;
using Stochastix.Numerics.LA;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics
{
	/// <summary>
	///   A strongly-typed vector of <see cref="double"/>s.
	///   Intended for vector algebra, see <see cref="Matrix"/>
	///   for the corresponding matrix class.
	/// </summary>
	[ RcsId("$Id: Vector.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("A strongly-typed vector of doubles.") ]
	public class Vector : 
		ICollection, IList, IEnumerable, ICloneable
	{
		// TODO: IFormatable, operator==, operator !=, IsEqual(tolerance)
		//			vec/vec div, vec*vec mult,
		//			r/o Adapter (for IList), 
		//			Data(), DataRelease(), DataUpdate()

		#region Constructors

		/// <summary>
		///		Default constructor. Initializes a new instance of the <c>Vector</c> class
		///		with the size=0.
		/// </summary>
		public Vector() : this(0)
		{
		}

		/// <summary>
		///		Initializes a new instance of the <c>Vector</c> class
		///		with the specified size. All elements are set to 0.0.
		/// </summary>
		/// <param name="size">
		///		The number of elements that the new <c>Vector</c>
		///		can store.
		///	</param>
		public Vector(int size)
		{
			if( size<0 )
				throw new ArgumentOutOfRangeException( "size", size, 
					Strings.Get( GetType(), "VecDimNeg"));
			this.count = size;
			data = new double[size];
			// this.offset = 0;
			// this.inc = 1;
		}

		/// <summary>
		///		Initializes a new instance of the <c>Vector</c> class
		///		that contains elements copied from the specified 
		///		<see cref="System.Array"/>.
		/// </summary>
		/// <param name="data">The <see cref="System.Array"/> whose elements are copied.</param>
		public Vector( double[] data) : this(data, true)
		{}

		/// <summary>
		///		Initializes a new instance of the <c>Vector</c> class
		///		that contains elements (optionally copied) from the specified 
		///		<see cref="System.Array"/>.
		/// </summary>
		/// <param name="data">The <see cref="System.Array"/> used to initialize this Vector.</param>
		/// <param name="copy">Specifies wether the array is copied.</param>
		public Vector( double[] data, bool copy)
		{
			if( data == null )
				throw new ArgumentNullException( "data", 
					Strings.Get( GetType(), "VecNull"));
			this.count = data.Length;
			if( copy ) 
			{
				this.data = new double[count];
				data.CopyTo(this.data, 0);
			} 
			else 
			{
				this.data = data;
			}
		}


		/// <summary>
		///		Initializes a new instance of the <c>Vector</c> class
		///		that contains elements copied from the specified <c>Vector</c>.
		/// </summary>
		/// <param name="other">The <c>Vector</c> whose elements are copied.</param>
		public Vector(Vector other)
		{
			if( other == null )
				throw new ArgumentNullException( "other", 
					Strings.Get( GetType(), "VecNull"));
			this.count = other.count;
			this.data = (double[])other; // create new array
			// this.offset = 0;
			// this.inc = 1;
		}

		/// <summary>
		/// Protected constructor to attach this vector to an existing 
		/// double[] array. Arguments are not validated!
		/// </summary>
		/// <remarks>
		/// This ctor allows a vector that is attached to some underlying
		/// data. The data is not copied, but only the vector's elements can
		/// be modified.
		/// </remarks>
		/// <example>
		/// The following code would create a vector that corresponds to
		/// the diagonal elements of an nxn matrix:
		/// <code>
		/// int n=5;
		/// double[] mat = new double[n*n];
		/// Vector diag = new Vector(mat, n, 0, n+1);
		/// </code>
		/// </example>
		/// <param name="data">Existing double[] array</param>
		/// <param name="count">Number of vector elements.</param>
		/// <param name="offset">Zero-based offset from array begin.</param>
		/// <param name="inc">Increment between vector elements (usually 1).</param>
		protected Vector( double[] data, int count, int offset, int inc)
		{
			this.data = data;
			this.count = count;
			this.offset = offset;
			this.inc = inc;
		}

		/// <summary>
		/// Creates the array and fills it with <paramref name="value"/>.
		/// </summary>
		public Vector(int size, double value) : this(size)
		{
			if(value!=0.0)
				// Fill(value);
				for(int i=0; i<data.Length; i++)
					data[i] = value;
		}

		/// <summary>
		/// Creates the array and fills it according to
		/// <code>a[0]=value; a[i]=a[i-1]+increment;</code>.
		/// </summary>
		/// <param name="size"></param>
		/// <param name="value"></param>
		/// <param name="increment"></param>
		public Vector(int size, double value, double increment)
			: this(size)
		{
			Fill(value, increment);
		}

		#endregion

		#region Internal data

		internal double[] data;
		private int count = 0;
		internal int offset = 0;
		internal int inc = 1;

		#endregion

		#region Fill, Reset & Copy

		public void Reset()
		{
			Fill(0.0);
		}

		public void Fill(double value)
		{
			if( inc==1 )
			{
				if( value != 0.0)
					for( int i=offset; i<count+offset; )
						data[i++] = value;
				else
					Array.Clear(data, offset, count);
			}
			else
			{
				for( int i=0; i<count;i++)
					this[i++] = value;
			}
		}

		public void Fill(double value, double increment)
		{
			if( inc==1 )
				for( int i=offset; i<count+offset; value+=increment)
					data[i++] = value;
			else
				for( int i=0; i<count; value+=increment)
					this[i++] = value;
		}

		/// <summary>
		///		Copies the entire <c>Vector</c> to another <c>Vector</c>.
		/// </summary>
		/// <param name="destination">The <c>Vector</c> to copy to.</param>
		public void CopyTo(Vector destination)
		{
			if (Count > destination.Count)
				throw new System.ArgumentException("TODO: Destination vector was not long enough.");
			BLAS.copy(count, ref data[offset], inc, 
				ref destination.data[destination.offset], destination.inc);
		}

		#endregion

		#region Vector add/sub - prefer in-place operation

		public void Add( Vector other) 
		{
			this.Add(other, 1.0);
		}

		public void Add( Vector other, double scalar) 
		{
			int n = this.Count;
			if( n != other.Count )
				throw new ArgumentException( Strings.Get(
					GetType(), "VecDimNeq"));
			BLAS.axpy(n, scalar, 
				ref other.data[other.offset], other.inc, 
				ref this.data[this.offset], this.inc);
		}

		public static Vector operator+(Vector x, Vector y) 
		{
			Vector tmp = x.Clone();
			tmp.Add(y, 1.0);
			return tmp;
		}

		// Array operator+(const Array&, double);
		// Array operator+(double, const Array&);
		// Array& operator+=(double);
		// Vector Add(double number)

		public void Subtract(Vector other)
		{
			this.Add(other, -1.0);
		}

		public static Vector operator-(Vector x, Vector y)
		{
			Vector tmp = x.Clone();
			tmp.Add(y, -1.0);
			return tmp;
		}

		// Array operator-(const Array&, double);
		// Array operator-(double, const Array&);
		// Array& operator-=(double);
		// Vector Subtract(double number)

		#endregion

/*

		public static bool operator==(Vector x, Vector y) 
		{
			int n = x.Count;
			if( n != y.Count )
				throw new ArgumentException( Strings.Get(
					"VecDimNeq", typeof(Vector).FullName));
			for(int i=0; i<n; i++)
				if( x[i]!=y[i] ) return false;
			return true;
		}

		public static bool operator!=(Vector x, Vector y) 
		{
			return !(x==y);
		}

		//		
		//		Must also override Equals(object o) and GetHashCode()!!
		//

		public override bool Equals(object obj)
		{
			// TODO: write aproper implementation!!!
			return base.Equals(obj);
		}

		public override int GetHashCode()
		{
			return data.GetHashCode();
		}

*/

		#region Vector mult/div with scalar -- prefer in-place

		public void Multiply(double scalar)
		{
			if( scalar == 1.0 ) return;
			if( scalar == 0.0 && inc == 1) 
				Array.Clear(this.data, offset, count);
			else 
				BLAS.scal(Count, scalar, ref data[offset], this.inc);
		}
		public static Vector operator*(double scalar, Vector v)
		{
			Vector tmp = v.Clone();
			tmp.Multiply(scalar);
			return tmp;
		}
		public static Vector operator*(Vector v, double scalar)
		{
			Vector tmp = v.Clone();
			tmp.Multiply(scalar);
			return tmp;
		}
		public static Vector operator/(Vector v, double scalar)
		{
			Vector tmp = v.Clone();
			tmp.Multiply(1.0/scalar);
			return tmp;
		}

		public void Divide(double scalar)
		{
			this.Multiply(1.0/scalar);
		}
		// Array operator/(double, const Array&);

		#endregion

		#region unary operators

		public static Vector operator+(Vector v)
		{
			return v.Clone();	// create new object
		}
		public static Vector operator-(Vector v)
		{
			return v * -1.0;	// creates new object
		}

		#endregion

		#region vectorial products

		public void Multiply(Vector other)
		{
			int n = this.Count;
			if( n != other.Count )
				throw new ArgumentException( Strings.Get(
					GetType(), "VecDimNeq"));
			// TODO: optimize this!!!!!
			for(int i=0; i<n; i++)
				this[i] *= other[i];
		}

		public static Vector operator*(Vector x, Vector y)
		{
			Vector tmp = x.Clone();
			tmp.Multiply(y);
			return tmp;
		}

		public static Matrix OuterProduct(Vector x, Vector y)
		{
			Matrix temp = new Matrix(x.Count, y.Count);
			BLAS.ger(temp.Rows, temp.Columns, 1.0, 
				ref x.data[x.offset], x.inc, 
				ref y.data[y.offset], y.inc, 
				ref temp.data[0], temp.ld);
			return temp;
		}
		// Vector operator*=(Vector v);
		// Vector operator/=(Vector v);
		// public static Vector operator/(Vector x, Vector y);

		#endregion

		#region vector reductions

		public static double DotProduct(double[] x, double[] y)
		{
			if( x.Length != y.Length )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			return BLAS.dot(x.Length, ref x[0], 1, ref y[0], 1);
		}
		public static double DotProduct(Vector x, Vector y)
		{
			if( x.Count != y.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			return BLAS.dot(x.Count, ref x.data[x.offset], x.inc, ref y.data[y.offset], y.inc);
		}

		public double Norm2()
		{
			return BLAS.nrm2(Count, ref data[offset], inc);
		}

		#endregion

		#region (absolute) Min, Max and Sum

		public int IndexOfMax()
		{
			return BLAS.imax(Count, ref data[offset], inc);
		}
		public double Max()
		{
			return this[IndexOfMax()];
		}
		public int IndexOfMin()
		{
			return BLAS.imin(Count, ref data[offset], inc);
		}
		public double Min()
		{
			return this[IndexOfMin()];
		}
		public double Sum()
		{
			return BLAS.sum(Count, ref data[offset], inc);
		}

		public int IndexOfAbsMax()
		{
			return BLAS.iamax(Count, ref data[offset], inc);
		}
		public double AbsMax()
		{
			return this[IndexOfAbsMax()];
		}
		public int IndexOfAbsMin()
		{
			return BLAS.iamin(Count, ref data[offset], inc);
		}
		public double AbsMin()
		{
			return this[IndexOfAbsMin()];
		}
		public double AbsSum()
		{
			return BLAS.asum(Count, ref data[offset], inc);
		}

		#endregion


		#region vector math functions (tb replaced by vml)

		/// <summary>
		/// Calculate absolute values of the vector elements.
		/// </summary>
		public void Abs()
		{
			for(int i=0; i<Count; i++)
				this[i] = Math.Abs(this[i]);
		}
		public static Vector Abs(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			for(int i=0; i<n; i++)
				temp[i] = Math.Abs(a[i]);
			return temp;
		}
		public static void Abs(double[] a)
		{
			int i=a.GetLowerBound(0);
			for(int last=a.GetUpperBound(0); i<last; i++)
				a[i] = Math.Abs(a[i]);
		}

		/*
				
		public static Vector Abs(Vector v)
		{
			Vector temp = v.Clone();
			unsafe 
			{
				fixed ( double* tp = &temp.data[0] )
					for( double* tpp=tp, end=tp+temp.data.Length; tpp<end; tpp++ )
						*tpp = (*tpp >= 0) ? *tpp : - *tpp;
			}
			return temp;
		}
		
		public static Vector Sqrt(Vector v)
		{
			Vector temp = v.Clone();
			unsafe 
			{
				fixed ( double* tp = &temp.data[0] )
					for( double* tpp = tp, end = tp+temp.data.Length; tpp<end; tpp++ )
						*tpp = Math.Sqrt(*tpp);
			}
			return temp;
		}

		public static Vector Log(Vector v)
		{
			Vector temp = v.Clone();
			unsafe 
			{
				fixed ( double* tp = &temp.data[0] )
					for( double* tpp = tp, end = tp+temp.data.Length; tpp<end; tpp++ )
						*tpp = Math.Log(*tpp);
			}
			return temp;
		}

		public static Vector Exp(Vector v)
		{
			Vector temp = v.Clone();
			unsafe 
			{
				fixed ( double* tp = &temp.data[0] )
					for( double* tpp = tp, end = tp+temp.data.Length; tpp<end; tpp++ )
						*tpp = Math.Exp( *tpp );
			}
			return temp;
		}
*/
		#endregion

		#region VML Inv / Roots / Pow / Div

		/// <summary>
		/// Performs element by element inversion ( 1.0 / a) of the vector.
		/// </summary>
		public void Inv()
		{
			VML.Inv(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Inv(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Inv(n, ref a.data[a.offset], a.inc, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Inv(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Inv(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes square root ( a**0.5 ) of vector elements.
		/// </summary>
		public void Sqrt()
		{
			VML.Sqrt(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Sqrt(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Sqrt(n, ref a.data[a.offset], a.inc, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Sqrt(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Sqrt(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inversion square root ( 1.0 / a**0.5 ) of vector elements.
		/// </summary>
		public void InvSqrt()
		{
			VML.InvSqrt(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector InvSqrt(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.InvSqrt(n, ref a.data[a.offset], a.inc, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void InvSqrt(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.InvSqrt(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes cube root ( a**(1/3) ) of vector elements.
		/// </summary>
		public void Cbrt()
		{
			VML.Cbrt(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Cbrt(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Cbrt(n, ref a.data[a.offset], a.inc, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Cbrt(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Cbrt(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inversion cube root ( 1.0 / a**(1/3) ) of vector elements.
		/// </summary>
		public void InvCbrt()
		{
			VML.InvCbrt(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector InvCbrt(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.InvCbrt(n, ref a.data[a.offset], a.inc, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void InvCbrt(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.InvCbrt(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes power ( a**value ) of vector elements.
		/// </summary>
		public void Pow(double value)
		{
			VML.vdPowNum(count, ref data[offset], inc, value, ref data[offset], inc);
		}
		public static Vector Pow(Vector a, double value)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.vdPowNum(n, ref a.data[a.offset], a.inc, value, ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Pow(double[] a, double value)
		{
			int low = a.GetLowerBound(0);
			VML.vdPowNum(a.Length, ref a[low], 1, value, ref a[low], 1);
		}

		/// <summary>
		/// Computes power ( a**b ) of vector elements.
		/// </summary>
		public void Pow(Vector b)
		{
			if( this.Count != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			VML.Pow(count, ref data[offset], inc, ref b.data[b.offset], b.inc,
				ref data[offset], inc);
		}
		public static Vector Pow(Vector a, Vector b)
		{
			int n=a.Count;
			if( n != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			Vector temp = new Vector(n);
			VML.Pow(n, ref a.data[a.offset], a.inc, ref b.data[b.offset], b.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}

		public static void Pow(double[] a, double[] b)
		{
			if( a.Length != b.Length )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			int low = a.GetLowerBound(0);
			VML.Pow(a.Length, ref a[low], 1, ref b[b.GetLowerBound(0)], 1,
				ref a[low], 1);
		}

		/// <summary>
		/// Computes division ( a/b ) of vector elements.
		/// </summary>
		public void Div(Vector b)
		{
			if( this.Count != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			VML.Div(count, ref data[offset], inc, ref b.data[b.offset], b.inc,
				ref data[offset], inc);
		}
		public static Vector Div(Vector a, Vector b)
		{
			int n=a.Count;
			if( n != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			Vector temp = new Vector(n);
			VML.Div(n, ref a.data[a.offset], a.inc, 
				ref b.data[b.offset], b.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Div(double[] a, double[] b)
		{
			if( a.Length != b.Length )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			int low = a.GetLowerBound(0);
			VML.Div(a.Length, ref a[low], 1, ref b[b.GetLowerBound(0)], 1,
				ref a[low], 1);
		}

		#endregion

		#region VML Exp / Log / Log10

		/// <summary>
		/// Computes exponent ( e**a ) of vector elements.
		/// </summary>
		public void Exp()
		{
			VML.Exp(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Exp(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Exp(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Exp(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Exp(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes logarithm ( ln a ) of vector elements.
		/// </summary>
		public void Ln()
		{
			VML.Ln(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Ln(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Ln(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Ln(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Ln(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes decimal logarithm ( lg a ) of vector elements.
		/// </summary>
		public void Log10()
		{
			VML.Log10(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Log10(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Log10(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Log10(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Log10(a.Length, ref a[low], 1, ref a[low], 1);
		}

		#endregion

		#region (VML Gamma)
		
		/// <summary>
		/// Computes gamma function on vector elements.
		/// </summary>
		public void Gamma()
		{
			VML.Gamma(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Gamma(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Gamma(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Gamma(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Gamma(a.Length, ref a[low], 1, ref a[low], 1);
		}

		#endregion

		#region VML Sin / Asin / Cos / Acos / Tan / Atan / Atan2

		/// <summary>
		/// Computes sine (sin a) of vector elements.
		/// </summary>
		public void Sin()
		{
			VML.Sin(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Sin(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Sin(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Sin(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Sin(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse sine (arcsin a) of vector elements.
		/// </summary>
		public void Asin()
		{
			VML.Asin(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Asin(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Asin(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Asin(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Asin(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes cosine (cos a) of vector elements.
		/// </summary>
		public void Cos()
		{
			VML.Cos(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Cos(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Cos(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Cos(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Cos(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse cosine (arccos a) of vector elements.
		/// </summary>
		public void Acos()
		{
			VML.Acos(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Acos(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Acos(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Acos(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Acos(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes tangent (tan a) of vector elements.
		/// </summary>
		public void Tan()
		{
			VML.Tan(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Tan(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Tan(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Tan(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Tan(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse tangent (arctan a) of vector elements.
		/// </summary>
		public void Atan()
		{
			VML.Atan(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Atan(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Atan(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Atan(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Atan(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse tangent of the quotient (arctan a / b) of vector elements.
		/// </summary>
		public void Atan2(Vector b)
		{
			if( this.Count != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			VML.Atan2(count, ref data[offset], inc, ref b.data[b.offset], b.inc,
				ref data[offset], inc);
		}
		public static Vector Atan2(Vector a, Vector b)
		{
			int n=a.Count;
			if( n != b.Count )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			Vector temp = new Vector(n);
			VML.Atan2(n, ref a.data[a.offset], a.inc, 
				ref b.data[b.offset], b.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Atan2(double[] a, double[] b)
		{
			if( a.Length != b.Length )
				throw new ArgumentException( Strings.Get( typeof(Vector), "VecDimNeq"));
			int low = a.GetLowerBound(0);
			VML.Atan2(a.Length, ref a[low], 1, ref b[b.GetLowerBound(0)], 1,
				ref a[low], 1);
		}

		#endregion

		#region VML Sinh / Asinh / Cosh / Acosh / Tanh / Atanh

		/// <summary>
		/// Computes hyperbolic sine (sinh a) of vector elements.
		/// </summary>
		public void Sinh()
		{
			VML.Sinh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Sinh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Sinh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Sinh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Sinh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse hyperbolic sine (arcsinh a) of vector elements.
		/// </summary>
		public void Asinh()
		{
			VML.Asinh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Asinh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Asinh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Asinh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Asinh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes hyperbolic cosine (cosh a) of vector elements.
		/// </summary>
		public void Cosh()
		{
			VML.Cosh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Cosh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Cosh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Cosh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Cosh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse hyperbolic cosine (arccosh a) of vector elements.
		/// </summary>
		public void Acosh()
		{
			VML.Acosh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Acosh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Acosh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Acosh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Acosh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes hyperbolic tangent (tanh a) of vector elements.
		/// </summary>
		public void Tanh()
		{
			VML.Tanh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Tanh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Tanh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Tanh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Tanh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		/// <summary>
		/// Computes inverse hyperbolic tangent (arctanh a) of vector elements.
		/// </summary>
		public void Atanh()
		{
			VML.Atanh(count, ref data[offset], inc, ref data[offset], inc);
		}
		public static Vector Atanh(Vector a)
		{
			int n=a.Count;
			Vector temp = new Vector(n);
			VML.Atanh(n, ref a.data[a.offset], a.inc, 
				ref temp.data[temp.offset], temp.inc);
			return temp;
		}
		public static void Atanh(double[] a)
		{
			int low = a.GetLowerBound(0);
			VML.Atanh(a.Length, ref a[low], 1, ref a[low], 1);
		}

		#endregion

		// TODO: make this static Vector / public void  
		// like the other functions that return a new vector???

		/// <summary>
		/// Apply the given unary function to all elements.
		/// </summary>
		/// <param name="f">The unary function implementing <see cref="IUnaryFunction"/>.</param>
		/// <returns>The resulting vector.</returns>
		/// <seealso cref="IUnaryFunction"/>
		/// <seealso cref="UnaryFunction">UnaryFunction delegate</seealso>
		/// <seealso cref="Functional"/>
		public Vector Apply(IUnaryFunction f) 
		{
			return Apply(new UnaryFunction(f.Value));
		}

		/// <summary>
		/// Apply the given unary function to all elements.
		/// </summary>
		/// The <see cref="UnaryFunction"/> delegate applied to all
		/// values of x.
		/// <returns>The resulting vector.</returns>
		/// <seealso cref="IUnaryFunction"/>
		/// <seealso cref="UnaryFunction">UnaryFunction delegate</seealso>
		/// <seealso cref="Functional"/>
		public Vector Apply(UnaryFunction f) 
		{
			Vector temp = this.Clone();
			for( int i=0; i<Count; i++)
				temp.data[i] = f(this[i]);
			return temp;
		}

		/// <summary>
		/// Finds the position of the first element in an ordered range that has a 
		/// value that is greater than a specified value.
		/// </summary>
		/// <param name="val">
		/// The value in the ordered range that needs to be exceeded by the value of the
		/// element addressed by the index returned.
		/// </param>
		/// <returns>
		/// The index addressing the position of the first element in an ordered range
		/// that has a value that is greater than the specified value.
		/// </returns>
		// TODO: implement using binary search
		public int UpperBound(double val)
		{
			for(int i=0; i<Count; i++)
				if( this[i]>val )
					return i;
			return Count;
		}

		/// <summary>
		/// Finds the position where the first element in an ordered range is or would be
		/// if it had a value that is less than or equivalent to a specified value.
		/// </summary>
		/// <param name="val">
		/// The value whose first position or possible first position is being searched for
		/// in the ordered range. 
		/// </param>
		/// <returns>
		/// The index addressing the position where the first element in an ordered range
		/// is or would be if it had a value that is less than or equivalent to the
		/// specified value.
		/// </returns>
		// TODO: implement using binary search
		public int LowerBound(double val)
		{
			for(int i=Count; i!=0; )
				if( this[--i]<=val )
					return i;
			return 0;
		}



		#region type casts

		public static explicit operator double[](Vector v)
		{
			if( v.Count == v.data.Length ) 
				return (double[]) v.data.Clone();
			else
			{
				double[] data = new double[v.Count];
				v.CopyTo(data, 0);
				return data;
			}
		}

		#endregion

		#region Operations (type-safe ICollection)

		/// <summary>
		///		Gets the number of elements actually contained in the <c>Vector</c>.
		/// </summary>
		public virtual int Count
		{
			get { return count; }
			set 
			{ 
				if( value<0 )
					throw new ArgumentOutOfRangeException();
				count = value;
				offset = 0;
				inc = 1;
				data = new double[value];
			}
		}

		/// <summary>
		///		Copies the entire <c>Vector</c> to a one-dimensional
		///		<see cref="double"/> array.
		/// </summary>
		/// <param name="array">The one-dimensional <see cref="double"/> array to copy to.</param>
		public virtual void CopyTo(double[] array)
		{
			this.CopyTo(array, 0);
		}

		/// <summary>
		///		Copies the entire <c>Vector</c> to a one-dimensional
		///		<see cref="double"/> array, starting at the specified 
		///		index of the target array.
		/// </summary>
		/// <param name="array">The one-dimensional <see cref="double"/> array to copy to.</param>
		/// <param name="start">The zero-based index in <paramref name="array"/> at which copying begins.</param>
		public virtual void CopyTo(double[] array, int start)
		{
			if (Count > array.GetUpperBound(0) + 1 - start)
				throw new System.ArgumentException( Strings.Get(
					GetType(), "TODO: Destination array was not long enough."));
		//	if( inc == 1)
		//		Array.Copy(data, offset, array, start, count);
		//	else
				BLAS.copy(count, ref data[offset], inc, ref array[start], 1);
		}

		/// <summary>
		///		Gets a value indicating whether access to the collection is synchronized (thread-safe).
		/// </summary>
		/// <returns>true if access to the ICollection is synchronized (thread-safe); otherwise, false.</returns>
		public virtual bool IsSynchronized
		{
			get { return this.data.IsSynchronized; }
		}

		/// <summary>
		///		Gets an object that can be used to synchronize access to the collection.
		/// </summary>
		public virtual object SyncRoot
		{
			get { return this.data.SyncRoot; }
		}
		#endregion

		#region Operations (type-safe IList)

		/// <summary>
		///		Gets or sets the <see cref="double"/> at the specified index.
		/// </summary>
		/// <param name="index">The zero-based index of the element to get or set.</param>
		/// <exception cref="ArgumentOutOfRangeException">
		///		<para><paramref name="index"/> is less than zero</para>
		///		<para>-or-</para>
		///		<para><paramref name="index"/> is equal to or greater than <see cref="Vector.Count"/>.</para>
		/// </exception>
		public virtual double this[int index]
		{
			get 
			{ 
#if DEBUG
				ValidateIndex(index); // throws
#endif
				return data[offset+index*inc]; 
			}
			set 
            { 
#if DEBUG
				ValidateIndex(index); // throws
#endif
				data[offset+index*inc] = value;
				// ++m_version; 
			}
		}

		/// <exception cref="ArgumentOutOfRangeException">
		///		<para><paramref name="index"/> is less than zero</para>
		///		<para>-or-</para>
		///		<para><paramref name="index"/> is equal to or greater than <see cref="Vector.Count"/>.</para>
		/// </exception>
		[ Conditional("DEBUG") ]
		private void ValidateIndex(int i)
		{
			if (i < 0 || i >= count)
				throw new System.ArgumentOutOfRangeException( "i",  i, Strings.Get(
					GetType(), "TODO: Index was out of range.") );
		}

		/// <summary>
		///		Adds a <see cref="double"/> to the end of the <c>Vector</c>.
		///		This method is not supported on fixed-size lists.
		/// </summary>
		/// <exception cref="NotSupportedException">
		///		This method is not supported on fixed-size lists.
		/// </exception>
		/// <param name="item">The <see cref="double"/> to be added to the end of the <c>Vector</c>.</param>
		/// <returns>The index at which the value has been added.</returns>
		public virtual int Add(double item)
		{
			throw new NotSupportedException();
		}
		
		/// <summary>
		///		Removes all elements from the <c>Vector</c>.
		///		This method is not supported on fixed-size lists.
		/// </summary>
		/// <exception cref="NotSupportedException">
		///		This method is not supported on fixed-size lists.
		/// </exception>
		public virtual void Clear()
		{
			throw new NotSupportedException();
		}
		
		/// <summary>
		///		Creates a shallow copy of the <see cref="Vector"/>.
		/// </summary>
		public virtual Vector Clone()
		{
			return new Vector(this);
		}

		/// <summary>
		///		Determines whether a given <see cref="double"/> is in the <c>Vector</c>.
		/// </summary>
		/// <param name="item">The <see cref="double"/> to check for.</param>
		/// <returns><c>true</c> if <paramref name="item"/> is found in the <c>Vector</c>; otherwise, <c>false</c>.</returns>
		public virtual bool Contains(double item)
		{
			return IndexOf(item) >= 0;
		}

		/// <summary>
		///		Returns the zero-based index of the first occurrence of a <see cref="double"/>
		///		in the <c>Vector</c>.
		/// </summary>
		/// <param name="item">The <see cref="double"/> to locate in the <c>Vector</c>.</param>
		/// <returns>
		///		The zero-based index of the first occurrence of <paramref name="item"/> 
		///		in the entire <c>Vector</c>, if found; otherwise, -1.
		///	</returns>
		public virtual int IndexOf(double item)
		{
			for (int i=0; i != count; ++i)
				if (this[i].Equals(item))
					return i;
			return -1;
		}

		/// <summary>
		///		Inserts an element into the <c>Vector</c> at the specified index.
		///		This method is not supported on fixed-size lists.
		/// </summary>
		/// <exception cref="NotSupportedException">
		///		This method is not supported on fixed-size lists.
		/// </exception>
		/// <param name="index">The zero-based index at which <paramref name="item"/> should be inserted.</param>
		/// <param name="item">The <see cref="double"/> to insert.</param>
		public virtual void Insert(int index, double item)
		{
			throw new NotSupportedException();
		}

		/// <summary>
		///		Removes the first occurrence of a specific <see cref="double"/> from the <c>Vector</c>.
		///		This method is not supported on fixed-size lists.
		/// </summary>
		/// <exception cref="NotSupportedException">
		///		This method is not supported on fixed-size lists.
		/// </exception>
		/// <param name="item">The <see cref="double"/> to remove from the <c>Vector</c>.</param>
		public virtual void Remove(double item)
		{		   
			throw new NotSupportedException();
		}

		/// <summary>
		///		Removes the element at the specified index of the <c>Vector</c>.
		///		This method is not supported on fixed-size lists.
		/// </summary>
		/// <exception cref="NotSupportedException">
		///		This method is not supported on fixed-size lists.
		/// </exception>
		/// <param name="index">The zero-based index of the element to remove.</param>
		public virtual void RemoveAt(int index)
		{
			throw new NotSupportedException();
		}

		/// <summary>
		///		Gets a value indicating whether the collection has a fixed size.
		/// </summary>
		/// <value>true if the collection has a fixed size; otherwise, false</value>
		public virtual bool IsFixedSize
		{
			get { return true; }
		}

		/// <summary>
		///		gets a value indicating whether the IList is read-only.
		/// </summary>
		/// <value>true if the collection is read-only; otherwise, false</value>
		public virtual bool IsReadOnly
		{
			get { return false; }
		}
		#endregion

		#region IVectorEnumerator Interface
		/// <summary>
		///		Supports type-safe iteration over a <see cref="Vector"/>.
		/// </summary>
		public interface IVectorEnumerator
		{
			/// <summary>
			///		Gets the current element in the collection.
			/// </summary>
			double Current {get;}

			/// <summary>
			///		Advances the enumerator to the next element in the collection.
			/// </summary>
			/// <exception cref="InvalidOperationException">
			///		The collection was modified after the enumerator was created.
			/// </exception>
			/// <returns>
			///		<c>true</c> if the enumerator was successfully advanced to the next element; 
			///		<c>false</c> if the enumerator has passed the end of the collection.
			/// </returns>
			bool MoveNext();

			/// <summary>
			///		Sets the enumerator to its initial position, before the first element in the collection.
			/// </summary>
			void Reset();
		}
        #endregion

		#region Operations (type-safe IEnumerable)
		
		/// <summary>
		///		Returns an enumerator that can iterate through the <c>Vector</c>.
		/// </summary>
		/// <returns>An <see cref="Enumerator"/> for the entire <c>Vector</c>.</returns>
		public virtual IVectorEnumerator GetEnumerator()
		{
			return new Enumerator(this);
		}

		#endregion

		#region Nested enumerator class

		/// <summary>
		///		Supports simple iteration over a <see cref="Vector"/>.
		/// </summary>
		private class Enumerator : IEnumerator, IVectorEnumerator
		{
			private Vector v;
			private int i = -1;
			
			internal Enumerator(Vector tc)
			{
				v = tc;
			}

			public double Current
			{
				get { return v[i];  }
			}

			/// <summary>
			///		Advances the enumerator to the next element in the collection.
			/// </summary>
			/// <returns>
			///		<c>true</c> if the enumerator was successfully advanced to the next element; 
			///		<c>false</c> if the enumerator has passed the end of the collection.
			/// </returns>
			public bool MoveNext()
			{
				return (++i < v.Count);
			}

			/// <summary>
			///		Sets the enumerator to its initial position, before the first element in the collection.
			/// </summary>
			public void Reset()
			{
				i = -1;
			}
			
			object IEnumerator.Current
			{
				get { return (object)(this.Current); }
			}
			
		}
        #endregion
        
//		#region Implementation (IDisposable)
//
//		public void Dispose()
//		{
//			data = null;
//			count = 0;
//		}
//
//		#endregion
		
		#region Implementation (ICollection)

		void ICollection.CopyTo(Array array, int start)
		{
			this.CopyTo((double[])array, start);
		}

		#endregion

		#region Implementation (IList)

		object IList.this[int i]
		{
			get { return (object)this[i]; }
			set { this[i] = (double)value; }
		}

		int IList.Add(object x)
		{
			return this.Add((double)x);
		}

		bool IList.Contains(object x)
		{
			return this.Contains((double)x);
		}

		int IList.IndexOf(object x)
		{
			return this.IndexOf((double)x);
		}

		void IList.Insert(int pos, object x)
		{
			this.Insert(pos, (double)x);
		}

		void IList.Remove(object x)
		{
			this.Remove((double)x);
		}

		void IList.RemoveAt(int pos)
		{
			this.RemoveAt(pos);
		}

		#endregion

		#region Implementation (IEnumerable)

		IEnumerator IEnumerable.GetEnumerator()
		{
			return (IEnumerator)(this.GetEnumerator());
		}

		#endregion

		#region Implementation (ICloneable)

		object ICloneable.Clone()
		{
			return this.Clone();
		}

		#endregion

	}
}
