// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Coupon.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:26  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Business.Calendars;
using Stochastix.Utilities;


namespace Stochastix.QuantLib.CashFlows
{
	/// <summary>
	/// Coupon accruing over a fixed period.
	/// </summary>
	/// <remarks>
	/// This class implements part of the CashFlow interface but it
	/// is still abstract and provides derived classes with methods
	/// for accrual period calculations.
	/// </remarks>
	[ RcsId("$Id: Coupon.cs,v 1.1.1.1 2003/02/01 19:01:26 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Coupon : Observable, ICoupon, ICashFlow
	{
		/// <summary>
		/// Construct an abstract coupon payment. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="accrualStartDate"></param>
		/// <param name="accrualEndDate"></param>
		public Coupon(
			double nominal, DateTime paymentDate, 
			DateTime accrualStartDate, DateTime accrualEndDate)
			: this(nominal, paymentDate, accrualStartDate, accrualEndDate,
					CalendarBase.NullDate, CalendarBase.NullDate)
		{}
		
		/// <summary>
		/// Construct an abstract coupon payment. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="accrualStartDate"></param>
		/// <param name="accrualEndDate"></param>
		/// <param name="refPeriodStart"></param>
		/// <param name="refPeriodEnd"></param>
		public Coupon(
			double nominal, DateTime paymentDate, 
			DateTime accrualStartDate, DateTime accrualEndDate,
			DateTime refPeriodStart, DateTime refPeriodEnd )
		{
			this.nominal = nominal;
			// normalize dates
			this.paymentDate = paymentDate.Date;
			this.accrualStartDate = accrualStartDate.Date;
			this.accrualEndDate = accrualEndDate.Date;
			this.refPeriodStart = refPeriodStart.Date;
			this.refPeriodEnd = refPeriodEnd.Date;
		}
		
		private double nominal;
		private DateTime paymentDate, 
			accrualStartDate, accrualEndDate;
		protected DateTime // these currently have no accessors
			refPeriodStart, refPeriodEnd;

#region Partial Implementation of ICashFlow

		override public string ToString() 
		{
			return string.Format(
				"{0:d}: {1:f} for nominal={2:n}; accrual period {3:d}-{4:d}",
				new Object[] {
					Date, Amount, Nominal, AccrualStartDate, accrualEndDate
				 }
			);
		}	

		abstract public double Amount
		{
			get;
		}

		public System.DateTime Date
		{
			get	{ return paymentDate; }
		}

#endregion

#region Partial Implementation of ICoupon

		public double Nominal
		{
			get { return nominal; }
		}

		public System.DateTime AccrualStartDate
		{
			get { return accrualStartDate; }
		}

		public System.DateTime AccrualEndDate
		{
			get { return accrualEndDate; }
		}

		public double AccrualPeriod
		{
			get
			{
				return DayCounter.YearFraction(
					accrualStartDate, accrualEndDate,
					refPeriodStart, refPeriodEnd
				);
			}
		}

		public int AccrualDays
		{
			get
			{
				return DayCounter.DayCount(
					accrualStartDate, accrualEndDate
				);
			}
		}

		abstract public IDayCounter DayCounter
		{
			get;
		}

		abstract public double AccruedAmount(DateTime date);

#endregion


	}

}
