// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Scheduler.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/31 18:49:39  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Diagnostics;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Business;
using Stochastix.Business.DayCounters;
using Stochastix.QuantLib.Indexes;

namespace Stochastix.QuantLib.CashFlows
{
	/// <summary>
	/// Scheduler to calculate repetitve payment dates.
	/// </summary>
	/// <remarks>
	/// TODO: implement an interface for COM export.
	/// </remarks>
	[ RcsId("$Id: Scheduler.cs,v 1.1.1.1 2003/02/01 19:01:27 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Scheduler to calculate repetitve payment dates.") ]
	[ ClassInterface(ClassInterfaceType.AutoDual) ]
	public class Scheduler : ICollection
	{
		// Since we store DateTime value types, an implementation
		// on an Array DateTime[] might be more efficient.
		// (reason: Boxing/Unboxing of ValueTypes is inefficient)
		private DateTime[] dt = {};

		public Scheduler()
		{
			firstIsRegular = true;
			lastIsRegular = true;
		}

		public Scheduler(
			IBusinessCalendar calendar,
			DateTime startDate, DateTime endDate,
			int frequency, RollingConvention rollingConvention,
			bool isAdjusted, DateTime stubDate)
		{
			Run(calendar, startDate, endDate, frequency,
                rollingConvention, isAdjusted, stubDate);
		}

		public void Run(
			IBusinessCalendar calendar,
			DateTime startDate, DateTime endDate,
			int frequency, RollingConvention rollingConvention,
			bool isAdjusted, DateTime stubDate)
		{
			firstIsRegular = true;
			lastIsRegular = true;
			dt = new DateTime[] {};

			int guess = 1 + (int)Math.Floor( // guess needed capacity
				frequency*Actual365.Instance.YearFraction(startDate, endDate) );
			ArrayList dates = new ArrayList(guess); // start+stub+(int)cast

			// normalize
			startDate = startDate.Date;
			endDate = endDate.Date;
			stubDate = stubDate.Date;

			// sanity checks
			// QL_REQUIRE(startDate_ != Date(), "null start date");
			// QL_REQUIRE(endDate_ != Date(),   "null end date");
			if( startDate >= endDate )
				throw new ArgumentException( String.Format(
					"start date ({0}) later than end date ({1})",
					startDate.ToShortDateString(),
					endDate.ToShortDateString()) );

			if (stubDate != startDate) 
			{
				if( stubDate < startDate )
					throw new ArgumentException( String.Format(
						"stub date ({0}) before start date ({1})",
						stubDate.ToShortDateString(),
						endDate.ToShortDateString()) );
/* Does it make any sense to generate some kind of warning
 * iff we use Following and meet this condition?
				QL_REQUIRE(!calendar_.isHoliday(stubDate_) ||
						!isEndOfMonth(stubDate_),
					"stub date (" +
						DateFormatter::toString(stubDate_) +
						") is holiday and end of month for " +
						calendar_.name() + " calendar");
*/
			} else {
/* Does it make any sense to generate some kind of warning
 * iff we use Following and meet this condition?
				QL_REQUIRE(!calendar_.isHoliday(startDate_) ||
						!isEndOfMonth(startDate_),
					"start date (" +
						DateFormatter::toString(startDate_) +
						") is holiday and end of month for " +
						calendar_.name() + " calendar");
 */
			}

			if( 12%frequency != 0 )
				throw new ArgumentException( String.Format(
					"frequency ({0} per year) does not correspond to a whole number of months",
					frequency) );

			// calculations
			DateTime seed = startDate;
			DateTime last = isAdjusted ?
				calendar.Roll(endDate,rollingConvention) : endDate;

			// add start date
			dates.Add(startDate);

			// add stub date if given
			if (stubDate != startDate) // CHANGED: I made this different from QL
			{
				seed = stubDate;
				dates.Add( isAdjusted ?
					calendar.Roll(stubDate, rollingConvention) : stubDate
					// CHANGED: added the above rollingConvention different from QL
				);
				firstIsRegular = false;
			}

			// add subsequent dates
			int periods = 1;
			int months = 12/frequency;
			while (true) {
				DateTime temp = seed.AddMonths(periods*months);
				if (isAdjusted)
					temp = calendar.Roll(temp,rollingConvention);
				dates.Add(temp);
				// check exit condition
				if (temp >= last)
					break;
				else
					periods++;
			}

			// possibly correct last inserted date
			if ((DateTime)dates[dates.Count-1] > last) {
				dates[dates.Count-1] = last;
				lastIsRegular = false;
			}

			// possibly collapse last two dates
			int N = dates.Count;
			          // Jens: the Rolling next is only neccesary if not already adjusted
			if (calendar.Roll((DateTime)dates[N-2],rollingConvention) ==
				calendar.Roll((DateTime)dates[N-1],rollingConvention)) {
					dates.RemoveAt(N-2);
					// if not on a list we do d[N-2] = d[N-1], d.pop()
					lastIsRegular = true;
			}

			Debug.WriteLineIf( guess != dates.Count, String.Format( 
				"Scheduler.Run(): Capacity guess was {0}, dates.Count is {1}",
				guess, dates.Count ));
			dt = new DateTime[dates.Count];
			dates.CopyTo(dt);

			// done

		}

		private bool firstIsRegular;
		private bool lastIsRegular;

		public bool IsRegular(int i)
		{
			if( Count == 0 ) 
				throw new ArgumentOutOfRangeException( "i", i, 
					"Maybe the Scheduler has not been initialized?"
				);

			if (i == 1)
				return firstIsRegular;	// Jens: Different from QL
			else if (i == Count-1)
				return lastIsRegular;
			else
				return true;
		}

/*
		private bool IsEndOfMonth(DateTime date)
		{
			return (date.Month != calendar.Advance(
				date, 1, TimeUnit.Days, 
				RollingConvention.Following).Month );
		}
*/
		public DateTime this[int i]
		{
			get	{ return dt[i]; }
		}

		#region Implementation of ICollection

		public int Count
		{ 
			get { return dt.Length; }
		}

		public void CopyTo(System.Array array, int index)
		{
			dt.CopyTo(array, index);
		}

		public bool IsSynchronized
		{
			get	{ return dt.IsSynchronized; }
		}

		public object SyncRoot
		{
			get { return dt.SyncRoot; }
		}

		#endregion

		#region Implementation of IEnumerable

		public System.Collections.IEnumerator GetEnumerator()
		{
			return dt.GetEnumerator();
		}

		#endregion
	
		#region Implementation of static ...CouponVector functions

        // std::vector<Handle<CashFlow> > 
		static public ICollection FixedRateCouponVector(
			// const std::vector<double>& nominals,
			// const std::vector<Rate>& couponRates,
			double[] nominals, double[] couponRates,
			DateTime startDate, DateTime endDate,
			int frequency, IBusinessCalendar calendar,
			RollingConvention rollingConvention, bool isAdjusted,
			IDayCounter dayCount, IDayCounter firstPeriodDayCount,
			DateTime stubDate) 
		{
			if( couponRates.Length == 0 )
				throw new ArgumentException(
					"unspecified coupon rates", "couponRates"
				);
			if( nominals.Length == 0 )
				throw new ArgumentException(
					"unspecified nominals", "nominals"
				);

            Scheduler scheduler = new Scheduler(
				calendar, startDate, endDate, frequency,
                rollingConvention, isAdjusted, stubDate
			);

			// std::vector<Handle<CashFlow> > leg;
			ArrayList leg = new ArrayList(scheduler.Count-1);  // capacity

            // first period might be short or long
            DateTime start = scheduler[0];
			DateTime end = scheduler[1];
            DateTime paymentDate = calendar.Roll(end,rollingConvention);
            double rate = couponRates[0];
            double nominal = nominals[0];

            if (scheduler.IsRegular(1)) 
			{
				// TODO: check this out, Equals(), ref cmp, val cmp, etc.
				if( dayCount != firstPeriodDayCount )
					throw new ArgumentException(
						"regular first coupon does not allow a first period day count",
						"firstPeriodDayCount"
					);
                leg.Add( new FixedRateCoupon(
					nominal, paymentDate, rate, dayCount,
                    start, end, start, end)
				);

			} else {

				DateTime reference = calendar.Calendar.AddMonths(end, -12/frequency);
									// CHANGED: was end.plusMonths(-12/frequency);
                if (isAdjusted)
                    reference = calendar.Roll(reference, rollingConvention);
                leg.Add( new FixedRateCoupon(
					nominal, paymentDate, rate, 
                    firstPeriodDayCount, start, end, reference, end)
				);

            }

            // regular periods
            for(int i=2; i<scheduler.Count-1; i++) {
                start = end; 
				end = scheduler[i];
                paymentDate = calendar.Roll(end, rollingConvention);
                if ((i-1) < couponRates.Length)
                    rate = couponRates[i-1];
                else
                    rate = couponRates[couponRates.Length-1];
                if ((i-1) < nominals.Length)
                    nominal = nominals[i-1];
                else
                    nominal = nominals[nominals.Length-1];
                leg.Add( new FixedRateCoupon(
					nominal, paymentDate, rate, dayCount, 
                    start, end, start, end)
				);
            }

            if (scheduler.Count > 2) {
                // last period might be short or long
                int N = scheduler.Count;
                start = end; 
				end = scheduler[N-1];
                paymentDate = calendar.Roll(end, rollingConvention);
                if ((N-2) < couponRates.Length)
                    rate = couponRates[N-2];
                else
                    rate = couponRates[couponRates.Length-1];
                if ((N-2) < nominals.Length)
                    nominal = nominals[N-2];
                else
                    nominal = nominals[nominals.Length-1];
                if (scheduler.IsRegular(N-1)) {
                    leg.Add( new FixedRateCoupon(
						nominal, paymentDate, rate, dayCount, 
                        start, end, start, end)
					);
                } else {
					DateTime reference = calendar.Calendar.AddMonths(start, 12/frequency);
									// CHANGED: was start.plusMonths(12/frequency);
                    if (isAdjusted)
                        reference = calendar.Roll(reference, rollingConvention);
                    leg.Add( new FixedRateCoupon(
						nominal, paymentDate, rate, dayCount, 
                        start, end, start, reference)
					);
                }
            }
            return leg;
        }

		static public ICollection FloatingRateCouponVector(
			double[] nominals, double[] spreads,
			DateTime startDate, DateTime endDate,
			int frequency, IBusinessCalendar calendar,
			RollingConvention rollingConvention,
			IXibor index, int fixingDays,
			DateTime stubDate) 
		{
			if( nominals.Length == 0 )
				throw new ArgumentException(
					"unspecified nominals", "nominals"
					);

			Scheduler scheduler = new Scheduler(
				calendar, startDate, endDate, frequency,
				rollingConvention, true, stubDate
				);

			ArrayList leg = new ArrayList(scheduler.Count-1);  // capacity

			// first period might be short or long
			DateTime start = scheduler[0];
			DateTime end = scheduler[1];
			DateTime paymentDate = calendar.Roll(end,rollingConvention);

			double spread;
            if (spreads.Length > 0)
                spread = spreads[0];
            else
                spread = 0.0;
			double nominal = nominals[0];

			if (scheduler.IsRegular(1)) 
			{
				leg.Add( new FloatingRateCoupon(
                    nominal, paymentDate, index, fixingDays, spread, 
					start, end, start, end)
				);
            } else {
				DateTime reference = calendar.Calendar.AddMonths(end, -12/frequency);
									// CHANGED: was end.plusMonths(-12/frequency);
				reference = calendar.Roll(reference, rollingConvention);
				leg.Add( new ShortFloatingCoupon(
					nominal, paymentDate, index, fixingDays, spread,
					start, end, reference, end)
				);
            }

			// regular periods
			for(int i=2; i<scheduler.Count-1; i++) 
			{
				start = end; 
				end = scheduler[i];
				paymentDate = calendar.Roll(end, rollingConvention);
				if ((i-1) < spreads.Length)
					spread = spreads[i-1];
				else if (spreads.Length > 0)
					spread = spreads[spreads.Length-1];
				else
					spread = 0.0;
				if ((i-1) < nominals.Length)
					nominal = nominals[i-1];
				else
					nominal = nominals[nominals.Length-1];

				leg.Add( new FloatingRateCoupon(
					nominal, paymentDate, index, fixingDays, spread, 
					start, end, start, end)
				);
			}

			if (scheduler.Count > 2) 
			{
				// last period might be short or long
				int N = scheduler.Count;
				start = end; 
				end = scheduler[N-1];
				paymentDate = calendar.Roll(end, rollingConvention);
				if ((N-2) < spreads.Length)
					spread = spreads[N-2];
				else if (spreads.Length > 0)
					spread = spreads[spreads.Length-1];
				else
					spread = 0.0;

				if ((N-2) < nominals.Length)
					nominal = nominals[N-2];
				else
					nominal = nominals[nominals.Length-1];
				if (scheduler.IsRegular(N-1)) 
				{
					leg.Add( new FloatingRateCoupon(
						nominal, paymentDate, index, fixingDays, spread, 
						start, end, start, end)
						);

				} else {

					DateTime reference = calendar.Calendar.AddMonths(start, 12/frequency);
									// CHANGED: was start.plusMonths(12/frequency);
					reference = calendar.Roll(reference, rollingConvention);
					leg.Add( new ShortFloatingCoupon(
						nominal, paymentDate, index, fixingDays, spread,
						start, end, reference, end)
					);
                }
            }
            return leg;
        }

		#endregion

	}
}
