// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BoundaryCondition.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:13:01  stochastix
 *  small changes for DotNum and license
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// Abstract boundary condition class for finite difference problems.
	/// </summary>
//	/// <remarks>
//	/// Three possibilities are given for setting boundary conditions,
//	/// namely, no boundary condition, Dirichlet boundary condition
//	/// (i.e., constant value), and Neumann boundary condition (i.e.,
//	/// constant derivative).
//	/// 
//	/// Warning: For Neumann conditions, the value passed must not be
//	/// the value of the derivative. Instead, it must be comprehensive
//	/// of the grid step between the first two points--i.e., it must be
//	/// the difference between f[0] and f[1].
//	/// </remarks>
	[ RcsId("$Id: BoundaryCondition.cs,v 1.1.1.1 2003/02/01 19:01:27 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Abstract boundary condition class for finite difference problems.") ]
	public abstract class BoundaryCondition
	{
		public enum BoundarySide { None, Upper, Lower };

		public BoundaryCondition() : this (BoundarySide.None)
		{}

		public BoundaryCondition(BoundarySide side) : base()
		{
			this.side = side; 
		}

		private BoundarySide side;

		public BoundarySide Side
		{
			get { return side; }
		}

		/// <summary>
		/// This method modifies an operator <code>L</code> before it is
		/// applied to an array <code>u</code> so that <i>v = L u</i> will
		/// satisfy the given condition.
		/// </summary>
		/// <param name="L"></param>
		public abstract void ApplyBeforeApplying(IOperator L);

		/// <summary>
		/// This method modifies an array <code>u</code> so that it
		/// satisfies the given condition.
		/// </summary>
		/// <param name="u"></param>
		public abstract void ApplyAfterApplying(Vector u);

		/// <summary>
		/// This method modifies an operator <code>L</code> before the linear
		/// system <i>L u' = u</i> is solved so that <i>u'</i> will 
		/// satisfy the given condition.
		/// </summary>
		/// <param name="L"></param>
		/// <param name="rhs"></param>
		public abstract void ApplyBeforeSolving(IOperator L, Vector rhs);

		/// <summary>
		/// This method modifies an array <code>u</code> so that it satisfies
		/// the given condition.
		/// </summary>
		/// <param name="u"></param>
		public abstract void ApplyAfterSolving(Vector u);

		/// <summary>
		/// This method sets the current time for time-dependent boundary conditions.
		/// </summary>
		/// <param name="time"></param>
		public abstract void SetTime(double time);

	}
}
