// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DirichletBC.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:28  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.Numerics;

namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// Dirichlet boundary condition (i.e., constant value).
	/// </summary>
	[ RcsId("$Id: DirichletBC.cs,v 1.1.1.1 2003/02/01 19:01:28 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Dirichlet boundary condition (i.e., constant value).") ]
	public abstract class DirichletBC : BoundaryCondition
	{

		public DirichletBC(BoundarySide side, double value) : base(side)
		{
			this.value  = value; 
		}

		private double value;

		public double Value
		{
			get { return value; }
		}

		/// <summary>
		/// This method modifies an operator <code>L</code> before it is
		/// applied to an array <code>u</code> so that <i>v = L u</i> will
		/// satisfy the given condition.
		/// </summary>
		/// <param name="L"></param>
		public override void ApplyBeforeApplying(IOperator L)
		{
			TridiagonalOperator op = (TridiagonalOperator) L;

			switch(Side)
			{
				case BoundarySide.Lower:
					op.SetFirstRow(1.0, 0.0);
					break;
				case BoundarySide.Upper:
					op.SetLastRow(0.0, 1.0);
					break;
				default:
					throw new ApplicationException( Strings.Get( GetType(),
						"TODO: Unknown side for Neumann boundary condition" ) );
			}
		}

		/// <summary>
		/// This method modifies an array <code>u</code> so that it
		/// satisfies the given condition.
		/// </summary>
		/// <param name="u"></param>
		public override void ApplyAfterApplying(Vector u)
		{
			switch(Side)
			{
				case BoundarySide.Lower:
					u[0] = value;
					break;
				case BoundarySide.Upper:
					u[u.Count-1] = value;
					break;
				default:
					throw new ApplicationException( Strings.Get( GetType(),
						"TODO: Unknown side for Neumann boundary condition" ) );
			}
		}

		/// <summary>
		/// This method modifies an operator <code>L</code> before the linear
		/// system <i>L u' = u</i> is solved so that <i>u'</i> will 
		/// satisfy the given condition.
		/// </summary>
		/// <param name="L"></param>
		/// <param name="rhs"></param>
		public override void ApplyBeforeSolving(IOperator L, Vector rhs)
		{
			TridiagonalOperator op = (TridiagonalOperator) L;

			switch(Side)
			{
				case BoundarySide.Lower:
					op.SetFirstRow(1.0, 0.0);
					rhs[0] = value;
					break;
				case BoundarySide.Upper:
					op.SetLastRow(0.0, 1.0);
					rhs[rhs.Count-1] = value;
					break;
				default:
					throw new ApplicationException( Strings.Get( GetType(),
						"TODO: Unknown side for Neumann boundary condition" ) );
			}
		}


		/// <summary>
		/// This method modifies an array <code>u</code> so that it satisfies
		/// the given condition.
		/// </summary>
		/// <param name="u"></param>
		public override void ApplyAfterSolving(Vector u)
		{}

		/// <summary>
		/// This method sets the current time for time-dependent boundary conditions.
		/// </summary>
		/// <param name="time"></param>
		public override void SetTime(double time)
		{}

	}
}
