// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TridiagonalOperator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:28  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:19:55  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:13:01  stochastix
 *  small changes for DotNum and license
 *
 *  Revision 1.2  2002/08/15 18:14:21  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using System.Diagnostics;
using Stochastix.Numerics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;


namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// Base implementation for a tridiagonal operator.
	/// </summary>
	/// <remarks>
	/// Warning: to use real time-dependant algebra, you must overload
	/// the corresponding operators in the inheriting time-dependent
	/// class.
	/// </remarks>
	[ RcsId("$Id: TridiagonalOperator.cs,v 1.1.1.1 2003/02/01 19:01:28 jens Exp $") ] 
	[ ComVisible(false) ]
	public class TridiagonalOperator : IOperator
	{
		#region Constructors

		public TridiagonalOperator() : this(0) {}
		public TridiagonalOperator(int size)
		{
			if (size==0) 
			{
				diagonal      = new Vector(0);
				belowDiagonal = new Vector(0);
				aboveDiagonal = new Vector(0);
			} 
			else if (size>=3)
			{
				diagonal      = new Vector(size);
				belowDiagonal = new Vector(size-1);
				aboveDiagonal = new Vector(size-1);
			} 
			else
			{
				throw new ArgumentOutOfRangeException( 
					"size", size, Strings.Format( GetType(),
					"FdTriCtor", size) );
			}
		}

		public TridiagonalOperator(Vector low, Vector mid, Vector high)
		{
			if( low.Count != mid.Count-1 )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriLoMid"), "low");
			if( high.Count != mid.Count-1 )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriHiMid"), "high");

			this.diagonal      = mid.Clone();
			this.belowDiagonal = low.Clone();
			this.aboveDiagonal = high.Clone();
		}

		#endregion

		private Vector diagonal, belowDiagonal, aboveDiagonal;
		protected TimeSetterImpl timeSetter;

		public TridiagonalOperator Clone()
		{
			TridiagonalOperator temp = new TridiagonalOperator();
			temp.aboveDiagonal = this.aboveDiagonal.Clone();
			temp.diagonal      = this.diagonal.Clone();
			temp.belowDiagonal = this.belowDiagonal.Clone();
			// TODO: ****** can I do this???? *********
			// temp.timeSetter = this.timeSetter;
			return temp;
		}	

		/// <summary>
		/// Identity instance.
		/// </summary>
		/// <param name="size"></param>
		/// <returns></returns>
		public static TridiagonalOperator Identity(int size)
		{
			return new TridiagonalOperator(
				new Vector(size-1, 0.0),     // lower diagonal
				new Vector(size,   1.0),     // diagonal
				new Vector(size-1, 0.0));    // upper diagonal
		}

		/// <summary>
		/// Identity instance.
		/// </summary>
		public IOperator Identity()
		{
			return Identity(this.Count);
		}


		public void SetFirstRow(double valB, double valC) 
		{
			diagonal[0]      = valB;
			aboveDiagonal[0] = valC;
		}

		public void SetMidRow(int i, double valA, double valB, double valC) 
		{
			// QL_RequireArgRange(i>=1 && i<Count-1, "i", "FdTriMid", i);
			// would throw an exception anyways
			belowDiagonal[i-1] = valA;
			diagonal[i]        = valB;
			aboveDiagonal[i]   = valC;
		}

		public void SetMidRows(double valA, double valB, double valC)
		{
			for (int i=1; i<Count-1; i++) 
			{
				belowDiagonal[i-1] = valA;
				diagonal[i]        = valB;
				aboveDiagonal[i]   = valC;
			}
		}

		public void SetLastRow(double valA, double valB) 
		{
			belowDiagonal[Count-2] = valA;
			diagonal[Count-1]      = valB;
		}

		#region Implementation (IOperator)

		public int Count
		{
			get { return diagonal.Count; }
		}

		public bool IsTimeDependent
		{
			get { return timeSetter!=null; }
		}

		public double Time
		{
			set 
			{ 
				if (timeSetter!=null)
					timeSetter.SetTime(value, this);
			}
		}

		/// <summary>
		/// Add other operator.
		/// </summary>
		/// <remarks>
		/// Only supports other <see cref="TridiagonalOperator"/>.
		/// </remarks>
		/// <exception cref="InvalidCastException">
		/// Thrown when no conversion to <see cref="TridiagonalOperator"/> exists
		/// for the given <see cref="IOperator"/>.
		/// </exception>
		public void Add(IOperator other)
		{
			TridiagonalOperator tri = (TridiagonalOperator)other;

			this.belowDiagonal.Add(tri.belowDiagonal);
			this.diagonal.Add(tri.diagonal);
			this.aboveDiagonal.Add(tri.aboveDiagonal);
		}

		/// <summary>
		/// Subtract other operator.
		/// </summary>
		/// <remarks>
		/// Only supports other <see cref="TridiagonalOperator"/>.
		/// </remarks>
		/// <exception cref="InvalidCastException">
		/// Thrown when no conversion to <see cref="TridiagonalOperator"/> exists
		/// for the given <see cref="IOperator"/>.
		/// </exception>
		public void Subtract(IOperator other)
		{
			// cast - can only subtract other TridiagonalOperator
			if(! (other is TridiagonalOperator) )
				throw new ArgumentException( 
					Strings.Get( GetType(), "FdTriType"), "other" );
			TridiagonalOperator tri = (TridiagonalOperator)other;

			this.belowDiagonal.Subtract(tri.belowDiagonal);
			this.diagonal.Subtract(tri.diagonal);
			this.aboveDiagonal.Subtract(tri.aboveDiagonal);
		}

		/// <summary>
		/// Multiply operator with scalar.
		/// </summary>
		public void Multiply(double scalar)
		{
			belowDiagonal.Multiply(scalar);
			diagonal.Multiply(scalar);
			aboveDiagonal.Multiply(scalar);
		}

		/// <summary>
		/// Apply operator to a given array.
		/// </summary>
        public Vector ApplyTo(Vector v)
		{
			// not checking v since the following code would 
			// throw a null reference exception anyway
			if( v.Count != Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriSize"), "v");
			Vector result = new Vector(Count);
			int last = Count-1;

            // matricial product
            result[0] = diagonal[0]      * v[0] + 
						aboveDiagonal[0] * v[1];
            for (int j=1; j<last;j++)
                result[j] = belowDiagonal[j-1] * v[j-1] + 
							diagonal[j]        * v[j]   +
							aboveDiagonal[j]   * v[j+1];
            result[last] = belowDiagonal[last-1] * v[last-1] +
						diagonal[last]        * v[last];

            return result;
        }

		/// <summary>
		/// Solve linear system for a given right-hand side.
		/// </summary>
        public Vector SolveFor(Vector rhs)
		{
			// not checking rhs since the following code would 
			// throw a null reference exception anyway
			if( rhs.Count != Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriSize"), "rhs");
			int last = Count-1;

            // solve tridiagonal system
			Vector result = new Vector(Count);
			Vector tmp = new Vector(Count);

            double bet=diagonal[0];
			if(bet == 0.0)
                throw new DivideByZeroException( 
					Strings.Get( GetType(), "FdTriDiv0"));
            result[0] = rhs[0] / bet;
            for (int j=1; j<=last; j++){
                tmp[j] = aboveDiagonal[j-1] / bet;
                bet = diagonal[j] - belowDiagonal[j-1]*tmp[j];
				if(bet == 0.0)
					throw new DivideByZeroException( 
						Strings.Get( GetType(), "FdTriDiv0") );
                result[j] = (rhs[j] - belowDiagonal[j-1]*result[j-1]) / bet;
            }
            // cannot be j>=0 with Size j
            for (int j=last-1; j>0; j--)
                result[j] -= tmp[j+1]*result[j+1];
            result[0] -= tmp[1]*result[1];

            return result;
        }

		/// <summary>
		/// Solve linear system with SOR approach.
		/// </summary>
		public Vector SOR(Vector rhs, double tolerance) 
		{
			// not checking rhs since the following code would 
			// throw a null reference exception anyway
			if( rhs.Count != Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriSize"), "rhs");
			if( tolerance <= 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"FdTriTol"), "tolerance");

			// initial guess
			Vector result = rhs.Clone();

            // solve tridiagonal system with SOR technique
            int sorIteration, i;
            double omega = 1.5;
            double err=2.0*tolerance;
            double temp;
            for (sorIteration=0; err>tolerance; sorIteration++) {
				if( sorIteration>=100000)
					throw new ApplicationException( Strings.Format(
						GetType(), "FdTriSOR", tolerance, sorIteration, err) );
                err=0.0;
                for (i=1; i<Count-1 ; i++) {
                    temp = omega * (rhs[i]     -
                         aboveDiagonal[i]   * result[i+1]-
                         diagonal[i]        * result[i] -
                         belowDiagonal[i-1] * result[i-1]) / diagonal[i];
                    err += temp * temp;
                    result[i] += temp;
                }
            }

            return result;
        }

		#endregion

		#region Operators

		// unary operators
		public static TridiagonalOperator operator+(TridiagonalOperator D)
		{
			return D.Clone();
		}
		public static TridiagonalOperator operator-(TridiagonalOperator D)
		{
			// mmmh. eventuell besser: 
			//	Clone me and then scale the diagonals

			Vector low = -D.belowDiagonal;
			Vector mid = -D.diagonal;
			Vector high = -D.aboveDiagonal;
			TridiagonalOperator temp = new TridiagonalOperator(low,mid,high);
			return temp;
		}

		// binary operators
		public static TridiagonalOperator operator+(TridiagonalOperator D1,
			TridiagonalOperator D2)
		{
			D1 = D1.Clone();
			D1.Add(D2);
			return D1;
		}
		public static TridiagonalOperator operator-(TridiagonalOperator D1,
			TridiagonalOperator D2)
		{
			D1 = D1.Clone();
			D1.Subtract(D2);
			return D1;
		}
		public static TridiagonalOperator operator*(double scalar,
			TridiagonalOperator D)
		{
			D = D.Clone();
			D.Multiply(scalar);
			return D;
		}
		public static TridiagonalOperator operator*(TridiagonalOperator D,
			double scalar)
		{
			D = D.Clone();
			D.Multiply(scalar);
			return D;
		}
		public static TridiagonalOperator operator/(TridiagonalOperator D,
			double scalar)
		{
			D = D.Clone();
			D.Multiply(1.0/scalar);
			return D;
		}

		#endregion

		#region Implementation (ICloneable)

		object ICloneable.Clone()
		{
			return this.Clone();
		}

		#endregion


		/// <summary>
		/// encapsulation of time-setting logic
		/// </summary>
		protected abstract class TimeSetterImpl 
		{
			public abstract void SetTime(double time, TridiagonalOperator L);
		};
	};
}
