// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Euribor.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/31 18:46:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:25  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;
using Stochastix.Business.Calendars;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Indexes
{
	/// <summary>
	/// Euribor index.
	/// </summary>
	/// <remarks>
	/// <para>
	/// <list type="table">
	/// <item>
	/// <term>Family name:</term>
	/// <description>Euribor</description>
	/// </item>
	/// <item>
	/// <term>Currency:</term>
	/// <description>EUR (see <see cref="Stochastix.Business.Currency"/>)</description>
	/// </item>
	/// <item>
	/// <term>Calendar:</term>
	/// <description><see cref="TARGET"/></description>
	/// </item>
	/// <item>
	/// <term>Settlement days:</term>
	/// <description>2</description>
	/// </item>
	/// <item>
	/// <term>Day counter:</term>
	/// <description>act/360 (see <see cref="Actual360"/>)</description>
	/// </item>
	/// <item>
	/// <term>Rolling convention:</term>
	/// <description>Modified Following (see <see cref="Stochastix.Business.RollingConvention"/>)</description>
	/// </item>
	/// </list>
	/// </para>
	/// </remarks>
	/// <seealso href="http://www.euribor.org/">Euribor homepage</seealso>
	/// <seealso href="http://mis.dresdner-bank.de/dremis/devisen/update_kurse/EURIBOR.htm">Euribor Rates</seealso>
	/// <seealso cref="IXiborHistory"/>
	/// <seealso cref="XiborManager"/>
	[ RcsId("$Id: Euribor.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	[ FactoryItem("Euribor"), FactoryItem("EUR") ]
	public sealed class Euribor : Xibor
	{
		/// <overloads>
		/// Initialize a new Euribor.
		/// </overloads>
		/// <summary>
		/// Initialize a new Euribor with a non-default <see cref="IDayCounter"/>.
		/// </summary>
		/// <param name="period">
		///		Tenor as <see cref="Period"/>.
		/// </param>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		/// </param>
		/// <param name="dayCounter">
		///		<see cref="IDayCounter"/> used for internal date/time conversions.
		/// </param>
		public Euribor(Period period, ITermStructure termStructure,
			IDayCounter dayCounter)
			: base( "Euribor", period, 2, Currency.EUR, TARGET.Instance, 
			true, RollingConvention.ModifiedFollowing, dayCounter, termStructure)
		{}

		/// <summary>
		/// Initialize a new Euribor for a tenor given as <see cref="Period"/>.
		/// </summary>
		/// <param name="tenor">
		///		Tenor as <see cref="Period"/>.
		/// </param>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		/// </param>
		public Euribor(Period tenor, ITermStructure termStructure)
			: this( tenor, termStructure, Actual360.Instance )
		{}

		
		/// <summary>
		/// Initialize a new Euribor for a tenor given as unit and length.
		/// </summary>
		/// <param name="tenorLength">
		///		Length of the tenor measured in units.
		///	</param>
		/// <param name="tenorUnits">
		///		<see cref="TimeUnit"/>s to be used for the tenor.
		///	</param>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		///	</param>
		public Euribor(int tenorLength, TimeUnit tenorUnits, ITermStructure termStructure)
			: this( new Period(tenorLength, tenorUnits), termStructure )
		{}

		/// <summary>
		/// Initialize a new Euribor for a tenor given as <see cref="String"/>.
		/// </summary>
		/// <param name="tenor">
		///		Tenor as <see cref="String"/>.
		///		Value must be convertable to a <see cref="Period"/> by <see cref="Period.Parse"/>.
		/// </param>
		/// <exception cref="ArgumentException">
		///		Thrown when the given tenor string can not be parsed
		///		into a valid <see cref="Period"/>.
		/// </exception>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		///	</param>
		public Euribor(String tenor, ITermStructure termStructure)
			: this( Period.Parse(tenor), termStructure )
		{}

	}
}
