// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Xibor.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:25  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Text;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Indexes
{
	/// <summary>
	/// Base class for Libor indexes.
	/// </summary>
	/// <seealso cref="IXiborHistory"/>
	/// <seealso cref="XiborManager"/>
	[ RcsId("$Id: Xibor.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	public class Xibor : Observable, IXibor
	{
		private String familyName;
		private Period tenor;
		private int settlementDays;
		private Currency currency;
		private IBusinessCalendar calendar;
		private bool isAdjusted;
		private RollingConvention rollingConvention;
		private IDayCounter dayCounter;
		private ITermStructure termStructure;

		#region Constructors

		/// <overloads>
		/// Initialize a new Xibor instance.
		/// </overloads>
		/// <summary>
		/// Initialize a new Xibor for a given unit length.
		/// </summary>
		/// <param name="familyName">
		///		Index family name.
		/// </param>
		/// <param name="tenorLength">
		///		Length of the tenor measured in units.
		///	</param>
		/// <param name="tenorUnits">
		///		<see cref="TimeUnit"/>s used for the tenor.
		///	</param>
		/// <param name="settlementDays">
		///		Settlement days, usually 2.
		/// </param>
		/// <param name="currency">
		///		Index <see cref="Stochastix.Business.Currency"/>.
		/// </param>
		/// <param name="calendar">
		///		<see cref="IBusinessCalendar"/> used for advancing dates and 
		///		rolling holidays to business days.
		/// </param>
		/// <param name="isAdjusted">
		/// </param>
		/// <param name="rollingConvention">
		///		Algorithm used to find the business day which is "closest" to a given holiday.
		///		See <see cref="Stochastix.Business.RollingConvention"/> for a list of possible values.
		/// </param>
		/// <param name="dayCounter">
		///		<see cref="IDayCounter"/> used for internal date/time conversions.
		/// </param>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		///	</param>
		public Xibor(String familyName, int tenorLength, TimeUnit tenorUnits, 
			int settlementDays, Currency currency, 
			IBusinessCalendar calendar, bool isAdjusted, 
			RollingConvention rollingConvention, IDayCounter dayCounter, 
			ITermStructure termStructure)
			: this(familyName, new Period(tenorLength, tenorUnits),
			settlementDays, currency, calendar, isAdjusted,
			rollingConvention, dayCounter, termStructure)
		{}

		/// <summary>
		/// Initialize a new Xibor with a tenor given as <see cref="Period"/>.
		/// </summary>
		/// <param name="familyName">
		///		Index family name.
		/// </param>
		/// <param name="tenor">
		///		Tenor as <see cref="Period"/>.
		/// </param>
		/// <param name="settlementDays">
		///		Settlement days, usually 2.
		/// </param>
		/// <param name="currency">
		///		<see cref="Stochastix.Business.Currency"/>.
		/// </param>
		/// <param name="calendar">
		///		<see cref="IBusinessCalendar"/> used for advancing dates and 
		///		rolling holidays to business days.
		/// </param>
		/// <param name="rollingConvention">
		///		Algorithm used to find the business day which is "closest" to a given holiday.
		///		See <see cref="Stochastix.Business.RollingConvention"/> for a list of possible values.
		/// </param>
		/// <param name="dayCounter">
		///		<see cref="IDayCounter"/> used for internal date/time conversions.
		/// </param>
		/// <param name="termStructure">
		///		<see cref="ITermStructure"/> used for forecasting index fixings.
		///	</param>
		public Xibor(String familyName, Period tenor, int settlementDays, 
			Currency currency, IBusinessCalendar calendar, bool isAdjusted, 
			RollingConvention rollingConvention, IDayCounter dayCounter, 
			ITermStructure termStructure)
		{
			this.familyName = familyName;
			this.tenor = tenor;
			this.settlementDays = settlementDays;
			this.currency = currency;
			this.calendar = calendar;
			this.isAdjusted = isAdjusted;
			this.rollingConvention = rollingConvention;
			this.dayCounter = dayCounter;
			this.termStructure = termStructure;
			RegisterWith(termStructure);
		} 

		#endregion

		#region Implementation of IXibor
		
		/// <summary>
		/// Index family name.
		/// </summary>
		public String FamilyName
		{	
			get { return familyName; }
		}

		/// <summary>
		/// Index tenor as <see cref="Period"/>.
		/// </summary>
		public Period Tenor
		{
			get { return tenor; }
		}

		/// <summary>
		/// Settlement days.
		/// </summary>
		public int SettlementDays
		{ 
			get { return settlementDays; }
		}
        
		/// <summary>
		/// Index <see cref="Stochastix.Business.Currency"/>.
		/// </summary>
		public Currency Currency
		{ 
			get { return currency; } 
		}
        
		/// <summary>
		///		<see cref="IBusinessCalendar"/> used for advancing dates and 
		///		rolling holidays to business days.
		/// </summary>
		public IBusinessCalendar Calendar
		{ 
			get { return calendar; }
		}

		public bool IsAdjusted
		{ 
			get { return isAdjusted; }
		}
        
		/// <summary>
		///		Algorithm used to find the business day which is "closest" to a given holiday.
		///		See <see cref="Stochastix.Business.RollingConvention"/> for a list of possible values.
		/// </summary>
		public RollingConvention RollingConvention
		{
			get { return rollingConvention; }
		}

        /// <summary>
		///	<see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public IDayCounter DayCounter
		{ 
			get { return dayCounter; }
		}
        
		/// <summary>
		/// <see cref="ITermStructure"/> used for forecasting index fixings.
		/// </summary>
		public ITermStructure TermStructure
		{
			get { return termStructure; }
		}

		/// <summary>
		/// Fixing frequency in 1/years.
		/// </summary>
		/// <value>The fixing frequency in times/year</value>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the period is not one of 1m, 2m, 3m, 4m, 6m, 12m and 1y.
		/// </exception>
		public int Frequency
		{
			get { return tenor.Frequency; }
		}

		#endregion

		#region Implementation of IIndex

		/// <summary>
		/// A stringified representation of the Index.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public string ToString()
		{	// original: familyName_+tenor+" "+dayCounter_.name();
			return String.Format("{0}-{1} ({2})", 
				familyName, tenor, dayCounter
				);
		}

		/// <summary>
		/// Index fixing at the given date.
		/// </summary>
		/// <remarks>
		/// <para>
		/// Any date passed as arguments must be a value date,
		/// i.e., the real calendar date advanced by a number of
		/// settlement days.
		/// </para>
		/// </remarks>
		/// <param name="fixingDate">The fixing date as a value date.</param>
		/// <returns>The fixing at the given date.</returns>
		public double Fixing(System.DateTime fixingDate)
		{
			fixingDate = fixingDate.Date;	// normalize
			DateTime todaysDate = termStructure.TodaysDate; // TODO: normalize?
			if (fixingDate < todaysDate) 
			{
                // must have been fixed
				return XiborManager.History.Fixing(
					familyName, tenor.ToString(), fixingDate
				);
            }
            if (fixingDate == todaysDate) 
			{
                // might have been fixed
                try {
					return XiborManager.History.Fixing(
						familyName, tenor.ToString(), fixingDate
					);
				} 
				catch 
				{
                    ;       // fall through and forecast
                }
            }

			DateTime fixingValueDate = calendar.Advance(fixingDate,
				settlementDays, TimeUnit.Days, 
				// TODO: use rollingConvention instead of Following
				RollingConvention.Following);
			DateTime endValueDate = calendar.Advance(fixingValueDate, tenor, 
                rollingConvention);
			double fixingDiscount = termStructure.Discount(fixingValueDate, false);
			double endDiscount = termStructure.Discount(endValueDate, false);
			double fixingPeriod = dayCounter.YearFraction(fixingValueDate, endValueDate);
			return (fixingDiscount/endDiscount-1.0) / fixingPeriod;
		}

		#endregion

	}
}

