// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Instrument.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Instruments
{
	/// <summary>
	/// Abstract instrument class.
	/// </summary>
	/// <remarks>
	/// This class is purely abstract and defines the interface of 
	/// concrete instruments which will be derived from this one.
	/// </remarks>
	[ RcsId("$Id: Instrument.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Instrument : Observable, IInstrument
	{
		public Instrument() : this("", "") 
		{}

		public Instrument(string isinCode, string description)
		{
			this.isinCode = isinCode;
			this.description = description;
			this.npv_ = 0.0;
			this.isExpired_ = false;
			this.calculated = false;
		}

		protected double npv_;
		protected bool isExpired_;
		private string isinCode, description;
		private bool calculated;

		/// <summary>
		/// A stringified representation of this instrument.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		// string ToString();

		protected override void Notify(object sender, System.EventArgs e)
		{
			calculated = false;
			base.Notify(sender, e);
		}

		/// <summary>
		/// This method performs all needed calculations by calling
		/// the <i><b>performCalculations</b></i> method.
		/// </summary>
		/// <remarks>
		/// Instruments cache the results of the previous calculation. 
		/// Such results will be returned upon later invocations of 
		/// <i><b>calculate</b></i>. When the results depend on 
		/// arguments such as term structures which could change
		/// between invocations, the instrument must register itself 
		/// as observer of such objects for the calculations to be
		/// performed again when they change.
		/// </remarks>
		protected void Calculate()
		{
			if (!calculated) PerformCalculations();
			calculated = true;
		}

		/// <summary>
		/// This method must implement any calculations which must be
		/// (re)done in order to calculate the NPV of the instrument.
		/// </summary>
		abstract protected void PerformCalculations();

		#region Implementation of IInstrument

		/// <summary>
		/// Net present value of the instrument.
		/// </summary>
		/// <returns>Returns the net present value.</returns>
		public double NPV
		{
			get 
			{
				Calculate();
				return (isExpired_ ? 0.0 : npv_);
			}
		}

		/// <summary>
		/// This method force the recalculation of the instrument value 
		/// and other results which would otherwise be cached.
		/// </summary>
		/// <remarks>
		/// Explicit invocation of this method is <b>not</b> necessary 
		/// if the instrument registered itself as observer with the 
		/// structures on which such results depend. 
		/// It is strongly advised to follow this policy when possible.
		/// </remarks>
		public void Recalculate()
		{
			PerformCalculations();
			calculated = true;
			OnChanged();
		}

		/// <summary>
		/// ISIN code of the instrument, when given.
		/// </summary>
		public string IsinCode
		{
			get	{ return isinCode; }
		}

		/// <summary>
		/// A brief textual description of the instrument.
		/// </summary>
		public string Description
		{
			get { return description; }
		}

		/// <summary>
		/// Returns whether the instrument is still tradable.
		/// </summary>
		public bool IsExpired
		{
			get	{ Calculate(); return isExpired_; }
		}
		#endregion

	}
}
