// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Option.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.3  2002/09/02 09:55:04  stochastix
 *  added ExercisePayoff
 *
 *  Revision 1.2  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Data;
using System.Diagnostics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Instruments;
using Stochastix.QuantLib.PricingEngines;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Instruments {

	/// <summary>
	/// 
	/// </summary>
	[ RcsId("$Id: Option.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	abstract public class Option : Instrument 
	{
		public enum Type { Call, Put, Straddle };

		public static double ExercisePayoff(Type optionType, double price,
			double strike) 
		{
			switch(optionType)
			{
				case Type.Call:
					return Math.Max(price-strike, 0.0);
				case Type.Put:
					return Math.Max(strike-price, 0.0);
				case Type.Straddle:
					return Math.Abs(strike-price);
				default:
					throw new ArgumentOutOfRangeException( "optionType", optionType,
						Strings.Get(typeof(Option), "TODO: Unknown option type"));
			}
		}

		public Option(IPricingEngine engine, DataRow args) 
			: this (engine, args, "", "")
		{}

		public Option(IPricingEngine engine, DataRow args, 
			string isinCode, string description) : base(isinCode, description)
		{
			this.engine = engine;
			this.args = args;
			DataTable temp = new DataTable("Results");
			temp.Columns.AddRange(OptionValue.Columns);
            this.results = temp.NewRow();
		}

		private IPricingEngine engine;
		private DataRow args, results;

		public IPricingEngine PricingEngine
		{
			get { return engine; }
			set
			{
				if (value == null) 
					throw new NullReferenceException(
						Strings.Get(GetType(), "TODO: null pricing engine not allowed") );
				this.engine = engine;
				// this will trigger recalculation and notify observers
				Update();
				SetupEngine();
			}
		}

		protected abstract void SetupEngine();

		/*! \warning this method simply launches the engine and copies the 
				returned value into NPV_. It does <b>not</b> set isExpired_. 
				This should be taken care of by redefining this method in
				derived classes and calling this implementation after 
				checking for validity and only if the check succeeded.
		*/
		override protected void PerformCalculations()
		{
			SetupEngine();
			// assume 0 is the "value" column
			Debug.Assert( results.Table.Columns.IndexOf("value") == 0);
			results[0] = DBNull.Value;
			engine.Validate(args);
			engine.Calculate(args, results);
			if( results.IsNull(0) ) 
				throw new ApplicationException(
					"no results returned from option pricer");
			npv_ = (double)results[0];
		}
	}
}

