// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Swap.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.QuantLib.CashFlows;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Instruments
{
	/// <summary>
	/// Interest rate swap. The cash flows belonging to the first 
	/// leg are payed; the ones belonging to the first leg are received.
	/// </summary>
	[ RcsId("$Id: Swap.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	public class Swap : Instrument, IInstrument
	{
		public Swap(IEnumerable firstLeg_, IEnumerable secondLeg_,
			ITermStructure termStructure) 
			: this(firstLeg_, secondLeg_, termStructure, "", "")
		{}
		
		public Swap(IEnumerable firstLeg_, IEnumerable secondLeg_,
			ITermStructure termStructure, 
			string isinCode, string description)
		: base(isinCode, description)
		{
			this.firstLeg_ = firstLeg_;
			this.secondLeg_ = secondLeg_;
			this.termStructure = termStructure;
			RegisterWith(termStructure);
			RegisterWith(firstLeg_);
			RegisterWith(secondLeg_);
	   }

		// data members
		protected IEnumerable firstLeg_, secondLeg_;
		private ITermStructure termStructure;
		private double firstLegBPS, secondLegBPS;

		public IEnumerable FirstLeg
		{
			get { return firstLeg_; }
		}

		public IEnumerable SecondLeg
		{
			get { return secondLeg_; }
		}

		public double FirstLegBPS
		{
			get 
			{
				Calculate();
				return firstLegBPS;
			}
		}

		public double SecondLegBPS 
		{
			get 
			{
				Calculate();
				return secondLegBPS;
			}
		}

        override protected void PerformCalculations() 
		{
			if(termStructure == null)
				throw new NullReferenceException( 
					Strings.Get(GetType(), "SwapNullTs" ) );

			DateTime settlement = termStructure.ReferenceDate;
            npv_ = 0.0;
            firstLegBPS = 0.0;
            secondLegBPS = 0.0;
            isExpired_ = true;

			// TODO: run the following with a multiplicatr of +1./-1.

            // subtract first leg cash flows and BPS
			foreach( ICashFlow cashFlow in firstLeg_)
			{
                DateTime cashFlowDate = cashFlow.Date;
                if (cashFlowDate >= settlement) {
                    isExpired_ = false;	// keeping track of whether this
                                        // was already set isn't worth the
                                        // effort
					double discount = termStructure.Discount(cashFlowDate, false);
                    npv_ -= cashFlow.Amount * discount;
					if (cashFlow is ICoupon) 
					{
						ICoupon coupon = (ICoupon)cashFlow;
						// subtract coupon sensitivity
                        firstLegBPS -= coupon.AccrualPeriod *
                            coupon.Nominal * discount;
                    }
                }
            }
            // add second leg cash flows and BPS
			foreach( ICashFlow cashFlow in secondLeg_)
			{
                DateTime cashFlowDate = cashFlow.Date;
                if (cashFlowDate >= settlement) {
                    isExpired_ = false;	
					double discount = termStructure.Discount(cashFlowDate, false);
                    npv_ += cashFlow.Amount * discount;
					if (cashFlow is ICoupon) 
					{
						ICoupon coupon = (ICoupon)cashFlow;
						// add coupon sensitivity
                        secondLegBPS += coupon.AccrualPeriod *
                            coupon.Nominal * discount;
                    }
                }
            }
        }

    }

}

