// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: EverestPathPricer.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using Stochastix.Numerics;
using Stochastix.QuantLib.Instruments;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Path pricer for European-type Everest option.
	/// </summary>
	public class EverestPathPricer : PathPricer
	{
		public EverestPathPricer(double discountFactor, 
			bool useAntitheticVariance) 
			: base( discountFactor, useAntitheticVariance)
		{}

		/// <summary>
		/// Given one or more pathes, the value of an option is returned.
		/// </summary>
		/// <param name="pathes"></param>
		/// <returns></returns>
		public override double Value(Path[] pathes)
		{
			if( pathes.Length == 0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: No path given." ), "pathes" );
			if( pathes[0].Count == 0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: the path cannot be empty."), "pathes" );

			int numAssets = pathes.Length;
			Vector logDrift     = new Vector(numAssets);
			Vector logDiffusion = new Vector(numAssets);
			for(int j=0; j<numAssets; j++)
			{
				logDrift[j]     = pathes[j].Drift.Sum();
				logDiffusion[j] = pathes[j].Diffusion.Sum();
			}

			// Exp(Min(vector)) should be Min(Exp(vector))
			double minPrice1 = Math.Exp( (logDrift+logDiffusion).Min() );
			if (useAntitheticVariance)
			{
				double minPrice2 = Math.Exp( (logDrift-logDiffusion).Min() );
				return discount * ( minPrice1+minPrice2) / 2.0;
			}
			else
			{
				return discount * minPrice1;
			}
		}
	}
}
