// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: MultiPathGenerator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.2  2002/10/15 13:18:58  stochastix
 *  now using dotNum
 *
 *  Revision 1.1  2002/08/30 18:17:08  stochastix
 *  initial revision
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Numerics.Statistics;
using Stochastix.Random;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Generates multiple random pathes from a continous RNG.
	/// </summary>
	/// <remarks>
	/// See introduction to <see cref="PathGenerator"/> base class for more
	/// information about path generators.
	/// <para>
	/// The <see cref="MultiPathGenerator"/> is initialized with an array of
	/// constant drifts - one for each single asset - and a covariance matrix
	/// which encapsulates the relations between the diffusion components of
	/// the single assets.
	/// </para>
	/// <para>
	/// The time discretization of the paths can be specified either as
	/// a given number of equal time steps over a given time span, or as a
	/// vector of explicitly specified times at which the path will be sampled.
	/// </para>
	/// </remarks>
	public class MultiPathGenerator : PathGenerator
	{
		/// <overloads>
		/// Initialize a new MultiPathGenerator.
		/// </overloads>
		/// <summary>
		/// Initialize a new MultiPathGenerator which samples in equal time steps.
		/// </summary>
		/// <param name="drifts">
		/// A <see cref="Vector"/> of constant drifts - one for each single asset.
		/// </param>
		/// <param name="covariance">
		/// A covariance <see cref="Matrix"/> which encapsulates the relations between
		/// the diffusion components of the single assets.
		/// </param>
		/// <param name="time">
		/// Time span which is sampled in <paramref name="timeSteps"/> equal time steps.
		/// </param>
		/// <param name="timeSteps">
		/// Number of equal time steps at which the path will be sampled.
		/// </param>
		public MultiPathGenerator(IContinousRng generator,
			Vector drifts, Matrix covariance, double time, int timeSteps)
			: base(drifts, time, timeSteps)
		{
			initializeGenerator(generator, covariance);		
		}

		/// <summary>
		/// Initialize a new MultiPathGenerator with constant drifts and variances.
		/// </summary>
		/// <summary>
		/// Initialize a new MultiPathGenerator which samples at explicitly specified times.
		/// </summary>
		/// <remarks>
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </remarks>
		/// <param name="drifts">
		/// A <see cref="Vector"/> of constant drifts - one for each single asset.
		/// </param>
		/// <param name="covariance">
		/// A covariance <see cref="Matrix"/> which encapsulates the relations between
		/// the diffusion components of the single assets.
		/// </param>
		/// <param name="times">
		/// A <see cref="Vector"/> of explicitly specified times at which the
		/// path will be sampled. 
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </param>
		public MultiPathGenerator(IContinousRng generator,
			Vector drifts, Matrix covariance, Vector times) 
			: base(drifts, times)
		{
			initializeGenerator(generator, covariance);
		}

		private void initializeGenerator(IContinousRng generator, Matrix covariance)
		{
			Vector diagonal = covariance.Diagonal;
			if( this.Count != diagonal.Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Covariance matrix does not match number of assets"), "covariance" );
			if( diagonal.Min() < 0.0)
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Covariance matrix contains negative variance."), "covariance" );

			this.arrayGenerator = new RandomArrayGenerator( generator, covariance);
		}

		/// <summary>
		/// Generate next pathes.
		/// </summary>
		/// <returns>
		/// An array containing <see cref="PathGenerator.Count"/> <see cref="Path"/>s, 
		/// wrapped in a (possibly weighted) <see cref="Sample"/>.
		/// </returns>
		public override Sample Next()
		{
			Matrix diffusion = new Matrix(Count, TimeSteps);
			double weight = 1.0;
			for( int i=0; i<TimeSteps; i++)
			{
				Sample sample = arrayGenerator.NextSample();  // get a weighted Vector
				weight *= sample.weight;
				Vector v = (Vector)sample.value;
				v.Multiply( Math.Sqrt( timeDelays[i] ));
				v.CopyTo( diffusion.Column(i) );
			}

			Path[] pathes = new Path[Count];
			for( int j=0; j<Count; j++)
				pathes[j] = new Path( times, drift[j], diffusion.Row(j) );

			return new Sample( pathes, weight);
		}
	}
}
