// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RandomArrayGenerator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using Stochastix.Numerics;
using Stochastix.Random;
using Stochastix.Numerics.Statistics;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Generates random arrays from a random number generator.
	/// </summary>
    public class RandomArrayGenerator
	{
		/// <summary>
		/// equal average, different variances, no covariance
		/// </summary>
		/// <param name="generator"></param>
		/// <param name="variance"></param>
		public RandomArrayGenerator(
			IContinousRng generator, Vector variance)
		{
			this.size = variance.Count;
			this.generator = generator;

			// TODO: replace by VML equiv. and check return 
			// status for exceptions
			sqrtVariance = Vector.Sqrt(variance);
			for(int i=0; i<sqrtVariance.Count; i++)
			{
				if( variance[i] < 0.0)
					throw new ArgumentOutOfRangeException( "variance",
						variance[i], Strings.Get( GetType(), "TODO: negative variance"));
			}
		}

		/// <summary>
		/// different averages, different variances, covariance
		/// </summary>
		/// <param name="generator"></param>
		/// <param name="covariance"></param>
		public RandomArrayGenerator(
			IContinousRng generator, Matrix covariance)
		{
			if( covariance.Rows != covariance.Columns )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Covariance matrix must be square."), "covariance" );
			if( covariance.Rows == 0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Null covariance matrix given."), "covariance" );
			this.size = covariance.Rows;
			this.generator = generator;
			sqrtCovariance = Matrix.Sqrt(covariance);
		}

		private int size;
		private IContinousRng generator;
		private Vector sqrtVariance;
		private Matrix sqrtCovariance;

		public int Count
		{
			get { return size; }
		}

		public bool IsConstantWeight
		{
			get { return true; /* generator.IsConstantWeight; */ }
		}

		/// <summary>
		/// Generate a <see cref="Sample"/> containing
		/// a <see cref="Vector"/> of random numbers.
		/// </summary>
		/// <returns>A <see cref="Vector"/> of random numbers.</returns>
		public Sample NextSample()
		{
			// generate and consume the necessary samples
			// TODO: Vector.Random(generator)
			double[] r = new double[size];
			generator.Next(r);
			Vector samples = new Vector( r, false /* no copy, consume */);

			if (sqrtCovariance != null)
				samples = sqrtCovariance * samples;		// general case
			else
				samples.Multiply(sqrtVariance);			// degenerate case

			return new Sample( samples );
		}
    };
}
