// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IDiffusionProcess.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:23  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;


namespace Stochastix.QuantLib.NumericalMethods
{
	/// <summary>
	/// Diffusion process interface.
	/// </summary>
	/// <remarks>
	/// This class describes a stochastic process goverved by 
	/// \f[
	///		dx_t = \mu(t, x_t)dt + \sigma(t, x_t)dW_t.
	///	\f]
	/// </remarks>
	[ RcsId("$Id: IDiffusionProcess.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Diffusion process interface.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface IDiffusionProcess 
	{
		double X0 
		{
			get;
		}

		/// <summary>
		/// The drift part of the equation. 
		/// </summary>
		/// <remarks>
		/// i.e. \f$ \mu(t, x_t) \f$
		/// </remarks>
		/// <param name="time"></param>
		/// <param name="x"></param>
		/// <returns></returns>
		double Drift(double time, double x);

		/// <summary>
		/// The diffusion part of the equation.
		/// </summary>
		/// <remarks>
		/// i.e. \f$\sigma(t,x_t)\f$
		/// </remarks>
		/// <param name="time"></param>
		/// <param name="x"></param>
		/// <returns></returns>
		double Diffusion(double time, double x);

		/// <summary>
		/// The expectation of the process after a time interval.
		/// </summary>
		/// <remarks>
		///  By default, it returns the Euler approximation defined by
		///  \f$ x_0 + \mu(t_0, x_0) \Delta t \f$.
		/// </remarks>
		/// <param name="t0"></param>
		/// <param name="x0"></param>
		/// <param name="dt"></param>
		/// <returns>
		/// returns \f$ E(x_{t_0 + \Delta t} | x_{t_0} = x_0) \f$.
		/// </returns>
		double Expectation(double t0, double x0, double dt);

		/// <summary>
		/// The variance of the process after a time interval.
		/// </summary>
		/// <remarks>
		/// By default, it returns the Euler approximation defined by
		/// \f$ \sigma(t_0, x_0)^2 \Delta t \f$.
		/// </remarks>
		/// <param name="t0"></param>
		/// <param name="x0"></param>
		/// <param name="dt"></param>
		/// <returns>
		/// returns \f$ Var(x_{t_0 + \Delta t} | x_{t_0} = x_0) \f$.
		/// </returns>
		double Variance(double t0, double x0, double dt);
	}
}
