// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TimeGrid.cs,v $
 *  Revision 1.2  2003/02/04 11:52:42  jens
 *  made ctors public
 *
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Utilities;
using Stochastix.Utilities.Collections;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.NumericalMethods
{
	/// <summary>
	/// Time grid class.
	/// </summary>
	// TODO: Maybe I should impl. ICollection (or even IList) 
	//       and inherit from ReadOnlyCollectionBase ??
	[ RcsId("$Id: TimeGrid.cs,v 1.2 2003/02/04 11:52:42 jens Exp $") ] 
	[ ComVisible(false) ]
	public class TimeGrid : ITimeGrid // public std::vector<Time> 
	{
		public TimeGrid() 
		{}

		/// <summary>
		/// Regularly spaced time-grid.
		/// </summary>
		/// <param name="end"></param>
		/// <param name="steps"></param>
		public TimeGrid(double end, int steps)
		{
			timeVector = new double[steps+1];
			double dt = end/steps;
			for (int i=0; i<=steps; i++)
				timeVector[i] = dt*i;
		}
		
		/// <summary>
		/// Time grid with mandatory time-points (regularly spaced between them).
		/// </summary>
		public TimeGrid(IList times, int steps)
		{
			double dtMax = Double.MaxValue;
			// The resulting timegrid have points at times listed in the input
			// list. Between these points, there are inner-points which are
			// regularly spaced.
			if (steps == 0) 
			{
				double prev = 0.0;
				foreach (double t in times) 
				{
					if (prev == t) continue;	// skip a 0.0 as first element
					double diff = t-prev;
					if( diff < dtMax ) dtMax = diff;
					prev = t;
				}
			} 
			else 
			{
				dtMax = (double)times[times.Count-1] / steps;
			}

			DoubleVector coll = new DoubleVector();
			double periodBegin = 0.0;
			foreach (double periodEnd in times) 
			{
				if (periodBegin >= periodEnd) 
					continue;
				int nSteps = (int)((periodEnd - periodBegin)/dtMax + 1.0);
				double dt = (periodEnd - periodBegin)/nSteps;
				for (int n=0; n<nSteps; n++)
					coll.Add(periodBegin + n*dt);
				periodBegin = periodEnd;
			}
			coll.Add(periodBegin); // Note periodBegin = periodEnd
			timeVector = new double[coll.Count];
			coll.CopyTo(timeVector);
		}

		private double[] timeVector;

		public int Count
		{
			get { return timeVector.Length; }
		}

		public double this[int i]
		{
			get { return timeVector[i]; }
		}

		public int FindIndex(double time)
		{
			// Is the tree sorted??
			// int i = Array.BinarySearch(timeVector, time);
			int i = Array.IndexOf(timeVector, time);
			if( i>=0 ) return i;
			throw new ApplicationException( 
				Strings.Get(GetType(), "GridNotFound"));
		}

		public double Dt(int i)
		{
			  return timeVector[i+1] - timeVector[i];
		}
	}
}
