// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ArmijoLineSearch.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Armijo linesearch.
	/// </summary>
	/// <remarks>
	/// Let alpha and beta be 2 scalars in [0,1].
	/// Let x be the current value of the unknown, d the search direction and
	/// t the step. Let f be the function to minimize.
	/// 
	/// The line search stops when t verifies
	///		f(x+t*d) - f(x) &lt;= -alpha*t*f'(x+t*d) and
	///		f(x+t/beta*d) - f(x) &gt; -alpha*t*f'(x+t*d)/beta
	///   
	/// (see Polak. Algorithms and consitent approximations, Optimization,
	/// volume 124 of Applied Mathematical Sciences. Springer-Verlag, N-Y,
	/// 1997)
	/// </remarks>
	[ RcsId("$Id: ArmijoLineSearch.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Armijo linesearch.") ]
	public class ArmijoLineSearch : LineSearch 
	{
		/// <summary>
		/// Default constructor.
		/// </summary>
		public ArmijoLineSearch()
			: this(1e-8, 0.5, 0.65)
		{}
		/// <summary>
		/// Initiliazation constructor.
		/// </summary>
		/// <param name="eps"></param>
		/// <param name="alpha"></param>
		/// <param name="beta"></param>
		public ArmijoLineSearch(double eps, double alpha, double beta)
			: base(eps)
		{
			this.alpha = alpha;
			this.beta = beta;
		}
		
		//! Armijo paramters
		private double alpha, beta;

		/// <summary>
		/// Perform line search.
		/// </summary>
		/// <param name="P">Optimization problem.</param>
		/// <param name="t">Initial value of line-search step.</param>
		/// <returns></returns>
		override public double Search( Problem P, double t)
		{
			Method method = P.Method;
			Constraint constraint = P.Constraint;

			bool maxIter = false;
			double q0 = method.FunctionValue;
			double qp0 = method.GradientNormValue;
			qt = q0;
			qpt = qp0;
			double qtold;
			int loopNumber = 0;

			// Initialize gradient
			gradient = new Vector(method.X.Count);

			// Compute new point
			xtd = method.X.Clone();
			t = Update(xtd, method.SearchDirection, t, constraint);
			// Compute function value at the new point
			qt = P.Value(xtd);

			// Enter in the loop if the criterion is not satisfied
			if ((qt-q0) > -alpha*t*qpt) 
			{
				do 
				{
					loopNumber++;
					// Decrease step
					t *= beta;
					// Store old value of the function
					qtold = qt;
					// New point value
					xtd = method.X.Clone();
					t = Update(xtd, method.SearchDirection, t, constraint);

					// Compute function value at the new point
					qt = P.Value(xtd);
					P.Gradient(gradient, xtd);
					// and it squared norm
					maxIter = method.EndCriteria.
						CheckIterationNumber(loopNumber);
				} while (
					(((qt - q0) > (-alpha * t * qpt)) ||
					((qtold - q0) <= (-alpha * t * qpt / beta))) &&
					(!maxIter));
			}

			succeed = !maxIter;

			// Compute new gradient
			P.Gradient(gradient, xtd);
			// and it squared norm
			qpt = Vector.DotProduct(gradient, gradient);

			// Return new step value
			return t;
		}

	}
}