// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ConjugateGradient.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Multi-dimensionnal Conjugate Gradient class.
	/// </summary>
	/// <remarks>
	/// User has to provide line-search method and optimization 
	/// end criteria.
	/// 
	/// Search direction d_i = - f'(x_i) + c_i * d_{i-1} 
	/// where c_i = ||f'(x_i)||^2 / ||f'(x_{i-1})||^2 
	/// and d_1 = - f'(x_1)
	/// </remarks>
	[ RcsId("$Id: ConjugateGradient.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	public class ConjugateGradient: Method 
	{	
		/// <summary>
		/// Default constructor.
		/// </summary>
		public ConjugateGradient() : this(new ArmijoLineSearch())
		{}

		/// <summary>
		/// Initialization constructor.
		/// </summary>
		/// <param name="lineSearch"></param>
		public ConjugateGradient(LineSearch lineSearch)
			: base()
		{
			this.lineSearch = lineSearch;
		}

		//! line search
		private LineSearch lineSearch;

		/// <summary>
		/// Minimize the optimization problem P.
		/// </summary>
		/// <param name="P"></param>
		override public void Minimize(Problem P)
		{
			// classical initial value for line-search step
			double t = 1.0;

			Vector grad = new Vector(SearchDirection.Count);
			FunctionValue = P.ValueAndGradient(grad, X);
			SearchDirection = -grad;
			GradientNormValue = Vector.DotProduct(grad, grad);

			for( bool end=true; !end; Iterations++)
			{
				// Linesearch
				t = lineSearch.Search(P, t);
				if (!lineSearch.Succeeded)
					throw new ApplicationException(
						Strings.Get( GetType(), "OptLineSrch" ) );

				// New point
				X = lineSearch.LastX;
				// New function value
				double fold = FunctionValue;
				FunctionValue = lineSearch.LastFunctionValue;
				// New gradient and search direction vectors
				grad = lineSearch.LastGradient;
				// orthogonalization coef
				double gold2 = GradientNormValue;
				GradientNormValue = lineSearch.LastGradientNorm2;
				// conjugate gradient search direction
				Vector d = SearchDirection * ( GradientNormValue / gold2 );
				d.Subtract(grad);
				Vector sddiff = d - SearchDirection;
				double normdiff = Math.Sqrt(Vector.DotProduct(sddiff, sddiff));
				SearchDirection = d;
				// End criteria
				end = EndCriteria.Check( Iterations, fold, Math.Sqrt(gold2), 
					FunctionValue, Math.Sqrt(GradientNormValue), normdiff);
			}
		}
	}
}
