// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Method.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;


namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Optimization Method abstract class for constrained optimization 
	/// problem.
	/// </summary>
	[ RcsId("$Id: Method.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Method
	{
		protected Method()
		{
			// this.iterationNumber = 0;
			// this.functionEvaluation = 0;
			// this.gradientEvaluation = 0;
			// this.functionValue = 1.0;
			// this.squaredNorm = 1.0;
		}

		//! initial value of unknowns
		private Vector initialValue;
		//! current iteration step in the Optimization process
		private int iterationNumber;
		//! optimization end criteria
		private EndCriteria endCriteria;
		//! number of evaluation of cost function and its gradient
		private int functionEvaluation, gradientEvaluation;
		//! function and gradient norm values of the last step
		private double functionValue, squaredNorm;
		//! current values of the local minimum and the search direction
		private Vector x, searchDirection;

		/// <summary>
		/// Initial value of unknowns.
		/// </summary>
		/// <remarks>
		/// Resets iterations as well as the current values of the local 
		/// minimum and the search direction.
		/// </remarks>
		public Vector InitialValue 
		{
			get { return this.initialValue; }
			set 
			{
				iterationNumber = 0;
				initialValue = value.Clone();
				x = initialValue.Clone();
				searchDirection = new Vector(x.Count);
			}
		}

		/// <summary>
		/// Optimization end criteria.
		/// </summary>
		public EndCriteria EndCriteria
		{
			get { return endCriteria; }
			set { this.endCriteria = value.Clone(); }
		}

		/// <summary>
		/// Current iteration step in the optimization process.
		/// </summary>
		public int Iterations
		{ 
			get { return iterationNumber; }
			set { this.iterationNumber = value; }
		}

		/// <summary>
		/// Number of evaluations of the cost function.
		/// </summary>
		public int FunctionEvaluations
		{
			get { return functionEvaluation; }
			set { this.functionEvaluation = value; }
		}

		/// <summary>
		/// Number of evaluations of the cost function gradient.
		/// </summary>
		public int GradientEvaluations
		{
			get { return gradientEvaluation; }
			set { this.gradientEvaluation = value; }
		}

		/// <summary>
		/// Value of the cost function in the last step.
		/// </summary>
		public double FunctionValue
		{
			get { return functionValue; }
			set { this.functionValue = value; }

		}

		/// <summary>
		/// Value of the cost function gradient norm in the last step.
		/// </summary>
		public double GradientNormValue
		{
			get { return squaredNorm; }
			set { this.squaredNorm = value; }
		}

		/// <summary>
		/// Current value of the local minimum.
		/// </summary>
		public Vector X
		{
			get { return x; }
			set { x = value; }
		}

		/// <summary>
		/// Current value of the search direction.
		/// </summary>
		public Vector SearchDirection
		{
			get { return searchDirection; }
			set { searchDirection = value; }
		}

		/// <summary>
		/// Minimize the optimization problem P.
		/// </summary>
		/// <param name="P"></param>
		public abstract void Minimize(Problem P);
	};
}
