// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: NonLinearLeastSquare.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Default least square method using a given optimization algorithm 
	/// (default is conjugate gradient).
	/// </summary>
	/// <remarks>
	/// min { r(x) : x in R^n }
	/// 
	/// where r(x) = ||f(x)||^2 the euclidian norm of f(x)
	/// for some vector-valued function f from R^n to R^m
	/// f = (f1, ..., fm) with fi(x) = bi - phi(x,ti)
	/// where bi is the vector of target data and phi
	/// is a scalar function.
	/// 
	/// Assuming the differentiability of f, the gradient of r
	/// is defined by grad r(x) = f'(x)^t.f(x)
	/// </remarks>
	[ RcsId("$Id: NonLinearLeastSquare.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Default least square method using a given optimization algorithm.") ]
	public class NonLinearLeastSquare 
	{
		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="constraint"></param>
		public NonLinearLeastSquare(Constraint constraint)
			: this(constraint, 1e-4, 100)
		{}

		public NonLinearLeastSquare(Constraint constraint, double accuracy, 
			int maxIterations) 
			: this(constraint, accuracy, maxIterations, new ConjugateGradient())
		{}

		public NonLinearLeastSquare(Constraint constraint, double accuracy,
			int maxIterations, Method method)
		{
			this.method = method;
			this.constraint = constraint;

			// set end criteria with a given maximum number of iteration
			// and a given error eps
			method.EndCriteria = new EndCriteria(maxIterations, accuracy);
			method.EndCriteria.PositiveOptimization = true;
		}

		//! Optimization method
		private Method method;
		//constraint
		private Constraint constraint;

		public Method Method
		{
			get { return method; }
		}

		public Constraint Constraint
		{
			get { return constraint; }
		}

		public Vector InitialValue
		{
			get { return method.InitialValue; }
			set { method.InitialValue = value; }
		}

		//! return the results
		public Vector Results
		{
			get { return method.X; }
		}
	
		//! return the least square residual norm
		public double ResidualNorm
		{
			get { return method.FunctionValue; }
		}

		//! return last function value
		public double LastValue
		{
			get { return method.FunctionValue; }
		}

		//! return exit flag
		public EndCriteria.CriteriaType ExitFlag
		{ 
			get { return method.EndCriteria.Criteria; }
		}

		//! return the performed number of iterations
		public int Iterations
		{
			get { return method.Iterations; }
		}

		/// <summary>
		/// Solve least square problem using numerix solver.
		/// </summary>
		/// <param name="lsProblem"></param>
		/// <returns></returns>
		public Vector Perform(LeastSquareProblem lsProblem)
		{
			// wrap the least square problem in an optimization function
			LeastSquareFunction lsf = new LeastSquareFunction(lsProblem);

			// define optimization problem
			Problem P = new Problem(lsf, constraint, method);

			// minimize
			P.Minimize();

			return method.X;
		}
	}
}
