// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Problem.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;


namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Constrained optimization problem.
	/// </summary>
	[ RcsId("$Id: Problem.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Constrained optimization problem.") ]
	public class Problem 
	{
		/// <summary>
		/// Default constructor.
		/// </summary>
		/// <param name="costFunction">Function and its gradient vector.</param>
		/// <param name="constraint">Optimization Constraint.</param>
		/// <param name="method">Optimization method.</param>
		public Problem(CostFunction costFunction, Constraint constraint, 
			Method method)
		{
			this.costFunction = costFunction;
			this.constraint = constraint;
			this.method = method;
		}

		//! Unconstrained cost function
		private CostFunction costFunction;
		//! Constraint
		private Constraint constraint;
		//! constrained optimization method
		private Method method;

		/// <summary>
		/// Call cost function computation and increment evaluation counter.
		/// </summary>
		/// <param name="x"></param>
		/// <returns></returns>
		public double Value(Vector x)
		{
			method.FunctionEvaluations++;
			return costFunction.Value(x);
		}

		/// <summary>
		/// Call cost function gradient computation and increment
		///	evaluation counter.
		/// </summary>
		/// <param name="grad_f">On return, it contains the gradient.</param>
		public void Gradient(Vector grad_f, Vector x)
		{
			method.GradientEvaluations++;
			costFunction.Gradient(grad_f, x);
		}

		/// <summary>
		/// Call cost function computation and it gradient.
		/// </summary>
		/// <param name="grad_f">On return, it contains the gradient.</param>
		/// <param name="x"></param>
		/// <returns></returns>
		public double ValueAndGradient(Vector grad_f, Vector x)
		{
			method.FunctionEvaluations++;
			method.GradientEvaluations++;
			return costFunction.ValueAndGradient(grad_f, x);
		}

		/// <summary>
		/// Constrained optimization method.
		/// </summary>
		public Method Method 
		{
			get { return method; }
		}

		/// <summary>
		/// Constraint.
		/// </summary>
		public Constraint Constraint
		{
			get { return constraint; }
		}

		/// <summary>
		/// Cost function.
		/// </summary>
		public CostFunction CostFunction 
		{ 
			get { return costFunction; }
		}

		/// <summary>
		/// Minimization.
		/// </summary>
		public void Minimize() 
		{ 
			method.Minimize(this); 
		}

		/// <summary>
		/// Minimum value.
		/// </summary>
		public Vector MinimumValue 
		{ 
			get { return method.X; }
		}
	}
}
