// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: FlatForward.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.TermStructures
{
	
	/// <summary>
	/// Flat forward rate term structure.
	/// </summary>
	[ RcsId("$Id: FlatForward.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public class FlatForward : TermStructure, ITermStructure
		// ,ISpreadedTermStructure  TODO: make up own interface
	{
		/// <summary>
		/// Flat forward rate.
		/// </summary>
		private IMarketElement marketElement;

		private DateTime todaysDate, referenceDate;
		private IDayCounter dayCounter;

		/// <summary>
		/// 
		/// </summary>
		public FlatForward(
			DateTime todaysDate, DateTime referenceDate,
			double forwardRate, IDayCounter dayCounter) 
			: this(todaysDate, referenceDate,
				new SimpleMarketElement(forwardRate), dayCounter ) 
		{ }

		/// <summary>
		/// 
		/// </summary>
		/// <param name="marketElement">The added spread.</param>
		public FlatForward(
			DateTime todaysDate, DateTime referenceDate,
			IMarketElement marketElement, IDayCounter dayCounter
			) 
		{
			this.dayCounter = dayCounter;

			if( marketElement == null)
				throw new ArgumentNullException("marketElement", 
					"The MarketElement can not be null.");
			this.marketElement = marketElement;
			RegisterWith(marketElement);

			// normalize dates
			this.todaysDate = todaysDate.Date;	
			this.referenceDate = referenceDate.Date;
		}

		/// <summary>
		/// The <see cref="MarketElement"/> used as constant forward rate.
		/// </summary>
		public IMarketElement MarketElement 
		{
			get { return marketElement; }
			set 
			{ 
				if( value == null)
					throw new ArgumentNullException(); 
				UnregisterWith(marketElement);
				marketElement = value; 	
				RegisterWith(marketElement);
				OnChanged();
			}
		}

		/// <summary>
		/// The constant forward rate.
		/// </summary>
		public double ForwardRate 
		{
			get { return marketElement.Value; }
			set 
			{ 
				UnregisterWith(marketElement);
				// replace the market element -- or make sure
				// that we can write to it (TODO)
				marketElement = new SimpleMarketElement(value); 	
				RegisterWith(marketElement);
				OnChanged();
			}
		}

		/// <summary>
		/// Calculates the zero yield rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time.</returns>
		override protected double zeroYieldImpl(double time, bool extrapolate) 
		{
			return marketElement.Value;
		}

		/// <summary>
		/// Calculates the discount factor.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time</returns>
		override protected double discountImpl(double time, bool extrapolate)
		{
			return Math.Exp( -marketElement.Value*time );
		}

		/// <summary>
		/// Calculates the instantaneous forward rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The forward rate at the given time</returns>
		override protected double forwardImpl(double time, bool extrapolate) 
		{
			return marketElement.Value;
		}

		/// <summary>
		/// A stringified representation of this term structure.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{
			return String.Format(
				"FlatForward (todaysDate={0:d} constant rate={1})",	
				TodaysDate, marketElement.Value	
			);
		}

		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		override public DateTime MaxDate 
		{ 
			get { return DateTime.MaxValue; }
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		override public DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		override public DateTime ReferenceDate 
		{
			get { return referenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		override public IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}

	};


}
