// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ForwardSpreadedTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.7  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.6  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.5  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Business;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.TermStructures
{
	
	/// <summary>
	/// Term structure with an added spread on the instantaneous forward rate.
	/// </summary>
	/// <remarks>
	/// This term structure will remain linked to the original structure,
	/// i.e., any changes in the latter will be reflected in this 
	/// structure as well.
	/// </remarks>
	[ RcsId("$Id: ForwardSpreadedTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public class ForwardSpreadedTermStructure 
		: ForwardRateStructure, ISpreadedTermStructure
	{
		/// <summary>
		/// The original <see cref="TermStructure"/> which will
		/// remain linked.
		/// </summary>
		private ITermStructure originalCurve;

		/// <summary>
		/// Spread from the orinal curve.
		/// </summary>
		private IMarketElement marketElement;

		/// <summary>
		/// Create an unitialized ForwardSpreadedTermStructure.
		/// </summary>
		/// <remarks>
		/// This constructor is for compatibility with COM
		/// </remarks>
		public ForwardSpreadedTermStructure() 
			: this(null ) { }
		
		/// <summary>
		/// Link this ForwardSpreadedTermStructure to another 
		/// <see cref="TermStructure"/>.
		/// </summary>
		/// <param name="originalCurve">The original <see cref="TermStructure"/>.</param>
		public ForwardSpreadedTermStructure(ITermStructure originalCurve) 
			: this(originalCurve, 0.0 ) { }

		/// <summary>
		/// Link this ForwardSpreadedTermStructure to another 
		/// <see cref="TermStructure"/> with an added spread.
		/// </summary>
		/// <param name="originalCurve">The original <see cref="TermStructure"/>.</param>
		/// <param name="spread">The added spread.</param>
		public ForwardSpreadedTermStructure( ITermStructure originalCurve, double spread) 
			: this( originalCurve, new SimpleMarketElement(spread) )
		{}

		/// <summary>
		/// Link this ForwardSpreadedTermStructure to another 
		/// <see cref="TermStructure"/> with an added spread.
		/// </summary>
		/// <param name="originalCurve">The original <see cref="TermStructure"/>.</param>
		/// <param name="marketElement">The added spread.</param>
		public ForwardSpreadedTermStructure( ITermStructure originalCurve, 
			IMarketElement marketElement) 
		{
			this.originalCurve = originalCurve;
			RegisterWith(originalCurve);
			if( marketElement == null)
				throw new ArgumentNullException("marketElement");
			this.marketElement = marketElement;
			RegisterWith(marketElement);
		}

		/// <summary>
		/// The <see cref="MarketElement"/> added to the linked <see cref="TermStructure"/>.
		/// </summary>
		public IMarketElement MarketElement {
			get { return marketElement; }
			set { 
				if( value == null)
					throw new ArgumentNullException(); 
				UnregisterWith(marketElement);
				marketElement = value; 	
				RegisterWith(marketElement);
				OnChanged();
			}
		}

		/// <summary>
		/// The value of the <see cref="MarketElement"/> added to the linked <see cref="TermStructure"/>.
		/// </summary>
		public double Spread {
			get { return marketElement.Value; }
			set { 
				UnregisterWith(marketElement);
				marketElement = new SimpleMarketElement(value); 	
				RegisterWith(marketElement);
				OnChanged();
			}
		}

		/// <summary>
		/// The linked <see cref="TermStructure"/>.
		/// </summary>
		public ITermStructure OriginalCurve {
			get { return originalCurve; }
			set {
				if( value == null)
					throw new ArgumentNullException(); 
				UnregisterWith(originalCurve); // detach event handler
				originalCurve = value;
				RegisterWith(marketElement); // re-register
				OnChanged();
			}
		}

//		/// <summary>
//		/// Calculates the spreaded zero yield rate.
//		/// </summary>
//		/// <param name="time">Time from reference.</param>
//		/// <param name="extrapolate"></param>
//		/// <returns>The zero yield at the given time.</returns>
//		override protected double zeroYieldImpl(double time, bool extrapolate) {
//			return originalCurve.ZeroYield(time, extrapolate) + marketElement.Value;
//		}

		/// <summary>
		/// Calculates the spreaded instantaneous forward rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The forward rate at the given time.</returns>
		override protected double forwardImpl(double time, bool extrapolate) {
			return originalCurve.InstantaneousForward(time, extrapolate) + marketElement.Value;
		}

		/// <summary>
		/// A stringified representation of this term structure.
		/// </summary>
		/// <remarks>
		/// This method is used for output and comparison between indexes.
		///	It is <b>not</b> meant to be used for writing switch-on-type code.
		/// </remarks>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{
			return String.Format(
				"ForwardSpreadedTermStructure (spread={0}, originalCurve='{1}')",
				this.Spread, this.OriginalCurve
			);
		}

		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		override public DateTime MaxDate 
		{ 
			get { return originalCurve.MaxDate; }
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		override public DateTime TodaysDate {
			get { return originalCurve.TodaysDate; }
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		override public DateTime ReferenceDate {
			get { return originalCurve.ReferenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		override public IDayCounter DayCounter 
		{
			get { return originalCurve.DayCounter; }
		}

	};


}
