// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ITermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib
{

	/// <summary>
	/// Interface for concrete rate structures.
	/// Rates are assumed to be annual continuos compounding.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// </remarks>
	[ RcsId("$Id: ITermStructure.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface for concrete rate structures. " +
		"Rates are assumed to be annual continuos compounding.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface ITermStructure : IObservable {

		/// <summary>
		/// A stringified representation of the TermStructure.
		/// </summary>
		/// <remarks>
		/// This method is used for output and comparison.
		///	It is <b>not</b> meant to be used for writing switch-on-type code.
		/// </remarks>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		string ToString();

		/// <summary>
		/// Zero yield rate at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given date</returns>
		[ Description("Zero yield rate at a given date.") ]
		double ZeroYield( DateTime date, bool extrapolate);

		/// <summary>
		/// Zero yield rate at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Zero yield rate at a given time from reference.") ]
		double ZeroYield(double time, bool extrapolate);

		/// <summary>
		/// Discount factor at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given date.</returns>
		[ Description("Discount factor at a given date.") ]
		double Discount( DateTime date, bool extrapolate);


		/// <summary>
		/// Discount factor at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Discount factor at a given time from reference.") ]
		double Discount(double time, bool extrapolate);

		/// <summary>
		/// Instantaneous forward rate at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The instantaneous forward rate at the given date.</returns>
		[ Description("Instantaneous forward rate at a given date.") ]
		double InstantaneousForward( DateTime date, bool extrapolate);

		
		/// <summary>
		/// Instantaneous forward rate at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The instantaneous forward rate at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Instantaneous forward rate at a given time from reference.") ]
		double InstantaneousForward(double time, bool extrapolate);

		/// <summary>
		/// Discrete forward rate between two dates.
		/// </summary>
		/// <param name="date1">The start date of the period described by the two 
		/// <see cref="DateTime"/> parameters.</param>
		/// <param name="date2">The end date of the period described by the two 
		/// <see cref="DateTime"/> parameters.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discrete forward rate between the given dates.</returns>
		[ Description("Discrete forward rate between two dates.") ]
		double Forward( DateTime date1, DateTime date2, bool extrapolate);

		/// <summary>
		/// Discrete forward rate between two times.
		/// </summary>
		/// <param name="time1">The start time of the period.</param>
		/// <param name="time2">The end time of the period.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discrete forward rate between the given times.</returns>
		[ ComVisible(false) ]
		[ Description("Discrete forward rate between two times.") ]
		double Forward(double time1, double time2, bool extrapolate);

		/// <summary>
		/// The latest time for which the curve can return rates.
		/// </summary>
		double MaxTime {
			get;
		}
		
		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime MaxDate {
			get;
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime TodaysDate {
			get;
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime ReferenceDate {
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		IDayCounter DayCounter 
		{
			get;
		}


	}

}