// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RateHelper.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/07/31 18:47:47  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// Abstract base class for rate helpers.
	/// </summary>
	/// <remarks>
	/// This class provides an abstraction for the instruments used to
	/// bootstrap a term structure. 
	/// It is advised that a rate helper for an instrument contains an
	/// instance of the actual instrument class to ensure consistancy
	/// between the algorithms used during bootstrapping and later
	/// instrument pricing. This is not yet fully enforced in the
	/// available rate helpers, though - only <see cref="SwapRateHelper"/>
	/// contains a Swap instrument for the time being.
	/// RateHelper implements the <see cref="System.IComparable"/> interface. 
	/// The established sorting order only considers the time to maturity as 
	/// derived from the abstract Maturity() method.
	/// </remarks>
	[ RcsId("$Id: RateHelper.cs,v 1.1.1.1 2003/02/01 19:01:38 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class RateHelper : Observable, IRateHelper, IComparable
	{
		private IMarketElement marketElement;
		private ITermStructure termStructure;

		protected RateHelper(double quote)
			: this(new SimpleMarketElement(quote))
		{}

		protected RateHelper(IMarketElement marketElement)
		{
			if( marketElement == null)
				throw new ArgumentNullException("marketElement");
			this.marketElement = marketElement;
			termStructure = null;
		}

		abstract protected void initializeHelper(ITermStructure termStructure);

		#region Implementation of IRateHelper

		public virtual double DiscountGuess()
		{
			// Null<double> is QL_MAX_DOUBLE
			// TODO: I don't like that... maybe throw something...?
			return Double.MaxValue;
		}
		public double QuoteError()
		{
			return marketElement.Value - ImpliedQuote();
		}

		abstract public double ImpliedQuote();
		abstract public DateTime Maturity();

		public ITermStructure TermStructure 
		{
			get 
			{
				return termStructure;
			}
			set 
			{
				if( value == null)
					throw new ArgumentNullException(); 
				termStructure = value;
				initializeHelper(termStructure);
			}
		}

		public double Quote
		{
			get { return marketElement.Value; }
		}

		#endregion

		#region Implementation of IComparable
		/// <summary>
		/// Compares the current instance with another object of the same type.
		/// The established sorting order only considers the time to maturity 
		/// as derived from the abstract Maturity() method.
		/// </summary>
		/// <exception cref="System.InvalidOperationException">
		///		Signalling that two Rate helpers have the same maturity.
		/// </exception>
		/// <seealso>QuantLib.Tests.TestRateHelper.TestIComparable</seealso>
		/// <param name="obj">
		/// An object of type IRateHelper to compare with this instance.
		/// </param>
		/// <returns>
		/// A 32-bit signed integer that indicates the relative order of the 
		/// comparands. The return value has these meanings:
		/// <list type="table">
		/// <listheader>
		///   <term>Value </term>
		///   <description>Meaning </description>
		/// </listheader>
		/// <item>
		///   <term>Less than zero </term>
		///   <description>This instance is less than obj. </description>
		/// </item>
		/// <item>
		///   <term>Greater than zero </term>
		///   <description>This instance is greater than obj. </description>
		/// </item>
		/// </list>
		/// </returns>
		public int CompareTo(object obj)
		{
			if( ! (obj is IRateHelper) )
				throw new ArgumentException(
					Strings.Get(GetType(), "RhCmpCast") );
			IRateHelper other = (IRateHelper)obj;
			DateTime otherMaturity = other.Maturity();
			DateTime thisMaturity = this.Maturity();
			if (thisMaturity < otherMaturity) 
				return -1;
			else if (thisMaturity > otherMaturity) 
				return 1;
			else throw new InvalidOperationException( 
				Strings.Get(GetType(), "RhCmpMaturity" ));

		}
		#endregion

	}
}
