// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SwapRateHelper.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.QuantLib.Instruments;
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.TermStructures
{

	/// <summary>
	/// Swap rate.
	/// </summary>
	/// <remarks>
	/// This class assumes that the reference date does not change
	/// between assignments to <see cref="TermStructure"/>.
	/// </remarks>
	[ RcsId("$Id: SwapRateHelper.cs,v 1.1.1.1 2003/02/01 19:01:38 jens Exp $") ] 
	[ ComVisible(false) ]
	public class SwapRateHelper : RateHelper, IRateHelper
	{

		public SwapRateHelper( double quote, 
			int numberOfUnits, TimeUnit units, int settlementDays,
			IBusinessCalendar calendar, RollingConvention convention,
			int fixedFrequency, bool fixedIsAdjusted,
			IDayCounter fixedDayCounter, int floatingFrequency 
		) : this( new SimpleMarketElement(quote), numberOfUnits, units, 
			settlementDays, calendar, convention, 
			fixedFrequency, fixedIsAdjusted, fixedDayCounter, 
			floatingFrequency)
		{}

		public SwapRateHelper( IMarketElement marketElement,
			int numberOfUnits, TimeUnit units, int settlementDays,
			IBusinessCalendar calendar, RollingConvention convention,
			int fixedFrequency, bool fixedIsAdjusted,
			IDayCounter fixedDayCounter, int floatingFrequency
		) : base(marketElement)
		{
			this.numberOfUnits = numberOfUnits;
			this.units = units;
			this.settlementDays = settlementDays;
			this.calendar = calendar;
			this.convention = convention;
			this.fixedFrequency = fixedFrequency;
			this.fixedIsAdjusted = fixedIsAdjusted;
			this.fixedDayCounter = fixedDayCounter;
			this.floatingFrequency = floatingFrequency;
		}

		private int settlementDays;
		private int numberOfUnits;
		private TimeUnit units;
		private IBusinessCalendar calendar;
		private RollingConvention convention;
		private int fixedFrequency;
		private bool fixedIsAdjusted;
		private IDayCounter fixedDayCounter;
		private int floatingFrequency;
		private DateTime settlementDate;
		private SimpleSwap swap;

		/// <summary>
		/// Called everytime a new term structure is set. 
		/// </summary>
		/// <remarks>
		/// The new term structure is not null.
		/// </remarks>
		/// <param name="termStructure">The new term structure.</param>
		override protected void initializeHelper(ITermStructure termStructure)
		{
			settlementDate = calendar.Advance(
				termStructure.TodaysDate, settlementDays, TimeUnit.Days,
				RollingConvention.Following // TODO: always 'Following' ???
				);

            // dummy Libor index with curve/swap arguments
            IXibor dummyIndex = new Xibor("dummy",
                12/floatingFrequency, TimeUnit.Months, settlementDays,
                Currency.EUR, // any would do
                calendar, true, convention, termStructure.DayCounter,
                termStructure);
            swap = new SimpleSwap(true,	// pay fixed rate
                settlementDate, numberOfUnits, units, calendar,
                convention, 100.0, 
				fixedFrequency, 0.0, fixedIsAdjusted, fixedDayCounter,
                floatingFrequency, dummyIndex, settlementDays, 0.0,
                termStructure);
        }

		/// <summary>
		/// A stringified representation of the rate helper.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public string ToString()
		{
			return String.Format(
				"SwapRateHelper (settlementDate={0:d}, period={1}, quote={2})",
				settlementDate, new Period(numberOfUnits, units), Quote);
		}

		override public double ImpliedQuote() 
		{
			if( TermStructure == null) 
				; // throws an error if not
			// we didn't register as observers - force calculation
			swap.Recalculate();
			return swap.FairRate;
		}

		// implementing discountGuess() is not worthwhile,
		// and may not avoid the root-finding process

		// TODO: not implemented, the base class ......

		override public DateTime Maturity()
		{
			if( TermStructure == null) 
				; // throws an error if not
			return swap.Maturity;
		}

	}
	
}
