// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.TermStructures {

	/// <summary>
	/// This abstract class defines the base of concrete
	/// rate structures which will be derived from this one.
	/// Rates are assumed to be annual continuos compounding.
	/// </summary>
	[ RcsId("$Id: TermStructure.cs,v 1.1.1.1 2003/02/01 19:01:38 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class TermStructure : Observable, ITermStructure 
	{	

		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected TermStructure()
		{}

		/// <summary>
		/// Zero yield rate at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given date.</returns>
		public double ZeroYield( DateTime date, bool extrapolate) {
			return zeroYieldImpl( date2time(date), extrapolate );
		}

		/// <summary>
		/// Zero yield rate at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time.</returns>
		public double ZeroYield(double time, bool extrapolate) {
			return zeroYieldImpl( time, extrapolate );
		}

		/// <summary>
		/// Discount factor at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given date.</returns>
		public double Discount( DateTime date, bool extrapolate) 
		{
			return discountImpl( date2time(date), extrapolate );
		}

		/// <summary>
		/// Discount factor at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time.</returns>
		public double Discount(double time, bool extrapolate) 
		{
			return discountImpl( time, extrapolate );
		}

		/// <summary>
		/// Instantaneous forward rate at a given date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The instantaneous forward rate at the given date.</returns>
		public double InstantaneousForward( DateTime date, bool extrapolate) 
		{
			return forwardImpl( date2time(date), extrapolate );
		}

		/// <summary>
		/// Instantaneous forward rate at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The instantaneous forward rate at the given time.</returns>
		public double InstantaneousForward(double time, bool extrapolate) 
		{
			return forwardImpl( time, extrapolate );
		}

		/// <summary>
		/// Discrete forward rate between two dates.
		/// </summary>
		/// <param name="date1">The start date of the period described by the two 
		/// <see cref="DateTime"/> parameters.</param>
		/// <param name="date2">The end date of the period described by the two 
		/// <see cref="DateTime"/> parameters.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discrete forward rate between the given dates.</returns>
		public double Forward( DateTime date1, DateTime date2, bool extrapolate) 
		{
			return Forward(date2time(date1), date2time(date2), extrapolate);
		}

		/// <summary>
		/// Discrete forward rate between two times.
		/// </summary>
		/// <param name="time1">The start time of the period.</param>
		/// <param name="time2">The end time of the period.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discrete forward rate between the given times.</returns>
		public double Forward(double time1, double time2, bool extrapolate) 
		{
			if (time1==time2)
				return forwardImpl(time1, extrapolate);
			else 
				return Math.Log( discountImpl(time1,extrapolate)/
					discountImpl(time2,extrapolate)  
					) / (time2-time1);
		}

		/// <summary>
		/// The latest time for which the curve can return rates.
		/// </summary>
		public virtual double MaxTime {
			get{ return date2time(MaxDate); }
		}
		
		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime MaxDate 
		{
			get;
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime TodaysDate 
		{
			get;
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		/// <remarks>
		/// This is the date where Discount == 1.0.
		/// </remarks>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime ReferenceDate 
		{
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public abstract IDayCounter DayCounter 
		{
			get;
		}

		/// <summary>
		/// Implements the actual zero yield calculation in derived classes.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time.</returns>
		protected abstract double zeroYieldImpl(double time, bool extrapolate);

		/// <summary>
		/// Implements the actual discount calculation in derived classes.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time.</returns>
		protected abstract double discountImpl(double time, bool extrapolate);

		/// <summary>
		/// Implements the actual instantaneous forward rate calculation in 
		/// derived classes.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The forward rate at the given time.</returns>
		protected abstract double forwardImpl(double time, bool extrapolate);

		/// <summary>
		/// Convert a <see cref="DateTime"/> to continous time
		/// since reference.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to convert.</param>
		/// <returns>Continous time since reference date.</returns>
		private double date2time(DateTime date) {
			// TODO: check for valid dates eg. reference <= d <= maxdatetime
			// TODO: interpret certain dates (eg. serial <= 100) as time
			return DayCounter.YearFraction(ReferenceDate,date);
		}

	}

}