// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestSimpleSwap.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:23:26  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:15:20  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.QuantLib.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Business;
	using Stochastix.Business.Calendars;
	using Stochastix.Business.DayCounters;
	using Stochastix.QuantLib.TermStructures;
	using Stochastix.QuantLib.MarketElements;
	using Stochastix.QuantLib.Indexes;
	using Stochastix.QuantLib.Instruments;


	/// <summary>
	/// 
	/// </summary>
	public class TestSimpleSwap: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestSimpleSwap(String name): base(name) {}

		// declare private members here used for test setup

		private DateTime todaysDate = new DateTime(2002, 6, 17);
		private IBusinessCalendar calendar = TARGET.Instance;
		private const int settlementDays = 2;
		private const int fixingDays = 2;
		private DateTime settlementDate;
		private const RollingConvention rollingConvention = 
			RollingConvention.ModifiedFollowing;
		private const bool payFixed = true;
		private const double nominal = 100.0;
		private const int fixedFrequency = 1; 
		private const int floatingFrequency = 2;
		private IDayCounter fixedDayCount = Thirty360US.Instance;
		private const bool fixedIsAdjusted = false;

		private ITermStructure termStructure;
		private IXibor euribor;

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			settlementDate = calendar.Advance( 
				todaysDate, settlementDays, TimeUnit.Days, 
				RollingConvention.Following);
			termStructure = new FlatForward(todaysDate, settlementDate, 
				0.05, Actual365.Instance);
//			euribor = new Euribor("Euribor", 12/floatingFrequency, 
//				TimeUnit.Months, termStructure);
			euribor = new Xibor("Euribor", 12/floatingFrequency, TimeUnit.Months,
                2, Currency.EUR, TARGET.Instance, true, 
				RollingConvention.ModifiedFollowing, Actual360.Instance, 
				termStructure);
		}
		
		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}


		// depends on the above setup
		public void TestCachedValue()
		{
			SimpleSwap swap = makeSwap(10, 0.06, 0.001);
			const double cachedNPV = -5.883663676727;
			Assert( Math.Abs( swap.NPV - cachedNPV ) <= 1.0e-11 );
		}

		private SimpleSwap makeSwap( int length, double fixedRate, double spread)
		{
			return new SimpleSwap( payFixed, settlementDate, 
				length, TimeUnit.Years, calendar, rollingConvention,
				nominal, fixedFrequency, fixedRate, fixedIsAdjusted,
				fixedDayCount, floatingFrequency, euribor, fixingDays,
				spread, termStructure, "", String.Format(
				"Test Case: {0}y fixed={1:p} vs. spread={2:p}",
				length, fixedRate, spread));
		}

		// Test data for the following tests

		private int[]    years   = { 1, 2, 5, 10, 20};
		private double[] spreads = { -0.01, -0.002, -0.001, 0.0, 0.001, 0.002, 0.01 };
		private double[] rates   = { 0.03, 0.04, 0.05, 0.06, 0.07 };

		public void TestFairRate()
		{
			SimpleSwap swap;
			double fixedRate = 0.0;
			foreach( int length in years)
			{
				foreach( double spread in spreads)
				{
					swap = makeSwap(length, fixedRate, spread);
					swap = makeSwap(length, swap.FairRate, spread);
					Assert( Math.Abs( swap.NPV ) <= 1e-10);
				}
			}
		}

		public void TestFairSpread()
		{
			SimpleSwap swap;
			double spread = 0.0;
			foreach( int length in years)
			{
				foreach( double rate in rates)
				{
					swap = makeSwap(length, rate, spread);
					swap = makeSwap(length, rate, swap.FairSpread);
					Assert( Math.Abs( swap.NPV ) <= 1e-10);
				}
			}
		}

		public void TestRateDependency()
		{
			SimpleSwap swap;
			foreach( int length in years)
			{
				foreach( double spread in spreads)
				{
					ArrayList values = new ArrayList();
					foreach( double rate in rates)
					{
						swap = makeSwap(length, rate, spread);
						values.Add( swap.NPV);
					}
					
					double lastval = double.MaxValue;
					foreach(double val in values)
					{
						// We're paying fixed - NPV must decrease with rate
						Assert(val < lastval);
						lastval = val;
					}
				}
			}
		}

		public void TestSpreadDependency()
		{
			SimpleSwap swap;
			foreach( int length in years)
			{
				foreach( double rate in rates)
				{
					ArrayList values = new ArrayList();
					foreach( double spread in spreads)
					{
						swap = makeSwap(length, rate, spread);
						values.Add( swap.NPV );
					}
					
					double lastval = double.MinValue;
					foreach(double val in values)
					{
						// We're paying fixed - NPV must increase with spread
						Assert(val > lastval);
						lastval = val;
					}
				}
			}
		}

	}
}
