// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BlackVarianceSurface.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Business;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Numerics;
using Stochastix.Numerics.Interpolations;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Black volatility surface modelled as variance surface.
	/// </summary>
	/// <remarks>
	/// This class calculates time/strike dependant Black volatilities
	/// using as input a matrix of Black volatilities
	/// observed in the market.
	/// <para>
	/// The calculation is performed interpolating on the variance surface.
	/// </para>
	/// <para>
	/// For strike dependance see <see cref="BlackVarianceSurface"/>.
	/// </para>
	/// <para>
	/// Volatilities are assumed to be expressed on an annual basis.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: BlackVarianceSurface.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(true) ]
	public class BlackVarianceSurface : BlackVarianceTermStructure
	{

		public enum Extrapolation 
		{
			Constant,
			Default 
		};

		/// <summary>
		/// Constructor for BlackVarianceSurface class.
		/// </summary>
		public BlackVarianceSurface(DateTime referenceDate,
			IList dates, IList strikes, Matrix blackVolMatrix) 
			: this(referenceDate, dates, strikes, blackVolMatrix, 
				Extrapolation.Default, Extrapolation.Default)
		{}

		/// <summary>
		/// Constructor for BlackVarianceSurface class.
		/// </summary>
		public BlackVarianceSurface(DateTime referenceDate,
			IList dates, IList strikes, Matrix blackVolMatrix,
			Extrapolation lowerExtrapolation, Extrapolation upperExtrapolation) 
			: this(referenceDate, dates, strikes, blackVolMatrix, 
				lowerExtrapolation, upperExtrapolation, Actual365.Instance)
		{}

		/// <summary>
		/// Constructor for BlackVarianceSurface class.
		/// </summary>
		public BlackVarianceSurface(DateTime referenceDate,
			IList dates, IList strikes, Matrix blackVolMatrix,
			Extrapolation lowerExtrapolation, Extrapolation upperExtrapolation,
			IDayCounter dayCounter) 
			: this(referenceDate, dates, strikes, blackVolMatrix, 
				lowerExtrapolation, upperExtrapolation, dayCounter,
				new BilinearInterpolation() )
		{}

		/// <summary>
		/// Constructor for BlackVarianceSurface class.
		/// </summary>
		public BlackVarianceSurface(DateTime referenceDate,
			IList dates, IList strikes, Matrix blackVolMatrix,
			Extrapolation lowerExtrapolation, Extrapolation upperExtrapolation,
			IDayCounter dayCounter, Interpolation2D interpolator) : base()
		{
			if( dates.Count != blackVolMatrix.Columns )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: size mismatch between date vector and vol matrix colums" ), 
					"blackVolMatrix" );
			if( strikes.Count != blackVolMatrix.Rows )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: size mismatch between money-strike vector and vol matrix rows" ), 
					"blackVolMatrix" );

            // cannot have dates[0]==referenceDate, since the
            // value of the vol at dates[0] would be lost
            // (variance at referenceDate must be zero)
			if( (DateTime)dates[0] <= referenceDate )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: cannot have dates[0]<=referenceDate" ), "dates" );

			this.referenceDate = referenceDate;
			this.dayCounter = dayCounter;

			this.lowerExtrapolation = lowerExtrapolation;
			this.upperExtrapolation = upperExtrapolation;

			this.times     = new double[ dates.Count ];
			this.strikes   = strikes;
			this.variances = new Matrix(strikes.Count, dates.Count);

			for( int j=0; j<blackVolMatrix.Columns; j++)
			{
				times[j] = date2time( (DateTime)dates[j] );
				if( j>0 && times[j-1]>=times[j] )
					throw new ArgumentException( Strings.Get( GetType(),
						"TODO: dates must be sorted unique!" ), "dates" );
				for (int i=0; i<blackVolMatrix.Rows; i++) 
				{
					double vol = blackVolMatrix[i,j];
					variances[i,j] = times[j] * vol*vol;
				}
			}

			this.maxDate = (DateTime)dates[dates.Count-1];

			this.varianceSurface = interpolator;
			this.varianceSurface.Initialize(times, strikes, variances);
		}

		private DateTime referenceDate;
		private IDayCounter dayCounter;
		private DateTime maxDate;
		private double[] times = null;
		private IList strikes = null;
		private Matrix variances;
		private Interpolation2D varianceSurface;
		private Extrapolation lowerExtrapolation = Extrapolation.Default;
		private Extrapolation upperExtrapolation = Extrapolation.Default;

		/// <summary>
		/// Black present (a.k.a spot) variance at a given time from reference.
		/// </summary>
		/// <overloads>
		/// Black present (a.k.a spot) variance.
		/// </overloads>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike">(not used)</param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given time.</returns>
		public override double Variance( double maturity, double strike, 
			bool extrapolate)
		{
			if( maturity < 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: negative time not allowed." ), "maturity" );

			// enforce constant extrapolation when required
			if (strike < (double)strikes[0] && extrapolate
				&& lowerExtrapolation == Extrapolation.Constant)
				strike = (double)strikes[0];
			if (strike > (double)strikes[strikes.Count-1] && extrapolate
				&& upperExtrapolation == Extrapolation.Constant)
				strike = (double)strikes[strikes.Count-1];

			if( maturity <= times[0] )
			{
				return varianceSurface.Value(times[0], strike, extrapolate) * 
					maturity/times[0];
			}
			else if ( maturity <= MaxTime )
			{
				return varianceSurface.Value(maturity, strike, extrapolate);
			}
			else // maturity>MaxTime
			{
				if( !extrapolate )
					throw new InvalidOperationException( Strings.Format( GetType(),
						"TODO: maturity ({0}) > MaxTime ({1}) without extrapolation",
						maturity, MaxTime) );

				return varianceSurface.Value(MaxTime, strike, extrapolate) * 
					maturity/MaxTime;
			}

		}



		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime MaxDate 
		{
			get { return maxDate; }
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime ReferenceDate 
		{
			get { return referenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public override IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}


	}
}
