// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CapFlatVolatilityVector.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Business;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Numerics.Interpolations;

namespace Stochastix.QuantLib.Volatilities
{
	/// <summary>
	/// Cap/floor at-the-money flat volatility vector.
	/// </summary>
	/// <remarks>
	/// This class provides the at-the-money volatility for a given 
	/// cap by interpolating a volatility vector whose elements are 
	/// the market volatilities of a set of caps/floors with given length.
	/// <para>
	/// Volatilities are assumed to be expressed on an annual basis.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: CapFlatVolatilityVector.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(true) ]
	public class CapFlatVolatilityVector : CapFlatVolatilityStructure
	{
		/// <summary>
		/// Constructor for CapFlatVolatilityVector class.
		/// </summary>
		public CapFlatVolatilityVector(DateTime todaysDate, IBusinessCalendar calendar,
			int settlementDays, IList lengths, IList volatilities) 
			: this(todaysDate, calendar, settlementDays, lengths, volatilities,
				Actual365.Instance)
		{}

		/// <summary>
		/// Constructor for CapFlatVolatilityVector class.
		/// </summary>
		public CapFlatVolatilityVector(DateTime todaysDate, IBusinessCalendar calendar,
			int settlementDays, IList lengths, IList volatilities, 
			IDayCounter dayCounter) : base()
		{
			if( lengths.Count != volatilities.Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: size mismatch between lengths and volatility vectors" ), 
					"volatilities" );

			this.todaysDate = todaysDate;
			this.dayCounter = dayCounter;
			this.settlementDate = calendar.Advance(todaysDate,
				settlementDays, TimeUnit.Days, RollingConvention.Following);

			this.times     = new double[ lengths.Count+1 ];
			this.volatilities = new double[ volatilities.Count+1 ];

			// this.times[0] = 0.0;
			this.volatilities[0] = (double)volatilities[0];

			for( int i=1; i<this.times.Length; i++)
			{
				this.times[i] = date2time( SettlementDate+(Period)lengths[i-1] );
				this.volatilities[i] = (double)volatilities[i-1];
			}

			// this.maxDate = (DateTime)dates[dates.Count-1];

			this.interpolation = new LinearInterpolation(times, volatilities);

		}

		private DateTime todaysDate, settlementDate;
		private IDayCounter dayCounter;
		// private DateTime maxDate;
		private double[] times = null;
		private double[] volatilities = null;
		private Interpolation interpolation;

		/// <summary>
		/// Volatility for a given end time and strike rate.
		/// </summary>
		/// <overloads>
		/// Volatility.
		/// </overloads>
		/// <param name="time">End time from settlement date.</param>
		/// <param name="strike">(not used)</param>
		/// <returns>The volatility expressed on an annual basis.</returns>
		public override double Volatility(double time, double strike)
		{
			return interpolation.Value(time, false);
		}


//		/// <summary>
//		/// The latest date for which the term structure can return vols.
//		/// </summary>
//		/// <value><see cref="DateTime"/></value>
//		public override DateTime MaxDate 
//		{
//			get { return maxDate; }
//		}

		/// <summary>
		/// Today's date.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// Settlement date.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime SettlementDate 
		{
			get { return settlementDate; }
		}


		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public override IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}


	}
}
