// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: LocalVolatilityCurve.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:40  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Local volatility curve derived from a Black curve.
	/// </summary>
	/// <remarks>
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	// TODO: A LocalVarianceSurface with time/asset dependence.
	[ RcsId("$Id: LocalVolatilityCurve.cs,v 1.1.1.1 2003/02/01 19:01:40 jens Exp $") ] 
	public class LocalVolatilityCurve : LocalVolTermStructure
	{

		public LocalVolatilityCurve( BlackVarianceCurve blackVarianceCurve )
			:base()
		{
			this.blackVarianceCurve = blackVarianceCurve;
		}

		private BlackVarianceCurve blackVarianceCurve;

		/*! The relation 
			\f[
			\int_0^T \sigma_L^2(t)dt = \sigma_B^2 T
			\f]
			holds, where \f$ \sigma_L(t) \f$ is the local volatility at
			time \f$ t \f$ and \f$ \sigma_B(T) \f$ is the Black
			volatility for maturity \f$ T \f$. From the above, the formula
			\f[
			\sigma_L(t) = \sqrt{\frac{\mathrm{d}}{\mathrm{d}t}\sigma_B^2(t)t}
			\f]
			can be deduced which is here implemented.
		*/

		/// <summary>
		/// Local volatility at a given time from reference.
		/// </summary>
		/// <overloads>
		/// Local volatility.
		/// </overloads>
		/// <param name="time">Time from reference.</param>
		/// <param name="underlyingLevel">(not used)</param>
		/// <param name="extrapolate"></param>
		/// <returns>The local volatility at the given time.</returns>
		public override double Volatility(double time, double underlyingLevel, 
			bool extrapolate)
		{
			const double dT = 1.0 / 365.0;
			double var1 = blackVarianceCurve.Variance(time, underlyingLevel, extrapolate);
			double var2 = blackVarianceCurve.Variance(time+dT, underlyingLevel, true);
			return Math.Sqrt( (var2-var1) / dT );
		}

		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime MaxDate 
		{
			get { return blackVarianceCurve.MaxDate; }
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime ReferenceDate 
		{
			get { return blackVarianceCurve.ReferenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public override IDayCounter DayCounter 
		{
			get { return blackVarianceCurve.DayCounter; }
		}



	}


}
