// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SwaptionVolatilityStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:40  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Swaption volatility structure abstract base class.
	/// </summary>
	/// <remarks>
	/// This class is purely abstract and defines the interface of concrete
	/// swaption volatility structures which will be derived from this one.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: SwaptionVolatilityStructure.cs,v 1.1.1.1 2003/02/01 19:01:40 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class SwaptionVolatilityStructure : Observable  // , ISwaptionVolatilityStructure
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected SwaptionVolatilityStructure()
		{}

		/// <summary>
		/// Swaption volatility for a given starting time and length.
		/// </summary>
		/// <overloads>
		/// Swaption volatility.
		/// </overloads>
		/// <remarks>
		/// This method must be overridden to implement the actual volatility
		/// calculation in derived classes.
		/// </remarks>
		/// <param name="startTime">Start time from today's date.</param>
		/// <param name="length"></param>
		/// <param name="strike"></param>
		/// <returns>The swaption volatility expressed on an annual basis.</returns>
		[ ComVisible(false) ]
		[ Description("Swaption volatility for a given starting time and length.") ]
		public abstract double Volatility(double startTime, double length, double strike);

		/// <summary>
		/// Swaption volatility for a given starting date and time period.
		/// </summary>
		/// <param name="start">Start time from today's date.</param>
		/// <param name="length">Time <see cref="Period"/>.</param>
		/// <param name="strike"></param>
		/// <returns>The swaption volatility expressed on an annual basis.</returns>
		[ Description("Local volatility at a given date.") ]
		public double Volatility( DateTime start, Period length, double strike)
		{
			double l = DayCounter.YearFraction(start, start+length);
			return Volatility( date2time(start), l, strike);
		}

		/// <summary>
		/// Convert a <see cref="DateTime"/> to continous time
		/// since today's date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to convert.</param>
		/// <returns>Continous time since today's date.</returns>
		protected double date2time(DateTime date) 
		{
			// TODO: check for valid dates eg. reference <= d <= maxdatetime
			// TODO: interpret certain dates (eg. serial <= 100) as time
			return DayCounter.YearFraction(TodaysDate, date);
		}

//		/// <summary>
//		/// The latest time for which the term structure can return vols.
//		/// </summary>
//		public virtual double MaxTime 
//		{
//			get{ return date2time(MaxDate); }
//		}
//		
//		/// <summary>
//		/// The latest date for which the term structure can return vols.
//		/// </summary>
//		/// <value><see cref="DateTime"/></value>
//		public abstract DateTime MaxDate 
//		{
//			get;
//		}
//
		/// <summary>
		/// The date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime TodaysDate 
		{
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public abstract IDayCounter DayCounter 
		{
			get;
		}


	}
}
