// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BasicRng.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion


using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Random
{
	/// <summary>
	/// Abstract base for a basic random number generator.
	/// </summary>
	[ RcsId("$Id: BasicRng.cs,v 1.1.1.1 2003/02/01 19:02:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class BasicRng : IBasicRng
	{
		#region Implementation of ICloneable
		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		/// <returns>
		/// A new object that is a copy of this instance.
		/// </returns>
		public virtual object Clone()
		{
			return this.MemberwiseClone();
		}
		#endregion

		#region Implementation of IDisposable

		/// <summary>
		/// Performs application-defined tasks associated with freeing, 
		/// releasing, or resetting unmanaged resources.
		/// </summary>
		public void Dispose()
		{}
		#endregion

		/// <summary>
		/// Draw a random sample for the generator.
		/// </summary>
		/// <remarks>
		/// Must be overridden by derived classes with a concrete RNG.
		/// </remarks>
		/// <returns>
		/// A double-precision floating point number greater than or equal to 0.0, 
		/// and less than 1.0.
		/// </returns>
		protected abstract double Sample();

		#region Implementation of IBasicRng

		/// <summary>
		/// Returns a random number between 0.0 and 1.0.
		/// </summary>
		/// <returns>
		/// A double-precision floating point number greater than or equal to 0.0, 
		/// and less than 1.0.
		/// </returns>
		public virtual double NextDouble()
		{
			return Sample();
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		public void Next(double[] r)
		{
			Next(r, r.GetLowerBound(0), r.Length);
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		public void Next(double[] r, int start)
		{
			Next( r, start, r.GetUpperBound(0)-start+1);
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		/// <param name="length">Number of random numbers to generate.</param>
		public virtual void Next(double[] r, int start, int length)
		{
			for( length += start; start<length; start++)
				r[start] = Sample();
		}

		#endregion
	}
}
