// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CentralLimitGaussianRng.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;

namespace Stochastix.Random.Distributions
{
	/// <summary>
	/// Gaussian random number generator. 
	/// It uses the well-known fact that the sum of 12 uniform deviate
	/// in (-.5,.5) is approximately a Gaussian deviate with average 0
	/// and standard deviation 1.
	/// </summary>
	/// <remarks>
	/// The <see cref="BoxMullerGaussianRng"/> will be at least 6
	/// times faster because generation of random numbers tends to be
	/// expensive.
	/// </remarks>
	public sealed class CentralLimitGaussianRng : CachedContinousRng
	{
		/// <summary>
		/// Initialize a new CentralLimitGaussianRng.
		/// </summary>
		public CentralLimitGaussianRng( IBasicRng basicRng, double mean, double sigma)
		{
			this.basicRng = basicRng;
			this.mean = mean;
			this.sigma = sigma;
		}

		private IBasicRng basicRng;
		private double mean, sigma;

		/// <summary>
		/// Draw random samples from the generator.
		/// </summary>
		/// <param name="r">
		///		An <see cref="Array"/> to be filled with double-precision floating point 
		///		numbers.</param>
		/// <param name="start">
		///		The (zero-based) index of the first array element to fill.
		/// </param>
		/// <param name="length">
		///		The number of random samples to generate.
		/// </param>
		override protected void Generate(double[] r, int start, int length)
		{
			// we need 12 times the amount of required samples
			double[] drand = new double[length*12];
			basicRng.Next(drand);

			unsafe
			{
				fixed(double* pdrand = &drand[0], pr = &r[start])
				{
					// make pointer writable
					double* prw = pr, pdrandw = pdrand;
					for( int i=0; i<length; i++ )
					{
						double gauss = -6.0 +
							pdrandw[0] + pdrandw[1] +
							pdrandw[2] + pdrandw[3] +
							pdrandw[4] + pdrandw[5] +
							pdrandw[6] + pdrandw[7] +
							pdrandw[8] + pdrandw[9] +
							pdrandw[10] + pdrandw[11];
						pdrandw += 12;
						*prw++ = mean + gauss * sigma;
					}
				}
			}
		}


	}
}
