// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: MRG32k3a.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:54  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;

namespace Stochastix.Random.Generators
{
	/// <summary>
	/// Summary description for MRG32k3a.
	/// ]0..1[
	/// </summary>
	public sealed class MRG32k3a : BasicRng
	{
		private const double m1		= 4294967087.0;	// 2^32-209
		private const double m2		= 4294944443.0;	// 2^32-22853
		private const double a12	=    1403580.0;
		private const double a13n	=     810728.0;
		private const double a21	=     527612.0;
		private const double a23n	=    1370589.0;
		private const double norm	= 2.328306549295728e-10; // 1/(m1+1)

		// can use default clone with these state variables
		private double s10, s11, s12, s20, s21, s22;

		public MRG32k3a() : this(1)	// default seed is 1
		{}

		public MRG32k3a(int seed)
		{

			// An entirely adhoc way of seeding! 
			// This does **not** come from L'Ecuyer et al

			uint s = (uint)seed;

			if (s == 0)
				s = 1;	/* default seed is 1 */

			s = (69069*s) & 0xffffffffU;
			s10 = (double)(s%m1);
			s = (69069*s) & 0xffffffffU;
			s11 = (double)(s%m1);
			s = (69069*s) & 0xffffffffU;
			s12 = (double)(s%m1);

			s = (69069*s) & 0xffffffffU;
			s20 = (double)(s%m2);
			s = (69069*s) & 0xffffffffU;
			s21 = (double)(s%m2);
			s = (69069*s) & 0xffffffffU;
			s22 = (double)(s%m2);

			// "warm it up" with at least 3 calls to go 
			// through all the seed values

			Sample();
			Sample();
			Sample();
		}

		/// <summary>
		/// Draw a random sample for the generator.
		/// </summary>
		/// <returns>
		/// A double-precision floating point number greater than 0.0 
		/// and less than 1.0.
		/// </returns>
		override protected double Sample()
		{
			// Component 1
			double p1 = a12*s11 - a13n*s10;
			int k = (int)(p1/m1); p1 -= k*m1; // p1 %= m1;
			if (p1 < 0.0) p1 += m1;
			s10 = s11; s11 = s12; s12 = p1;
			// Component 1
			double p2 = a21*s22 - a23n*s20;
			k = (int)(p2/m2); p2 -= k*m2; //  p2 %= m2;
			if (p2 < 0.0) p2 += m2;
			s20 = s21; s21 = s22; s22 = p2;
			// Combination
			if( p1 <= p2 )
				return (p1-p2+m1) * norm;
			else
				return (p1-p2) * norm;
		}

	}
}
