// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RandomStreamFactory.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Random
{
	/// <summary>
	/// Factory to produce random streams for a named RNG.
	/// </summary>
	/// <remarks>
	/// Currently supported random number generators are:
	/// <list type="table">
	/// <item>
	///		<term>MRG32k3a</term>
	///		<description>...</description>
	///	</item>
	/// <item>
	///		<term>R250</term>
	///		<description>A shift-register random number generator.</description>
	/// </item>
	/// <item>
	///		<term>MCG31</term>
	///		<description>...</description></item>
	/// </list>
	/// The returned streams are either based on Intel VSL (if available) or 
	/// implemented directly in C#.
	/// </remarks>
	[ RcsId("$Id: RandomStreamFactory.cs,v 1.1.1.1 2003/02/01 19:02:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public sealed class RandomStreamFactory
	{
		private static RandomStreamFactory instance = 
			new RandomStreamFactory();

		/// <summary>
		/// Holds a single instance of this class.
		/// </summary>
		/// <example>
		/// [C#]
		/// <code>
		/// IRandomStream rng;
		/// rng = RandomStreamFactory.Instance.Create("R250", 4711);
		/// </code>
		/// </example>
		public static RandomStreamFactory Instance
		{
			get { return instance; }
			set { instance = value; }
		}

		/// <summary>
		/// Create a random stream for a named RNG.
		/// </summary>
		/// <param name="basicRngName">Name of the requested RNG.</param>
		/// <param name="seed">Initial seed.</param>
		/// <returns>A random stream based on the requested RNG.</returns>
		/// <exception cref="ArgumentException">
		/// Thrown when <paramref name="basicRngName"/> is an unknown random number generator.
		/// </exception>
		/// <remarks>
		/// See <see cref="RandomStreamFactory"/> for a description of supported generators.
		/// </remarks>
		/// <example>
		/// [C#]
		/// <code>
		/// IRandomStream rng;
		/// rng = RandomStreamFactory.Instance.Create("R250", 4711);
		/// </code>
		/// </example>
		public IRandomStream Create( String basicRngName, int seed)
		{
			basicRngName = basicRngName.ToLower();

			if( VSL.Version != null )
			{
				// generate VSL Stream
				VSL.BasicRng brng;
				if( basicRngName.Equals("mrg32k3a") )
					brng = VSL.BasicRng.VSL_BRNG_MRG32K3A;
				else if ( basicRngName.Equals("r250") )
					brng = VSL.BasicRng.VSL_BRNG_R250;
				else if ( basicRngName.Equals("mcg31") )
					brng = VSL.BasicRng.VSL_BRNG_MCG31;
				else
					throw new ArgumentException(
						"Unknown basic RNG: " + basicRngName, 
						"basicRngName");
				return new VslRandomStream(	VSL.NewStream((int)brng, seed) );

			} 
			else 
			{
				// generate CLR compatible stream
				IBasicRng basicRng;

				if( basicRngName.Equals("mrg32k3a") )
					basicRng = new Generators.MRG32k3a(seed);
				else if ( basicRngName.Equals("r250") )
					basicRng = new Generators.R250(seed);
				else if ( basicRngName.Equals("mcg31") )
					basicRng = new Generators.MCG31vsl(seed);
				else
					throw new ArgumentException(
						"Unknown basic RNG: " + basicRngName, 
						"basicRngName");
				return new SimpleRandomStream( basicRng );
			}
		}
	}
}
