// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Strings.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:46:15  stochastix
 *  updated license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Resources;

namespace Stochastix.Random.Resources
{
	/// <summary>
	/// Localized resource strings and limited message formatting.
	/// </summary>
	internal sealed class Strings
	{

		/// <summary>
		/// Holds the resource manager for this class.
		/// </summary>
		private static ResourceManager resourceManager;
		
		/// <summary>
		/// Static constructor initializing the underlying <see cref="ResourceManager"/>.
		/// </summary>
		/// <remarks>
		/// The Name of the resource is derived from the type of this class. 
		/// If the type name eg. is Something.Resources.Strings, the resource manager will
		/// try to find Something.Resources.Strings.resources or eg. a localized
		/// Something.Resources.Strings.de-DE.resources.
		/// </remarks>
		static Strings()
		{
			resourceManager = new ResourceManager( typeof(Strings) );
		}

		/// <summary>
		/// Gets the value of the String resource.
		/// </summary>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <returns>
		/// The value of the resource localized for the caller's current culture settings. 
		/// If a match is not possible, the message "No such resource: " with the given 
		/// <c>resourceKey</c> appended is returned.
		/// </returns>
		private static string Get( string resourceKey )
		{
			String message = resourceManager.GetString(resourceKey);
			if(message == null) message = "No such resource: " + resourceKey;
			return message;
		}

		/// <summary>
		/// Gets the value of the String resource with prepended error object information.
		/// </summary>
		/// <param name="type">The object responsible for this message. 
		/// Can be a string with the name, and object convertible to a string respresentation
		/// or null (Nothing in Visual Basic).
		/// </param>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <remarks>
		/// Internally uses the first overload of <see cref="Get"/> to lookup the resource.
		/// </remarks>
		/// <returns>
		/// The value of the resource localized for the caller's current culture settings,
		/// optionally prepended with the error object information.
		/// </returns>
		public static string Get( Type type, string resourceKey )
		{
			if( type == null) 
				return Get(resourceKey);
			else 
			{
				return String.Format("{0}: {1}", type.FullName, Get(resourceKey));
			}
		}

		/// <summary>
		/// Formats the value of the String resource passing the supplied arguments.
		/// </summary>
		/// <remarks>
		/// Internally uses the second overload of <see cref="Get"/> to lookup the resource,
		/// so the given <c>type</c> is prepended to the result.
		/// </remarks>
		/// <param name="type">The object responsible for this message. 
		/// Can be a string with the name, and object convertible to a string respresentation
		/// or null (Nothing in Visual Basic).
		/// </param>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <param name="arg0">The first argument that is passed to <see cref="String.Format"/>.</param>
		/// <returns>
		/// The formatted value of the resource localized for the caller's current
		/// culture settings, optionally prepended with the error object information.
		/// </returns>
		public static string Format( Type type, string resourceKey, 
			object arg0)
		{
			return String.Format( Get(type, resourceKey), arg0);
		}

		/// <summary>
		/// Formats the value of the String resource passing the supplied arguments.
		/// </summary>
		/// <remarks>
		/// Internally uses the second overload of <see cref="Get"/> to lookup the resource,
		/// so the given <c>type</c> is prepended to the result.
		/// </remarks>
		/// <param name="type">The object responsible for this message. 
		/// Can be a string with the name, and object convertible to a string respresentation
		/// or null (Nothing in Visual Basic).
		/// </param>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <param name="arg0">The first argument that is passed to <see cref="String.Format"/>.</param>
		/// <param name="arg1">The second argument that is passed to <see cref="String.Format"/>.</param>
		/// <returns>
		/// The formatted value of the resource localized for the caller's current
		/// culture settings, optionally prepended with the error object information.
		/// </returns>
		public static string Format( Type type, string resourceKey, 
			object arg0, object arg1)
		{
			return String.Format( Get(type, resourceKey), arg0, arg1);
		}

		
		/// <summary>
		/// Formats the value of the String resource passing the supplied arguments.
		/// </summary>
		/// <remarks>
		/// Internally uses the second overload of <see cref="Get"/> to lookup the resource,
		/// so the given <c>type</c> is prepended to the result.
		/// </remarks>
		/// <param name="type">The object responsible for this message. 
		/// Can be a string with the name, and object convertible to a string respresentation
		/// or null (Nothing in Visual Basic).
		/// </param>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <param name="arg0">The first argument that is passed to <see cref="String.Format"/>.</param>
		/// <param name="arg1">The second argument that is passed to <see cref="String.Format"/>.</param>
		/// <param name="arg2">The third argument that is passed to <see cref="String.Format"/>.</param>
		/// <returns>
		/// The formatted value of the resource localized for the caller's current
		/// culture settings, optionally prepended with the error object information.
		/// </returns>
		public static string Format( Type type, string resourceKey, 
			object arg0, object arg1, object arg2)
		{
			return String.Format( Get(type, resourceKey), arg0, arg1, arg2);
		}

		/// <summary>
		/// Formats the value of the String resource passing the supplied arguments.
		/// </summary>
		/// <remarks>
		/// Internally uses the second overload of <see cref="Get"/> to lookup the resource,
		/// so the given <c>type</c> is prepended to the result.
		/// </remarks>
		/// <param name="type">The object responsible for this message. 
		/// Can be a string with the name, and object convertible to a string respresentation
		/// or null (Nothing in Visual Basic).
		/// </param>
		/// <param name="resourceKey">The name of the resource to get.</param>
		/// <param name="args">Arguments passed to <see cref="String.Format"/>.</param>
		/// <returns>
		/// The formatted value of the resource localized for the caller's current
		/// culture settings, optionally prepended with the error object information.
		/// </returns>
		public static string Format( Type type, string resourceKey, object[] args)
		{
			return String.Format( Get(type, resourceKey), args);
		}

	}
}
