// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Observable.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:03:25  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes

namespace Stochastix.Utilities
{
	/// <summary>
	/// Implements the Observer pattern using .NET events and delegates
	/// </summary>
	/// <example>
	/// [C#]
	/// <code>
	/// class Observer {
	///       private IObservable observed;
	///       public Observer(IObservable observed) {
	///           this.observed = observed;
	///           observed.Changed += new EventHandler(Notify);
	///       }
	///       private void Notify(object sender, EventArgs e) { 
	///			  // observed object changed
	///       }
	///       ~Observer {	// Detach the event and delete the reference
	///		      observed.Changed -= new EventHandler(Notify);
	///		      observed = null;
	///		  }
	///   }
	/// </code>
	/// [Visual Basic]
	/// <code>
	///   Class Observer {
	///       Private WithEvents observer as IObservable;
	///       Public Sub New(observed As IObservable)
	///           Me.observed = observed
	///           // AddHandler observed.Changed, AddressOf Me.Notify
	///       End Sub
	///       ' usually called observer_Changed
	///       Private Sub Notify(sender As Object, e As EventArgs) Handles observer.Changed
	///       End Sub
	/// </code>
	/// </example>
	[ ComVisible(false) ]
	public interface IObservable {
		/// <summary>
		/// The Changed event signals a change in this object. You can
		/// add your own <see cref="EventHandler"/> delegates
		/// to be notified of changes.
		/// </summary>
		event EventHandler Changed;
	}

	namespace COM 
	{
		/// <summary>
		/// This is for a COM backward-compatible event sink
		/// </summary>
		/// <remarks>
		/// See "Handling events from .NET Components in unmanaged Event Sinks" below.
		/// </remarks>
		/// <example>
		/// [C#]
		/// <code>
		///		[ComSourceInterfaces(typeof(IObserveableEvents))]
		///		public class Observable : IObservable {
		///			public event EventHandler Changed;
		///		}
		/// </code>
		/// </example>
		/// <seealso href="http://www.codeproject.com/dotnet/cominterop.asp#UnmanagedSinks">
		/// Handling events from .NET Components in unmanaged Event Sinks
		/// </seealso>
		[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsIDispatch) ]
		[ ComVisible(true) ]
		public interface IObserveableEvents 
		{
			/// <summary>
			/// The Changed event signals a change in this object.
			/// </summary>
			[ Description("The Changed event signals a change in this object.") ]
			void Changed();
		}
	}

	/// <summary>
	/// Abstract base class that implements an observable observer
	/// (that is, received changes will be passed on by default).
	/// </summary>
	/// <remarks>
	/// <para>
	/// The <see cref="OnChanged"/> overloads are used to signal our own 
	/// events to our listeners. They should never be called from outside
	/// the instance. 
	/// </para>
	/// <para>
	/// <see cref="Notify"/> is the event listener we will use to register and
	/// receive events from other classes. Derived classes should override
	/// this method to process received events. The default implementation
	/// forwards the received event to our registered listeners. 
	/// <see cref="Notify"/> should not be called directly by the class.
	/// </para>
	/// <para>
	/// Use <see cref="Update"/> to simulate events received from classes
	/// we are observing. This can be used to simulate a change to set eg. an
	/// initial dirty cache state. Events send by <see cref="Update"/> will
	/// be processed by <see cref="Notify"/> and should never be called from
	/// outside the instance.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: Observable.cs,v 1.1.1.1 2003/02/01 19:03:25 jens Exp $") ] 
	[ ComVisible(false) ]
	[ ComSourceInterfaces(typeof(COM.IObserveableEvents)) ]
	public abstract class Observable : IObservable 
	{
		/// <summary>
		/// Constructor.
		/// </summary>
		protected Observable() {}

		/// <summary>
		/// The Changed event signals a change in this object. You can
		/// add your own <see cref="EventHandler"/> delegates
		/// to be notified of changes.
		/// </summary>
		public event EventHandler Changed;

		/// <summary>
		/// Use this in derived classes to signal a change in your 
		/// object (==the subject) to your observers.
		/// </summary>
		/// <remarks>
		/// This calls the virtual <c>OnChanged(EventArgs.Empty)</c>
		/// internally, which can be overridden by derived classes.
		/// </remarks>
		protected void OnChanged() 
		{
			OnChanged(EventArgs.Empty);
		}
		
		/// <summary>
		/// This method fires the <see cref="Changed"/> event with
		/// the given arguments. 
		/// </summary>
		/// <remarks>
		/// Always use this function or the other
		/// <see cref="OnChanged"/> overload to fire events. Never
		/// invoke the <see cref="Changed"/> event directly.
		/// </remarks>
		/// <param name="e">Event arguments.</param>
		protected virtual void OnChanged(EventArgs e) 
		{
			if( Changed != null )
				Changed(this, e);
		}

		/// <summary>
		/// Default event listener for observable Observers. 
		/// Notify should never be called directly from your class.
		/// </summary>
		/// <remarks>
		/// Although this is NOT necessary for all Observeables, most
		/// of them observe other objects as well (being in an Observable
		/// chain).
		/// Overwrite this if you need to analyze or filter changes in 
		/// your observeables or need to set a dirty flag for cached data. 
		/// </remarks>
		/// <param name="sender">The object that fired the event.</param>
		/// <param name="e">
		///		The <see cref="System.EventArgs"/> containing more
		///		specific event information.
		/// </param>
		protected virtual void Notify(object sender, EventArgs e) 
		{ 
			// observed object changed
			// assume we have changed and forward event to our listeners
			OnChanged(e);
		}

		/// <summary>
		/// Send yourself an unspecific update notification.
		/// </summary>
		/// <remarks>
		/// This calls the virtual <c>Notify(this, EventArgs.Empty)</c>
		/// internally, which can be overridden by derived classes.
		/// </remarks>
		protected void Update()
		{
			this.Notify(this, EventArgs.Empty);
		}

		/// <summary>
		/// Register with multiple <see cref="IObservable"/> objects.
		/// </summary>
		/// <remarks>
		/// <see cref="Notify"/> is added as a new <see cref="EventHandler"/>
		/// to the <see cref="IObservable.Changed"/> event of each passed
		/// object.
		/// </remarks>
		/// <param name="observables">
		/// All objects in the passed Enumeration must implement
		/// the <see cref="IObservable"/> interface.
		/// </param>
		protected virtual void RegisterWith(IEnumerable observables)
		{
			if( observables == null) return;
			foreach (IObservable observable in observables)
			{
				if ( observable != null )
					observable.Changed += new EventHandler(Notify);
			}
		}

		/// <summary>
		/// Register with an observable object.
		/// </summary>
		/// <remarks>
		/// <see cref="Notify"/> is added as a new <see cref="EventHandler"/>
		/// to the <see cref="IObservable.Changed"/> event.
		/// </remarks>
		/// <param name="observable">
		/// The <see cref="IObservable"/> to register with.
		/// </param>
		protected virtual void RegisterWith(IObservable observable)
		{
			// TODO: we may need to keep a list of all observables
			//		 and unregister
			if ( observable != null )
				observable.Changed += new EventHandler(Notify);
		}

		/// <summary>
		/// Unregister from (= stop listening to events from) an obvserved object.
		/// </summary>
		/// <param name="observable">The <see cref="IObservable"/> to unregister.</param>
		protected virtual void UnregisterWith(IObservable observable)
		{
			if ( observable != null )
				observable.Changed -= new EventHandler(Notify);
		}
	}
}
